@@ -8,15 +8,18 @@
 #include "RemoveInvalid.h"
 #include "Layout.h"
 #include "Params.h"
-#include "../Exception.h"
+#include "Code/Binary.h"
+#include "Code/FnState.h"
+#include "Code/Exception.h"
 #include "Code/PosixEh/StackInfo.h"
+#include "Gc/DwarfTable.h"
 
 namespace code {
 	namespace arm64 {
 
 		Arena::Arena() {}
 
-		Listing *Arena::transform(Listing *l) const {
+		Arena::TransformInfo Arena::transformInfo(Listing *l) const {
 #if defined(POSIX) && defined(ARM64)
 			code::eh::activatePosixInfo();
 #endif
@@ -25,9 +28,10 @@ namespace code {
 			l = code::transform(l, this, new (this) RemoveInvalid());
 
 			// Expand variables and function calls as well as function prolog and epilog.
-			l = code::transform(l, this, new (this) Layout());
+			Layout *layout = new (this) Layout();
+			l = code::transform(l, this, layout);
 
-			return l;
+			return TransformInfo(l, layout->layout);
 		}
 
 		void Arena::output(Listing *src, Output *to) const {
@@ -48,6 +52,17 @@ namespace code {
 				from->remove(fnDirtyRegs[i]);
 		}
 
+		RegSet *Arena::fnResultRegs() const {
+			RegSet *result = new (this) RegSet();
+			result->put(xr(0));
+			result->put(xr(1));
+			result->put(dr(0));
+			result->put(dr(1));
+			result->put(dr(2));
+			result->put(dr(3));
+			return result;
+		}
+
 		Listing *Arena::redirect(Bool member, TypeDesc *result, Array<TypeDesc *> *params, Ref fn, Operand param) {
 			Listing *l = new (this) Listing(this);
 
@@ -171,5 +186,72 @@ namespace code {
 			return ptrr(17); // We can also use x16. x17 is nice as we use that elsewhere.
 		}
 
+		Arena::Skeleton *Arena::compatibleFrameSkeleton(Binary *binary, Nat offset) {
+			Arena::Skeleton *result = frameSkeletonHead(binary);
+			Array<Operand> *preservedRegs = result->savedRegs;
+			Array<Operand> *preservedLocs = result->savedLocs;
+
+			// Figure out which registers were spilled in the prolog:
+			{
+				FDE *desc = dwarfTable().find(binary->address());
+				if (desc)
+					code::dwarf::findPreservedRegs(preservedRegs, preservedLocs, desc,
+												&fromDwarfRegister, dataAlignment);
+
+				// Remove x30 from 'preservedRegs' - we don't bother with the link register.
+				for (Nat i = 0; i < preservedRegs->count(); i++) {
+					if (same(preservedRegs->at(i).reg(), ptrr(30))) {
+						preservedRegs->remove(i);
+						preservedLocs->remove(i);
+						break;
+					}
+				}
+			}
+
+			Nat wordsBelow = 2; // For return addr + old frame pointer.
+			Nat wordsAbove = preservedRegs->count();
+			{
+				code::Params *layout = layoutParams(binary->result(), binary->params());
+				for (Nat i = 0; i < layout->registerCount(); i++)
+					if (layout->registerParam(i).any())
+						wordsAbove++;
+			}
+
+			result->accessMode = Int(wordsBelow) * Offset::sPtr.current();
+
+			// Find current block and active piece:
+			Nat active = findFunctionState(binary->address(), offset);
+			decodeFnState(active, result->currentBlock, result->currentActivation);
+
+			frameSkeletonTailBelow(binary, result, wordsBelow, wordsAbove, Size::sPtr.current(), true);
+
+			return result;
+		}
+
+		void Arena::resizeStackFrame(Listing *out, Reg tmpReg, Binary *newSz) {
+			tmpReg = asSize(tmpReg, Size::sPtr);
+
+			// Load return address and old fp into registers.
+			*out << mov(ptrr(29), ptrRel(ptrStack));
+			*out << mov(ptrr(30), ptrRel(ptrStack, Offset::sPtr));
+
+			// Adjust the stack pointer. First, figure out the size of the stack.
+			*out << mov(tmpReg, ptrRel(out->meta()));
+			*out << band(tmpReg, ptrConst(~Nat(0x1))); // Note: This is large enough. Stacks are typically < 4GiB.
+
+			// Compute how to adjust the stack pointer to fit the new size.
+			*out << sub(tmpReg, ptrConst(Nat(newSz->stackSize())));
+
+			// Update the stack pointer.
+			*out << add(ptrStack, tmpReg);
+
+			// Store back return address and old fp.
+			*out << mov(ptrRel(ptrStack), ptrr(29));
+			*out << mov(ptrRel(ptrStack, Offset::sPtr), ptrr(30));
+
+			// Update frame pointer.
+			*out << mov(ptrFrame, ptrStack);
+		}
+
 	}
 }
