@@ -132,7 +132,9 @@ namespace code {
 			*dest << dest->meta();
 
 			// Offset between sp and fp. On ARM64, it is always zero.
-			*dest << dat(ptrConst(0));
+			// As such, store the size of the stack, but mark it by setting the LSB.
+			Nat size = Nat(layout->last().current());
+			*dest << dat(ptrConst(size | 0x1));
 
 			// Output metadata table.
 			Array<Var> *vars = src->allVars();
@@ -175,9 +177,9 @@ namespace code {
 				assert(tmpReg != noReg);
 				tmpReg = asSize(tmpReg, Size::sPtr);
 				*dest << mov(tmpReg, ptrRel(ptrFrame, layout->at(v.key())));
-				return xRel(size, tmpReg, op.offset());
+				return xRel(size, tmpReg, op.offsetRef());
 			} else {
-				return xRel(size, ptrFrame, layout->at(v.key()) + op.offset());
+				return xRel(size, ptrFrame, op.offsetRef() + layout->at(v.key()));
 			}
 		}
 
@@ -209,7 +211,7 @@ namespace code {
 					zeroVar(dest, layout->at(v.key()), v.size());
 			}
 
-			if (usingEH) {
+			if (true || usingEH) { // We need info for active updates.
 				padCallWithNop(dest);
 
 				// Remember where the block started.
@@ -290,7 +292,7 @@ namespace code {
 			}
 
 			currentBlock = dest->parent(currentBlock);
-			if (usingEH && table) {
+			if ((true || usingEH) && table) { // Need for active updates
 				padCallWithNop(dest);
 
 				Label lbl = dest->label();
@@ -305,23 +307,24 @@ namespace code {
 			*dest << instrSrc(engine(), op::prolog, ptrConst(stackSize));
 
 			// Preserve registers.
-			Nat offset = 16; // After sp and fp.
+			Offset offset = stackSize;
 
 			// Store x8 if we need it.
 			if (params->result().memoryRegister() != noReg) {
+				offset -= Offset::sPtr;
 				*dest << mov(resultLocation(), ptrr(8));
-				offset += 8;
+				*dest << preserve(longRel(ptrFrame, Offset(offset) - stackSize), xr(8));
 			}
 
 			// Preserve remaining registers.
 			for (RegSet::Iter begin = preserved->begin(), end = preserved->end(); begin != end; ++begin) {
+				offset -= Offset::sPtr;
 				Operand memory = longRel(ptrFrame, Offset(offset));
 				Reg reg = asSize(begin.v(), Size::sLong);
 				*dest << mov(memory, reg);
 				// Note: offsets are relative to the CFA, which is the location of the stack pointer
 				// at the start of the function:
 				*dest << preserve(longRel(ptrFrame, Offset(offset) - stackSize), reg);
-				offset += 8;
 			}
 
 			// Preserve parameters.
@@ -349,12 +352,12 @@ namespace code {
 			currentBlock = oldBlock;
 
 			// Restore spilled registers.
-			Nat offset = 16;
+			Offset offset = layout->last();
 			if (params->result().memoryRegister() != noReg)
-				offset += 8; // Adjust for preserving x8, but we don't need to restore it.
+				offset -= Offset::sPtr; // Adjust for preserving x8, but we don't need to restore it.
 			for (RegSet::Iter begin = preserved->begin(), end = preserved->end(); begin != end; ++begin) {
+				offset -= Offset::sPtr;
 				*dest << mov(asSize(begin.v(), Size::sLong), longRel(ptrFrame, Offset(offset)));
-				offset += 8;
 			}
 
 			// Emit the epilog, and related metadata.
@@ -640,6 +643,7 @@ namespace code {
 
 		Array<Offset> *layout(Listing *src, Params *params, Nat spilled) {
 			Array<Offset> *result = code::layout(src, Size::sPtr); // Specify minimum alignment.
+			Array<Var> *allVars = src->allVars();
 			Array<Var> *paramVars = src->allParams();
 
 			// A stack frame on Arm64 is as follows. Offsets are relative sp and x29 (frame
@@ -651,40 +655,48 @@ namespace code {
 			// ...
 			// param1
 			// param0    <- old sp
-			// localN
-			// ...
-			// local0
-			// spilled-paramN
+			// spilled-reg0
 			// ...
+			// spilled-regN
 			// spilled-param0
-			// spilledN
 			// ...
-			// spilled0
+			// spilled-paramN
+			// local0
+			// ...
+			// localN
+			// (padding)
 			// old-x30
-			// old-x29   <- sp, x29
+			// old-x29   <- sp, x29 (ptrFrame)
 
-			// Account for parameters that are in registers and need to be spilled to the stack.
-			// TODO: It would be nice to avoid this, but we need more advanced register allocation
-			// for that to work properly.
-			Nat spilledParams = 0;
+			// Note: The stack frame is "flipped" compared to how it was previously. This is to
+			// mirror the layout on x86, which makes active function replacement much easier, and
+			// somewhat consistent between platforms.
+
+			// 0: We need to keep the stack aligned to 16 bytes. It is nice to insert the padding at
+			// the end of the stack frame. So, compute the number of spilled parameters here and fix
+			// the align now already.
+			Nat totalSpilled = spilled + 2;
 			for (Nat i = 0; i < params->registerCount(); i++)
 				if (params->registerParam(i) != Param())
-					spilledParams++;
+					totalSpilled++;
 
-			// TODO: We might need to align spilled registers to a 16-byte boundary to make stp and
-			// ldp work properly.
+			if ((result->last() + Offset::sPtr * totalSpilled).v64() & 0xF)
+				result->last() += Offset::sPtr;
 
-			// Account for the "stack frame" (x30 and x29):
-			spilled += 2;
+			// 1: Flip the order of all variables and make space for the old sp and fp.
+			result->last() += Offset::sPtr * 2;
+			Offset totalSize = result->last();
+			for (Nat i = 0; i < allVars->count(); i++) {
+				Var v = allVars->at(i);
+				if (src->isParam(v))
+					continue;
 
-			// Adjust "result" to account for spilled registers.
-			Nat adjust = (spilled + spilledParams) * 8;
-			for (Nat i = 0; i < result->count(); i++) {
-				result->at(i) += Offset(adjust);
+				Offset varSize = Offset(v.size()).alignAs(Size::sPtr);
+				result->at(i) = totalSize - result->at(v.key()) - varSize;
+				// PLN(L"Var " << i << L": " << result->at(i) << L", sz: " << varSize);
 			}
 
-			// Fill in the parameters that need to be spilled to the stack.
-			Nat spillCount = spilled;
+			// 2: Fill in parameters that need to be spilled on the stack:
 			for (Nat i = 0; i < params->registerCount(); i++) {
 				Param p = params->registerParam(i);
 				// Some params are "padding".
@@ -693,16 +705,15 @@ namespace code {
 
 				// If multiple parts of the same parameter, then don't update the variable again.
 				if (p.offset() == 0)
-					result->at(paramVars->at(p.id()).key()) = Offset(spillCount * 8);
+					result->at(paramVars->at(p.id()).key()) = result->last();
 
-				spillCount += 1;
+				result->last() += Offset::sPtr;
 			}
 
-			// Ensure that the size of the stack is rounded up to a multiple of 16 bytes.
-			if (result->last().v64() & 0xF)
-				result->last() += Offset::sPtr;
+			// 3: Add space for spilled registers.
+			result->last() += Offset::sPtr * spilled;
 
-			// Now we can fill in the offset of parameters passed on the stack.
+			// 4: Finally, fill in the offset of parameters passed on the stack.
 			for (Nat i = 0; i < params->stackCount(); i++) {
 				Offset off(params->stackOffset(i));
 				off += result->last(); // Increase w. size of entire stack frame.
