@@ -1,6 +1,7 @@
 #include "stdafx.h"
 #include "Output.h"
 #include "Asm.h"
+#include "AsmOut.h"
 #include "Gc/DwarfTable.h"
 #include "Code/Binary.h"
 #include "Code/Dwarf/Stream.h"
@@ -10,11 +11,12 @@ namespace code {
 	namespace arm64 {
 
 		// Code are always 32-bit long, we use that fact.
-		static const Nat codeAlignment = 4;
+		const Nat codeAlignment = 4;
 		// We assume data alignment of -8.
-		static const Int dataAlignment = -8;
+		const Int dataAlignment = -8;
+
 		// Convert to DWARF registers.
-		static Nat dwarfRegister(Reg reg) {
+		Nat dwarfRegister(Reg reg) {
 			Nat r = intRegNumber(reg);
 			// SP is 31
 			if (r == 32)
@@ -22,6 +24,14 @@ namespace code {
 			return r;
 		}
 
+		// Convert from DWARF register.
+		Reg fromDwarfRegister(Nat reg) {
+			// We only need to switch SP.
+			if (reg == 31)
+				reg = 32;
+			return ptrr(reg);
+		}
+
 		void initCIE(CIE *cie) {
 			// Return value is in x30
 			Nat pos = code::dwarf::initStormCIE(cie, codeAlignment, dataAlignment, 30);
@@ -32,13 +42,18 @@ namespace code {
 			out.putUOp(DW_CFA_def_cfa, 31, 0);
 		}
 
+#ifndef DEBUG
+#undef assert
+#define assert(...)
+#endif
+
 		CodeOut::CodeOut(Binary *owner, Array<Nat> *lbls, Nat size, Nat numRefs) {
 			// Properly align 'size'.
 			this->size = size = roundUp(size, Nat(sizeof(void *)));
 
 			// Initialize our members.
 			this->owner = owner;
-			codeRefs = new (this) Array<Reference *>();
+			codeRefs = new (this) Array<TObject *>();
 			code = (byte *)runtime::allocCode(engine(), size, numRefs + 3);
 			labels = lbls;
 			pos = 0;
@@ -179,6 +194,10 @@ namespace code {
 			codeRefs->push(new (this) CodeUpdater(r, owner, code, ref - 1));
 		}
 
+		void CodeOut::markRef(OffsetRef r, Nat data) {
+			codeRefs->push(new (this) ArmOffsetUpdater(r, owner, code, pos - sizeof(Nat), data));
+		}
+
 		Nat CodeOut::labelOffset(Nat id) {
 			if (id < labels->count()) {
 				return labels->at(id);
@@ -192,6 +211,5 @@ namespace code {
 			return offset - (pos + 4); // NOTE: All relative things on the X86-64 are 4 bytes long, not 8!
 		}
 
-
 	}
 }
