@@ -30,6 +30,7 @@ namespace code {
 		protected:
 			virtual void STORM_FN markLabel(Nat id);
 			virtual void STORM_FN markGcRef(Ref ref);
+			virtual void STORM_FN markRef(OffsetRef ref, Nat data);
 			virtual Nat STORM_FN labelOffset(Nat id);
 			virtual Nat STORM_FN toRelative(Nat id);
 
@@ -38,7 +39,7 @@ namespace code {
 			Binary *owner;
 
 			// Updaters in the code.
-			Array<Reference *> *codeRefs;
+			Array<TObject *> *codeRefs;
 
 			// Code we're writing to.
 			UNKNOWN(PTR_GC) byte *code;
@@ -53,5 +54,15 @@ namespace code {
 			Array<Nat> *labels;
 		};
 
+		// Access to code and data alginment.
+		extern const Nat codeAlignment;
+		extern const Int dataAlignment;
+
+		// Convert to a DWARF register.
+		Nat dwarfRegister(Reg reg);
+
+		// Convert from DWARF register.
+		Reg fromDwarfRegister(Nat reg);
+
 	}
 }
