@@ -278,22 +278,27 @@ namespace code {
 		}
 
 		Operand RemoveInvalid::limitImm(Listing *to, const Operand &op, Nat line, Nat immBits, Bool immSigned) {
-			if (op.type() != opConstant)
+			if (op.type() == opRegister) {
 				return op;
+			} else if (op.type() == opConstant) {
+				Bool ok = false;
+				if (immSigned) {
+					Long v = op.constant();
+					Long maxVal = Long(1) << (immBits - 1);
+					ok = (v >= -maxVal) & (v < maxVal);
+				} else {
+					Word maxVal = Word(1) << immBits;
+					ok = op.constant() < maxVal;
+				}
 
-			Bool ok = false;
-			if (immSigned) {
-				Long v = op.constant();
-				Long maxVal = Long(1) << (immBits - 1);
-				ok = (v >= -maxVal) & (v < maxVal);
-			} else {
-				Word maxVal = Word(1) << immBits;
-				ok = op.constant() < maxVal;
+				if (ok)
+					return op;
+			} else if (op.type() == opOffReference) {
+				// Accept truncation for 12 bits or less.
+				if (immBits >= 12)
+					return op;
 			}
 
-			if (ok)
-				return op;
-
 			Reg r = unusedReg(used->at(line), op.size());
 			used->at(line)->put(r);
 			loadRegister(to, r, op);
@@ -309,35 +314,48 @@ namespace code {
 		}
 
 		Operand RemoveInvalid::regOrImm(Listing *to, const Operand &op, Nat line, Nat immBits, Bool immSigned) {
-			if (op.type() == opRegister)
+			switch (op.type()) {
+			case opRegister:
 				return op;
 
-			if (op.type() == opConstant)
+			case opConstant:
+			case opOffReference:
 				return limitImm(to, op, line, immBits, immSigned);
 
 			// TODO: Maybe exclude other types here as well?
 
-			// Otherwise, try to load it into a register.
-			Reg r = unusedReg(used->at(line), op.size());
-			used->at(line)->put(r);
-			loadRegister(to, r, op);
-			return Operand(r);
+			default:
+				// Otherwise, try to load it into a register.
+				Reg r = unusedReg(used->at(line), op.size());
+				used->at(line)->put(r);
+				loadRegister(to, r, op);
+				return Operand(r);
+			}
 		}
 
 		void RemoveInvalid::loadRegister(Listing *to, Reg reg, const Operand &load) {
 			switch (load.type()) {
-			case opConstant:
-				if (load.constant() <= 0xFFFF && isIntReg(reg)) // No load literal for fp registers.
+			case opConstant: {
+				Word constant = load.constant();
+				// Note: We can use MOVN for negative numbers.
+				Bool in16bits = (constant <= 0xFFFF) || (~constant <= 0xFFFF);
+				if (in16bits && isIntReg(reg)) // No load literal for fp registers.
 					*to << mov(reg, load);
 				else
 					*to << mov(reg, largeConstant(load));
 				break;
-
+			}
 			case opRegister:
 				if (!same(reg, load.reg()))
 					*to << mov(reg, load);
 				break;
 
+			case opOffReference:
+				// We assume that the offset is small enough. Note, we can use MOVN to store
+				// negative numbers.
+				*to << mov(reg, load);
+				break;
+
 				// TODO: Maybe others?
 			default:
 				*to << mov(reg, load);
@@ -471,6 +489,57 @@ namespace code {
 			*to << mov(dest, tmpReg);
 		}
 
+		void RemoveInvalid::shadowMovTfm(Listing *to, Instr *instr, Nat line) {
+			Operand dest = instr->dest();
+			// If dest is a register, we can just load that register.
+			if (dest.type() == opRegister) {
+				// Note: This is a copy of 'RemoveInvalid::loadRegister', but that emits 'shadowMov'
+				// instead of a plain 'mov'.
+				Operand load = instr->src();
+				Reg reg = dest.reg();
+				switch (load.type()) {
+				case opConstant: {
+					Word constant = load.constant();
+					// Note: We can use MOVN for negative numbers.
+					Bool in16bits = (constant <= 0xFFFF) || (~constant <= 0xFFFF);
+					if (in16bits && isIntReg(reg)) // No load literal for fp registers.
+						*to << mov(reg, load);
+					else
+						*to << mov(reg, largeConstant(load));
+					break;
+				}
+				case opRegister:
+					if (!same(reg, load.reg()))
+						*to << mov(reg, load);
+					break;
+
+				case opOffReference:
+					// We assume that the offset is small enough. Note, we can use MOVN to store
+					// negative numbers.
+					*to << mov(reg, load);
+					break;
+
+					// TODO: Maybe others?
+				default:
+					*to << mov(reg, load);
+					break;
+				}
+				return;
+			}
+
+			// If the source is a register, we can just emit a single store operation regardless.
+			Operand src = instr->src();
+			if (src.type() == opRegister) {
+				*to << instr;
+				return;
+			}
+
+			// Otherwise, we need to break the instruction into two pieces.
+			Reg tmpReg = unusedReg(used->at(line), src.size());
+			loadRegister(to, tmpReg, src);
+			*to << shadowMov(dest, tmpReg);
+		}
+
 		void RemoveInvalid::leaTfm(Listing *to, Instr *instr, Nat line) {
 			Operand dest = instr->dest();
 			if (dest.type() == opRegister) {
@@ -568,15 +637,21 @@ namespace code {
 		void RemoveInvalid::ucastTfm(Listing *to, Instr *instr, Nat line) {
 			// For icast and ucast we allow source operands in memory since there are special
 			// instructions that load from memory. Offsets are the same size as for regular loads
-			// and stores, so we don't handle them in a special way.
+			// and stores, so we don't handle them in a special way. We do, however, need to be
+			// careful if the offset is a reference. We don't handle them in codegen (even though we
+			// could do), so those are treated as if they are not real relative offsets.
 
 			Operand src = instr->src();
 			switch (src.type()) {
 			case opRegister:
-			case opRelative:
-			case opVariable:
 				// Supported!
 				break;
+			case opRelative:
+			case opVariable:
+				// Is the offset a OffsetRef? If not, the operation is supported already.
+				if (!src.offsetRef().source())
+					break;
+				// Fall thru
 			default:
 				// Add a separate move operation to load the source: we don't support loading
 				// arbitrary things. Note: We don't need to update 'used', it is fine if we use the
