@@ -16,9 +16,8 @@ namespace code {
 	public:
 		STORM_CTOR Content();
 
-		// Set the address of this content. Don't do this after the content has been added to a
-		// RefSource.
-		void set(const void *address, nat size);
+		// Set the address of this content.
+		void set(const void *address, Nat size);
 
 		// Get last address and size.
 		virtual const void *address() const;
@@ -26,15 +25,23 @@ namespace code {
 
 		// Get the name of the owning RefSource (if any).
 		MAYBE(Str *) STORM_FN ownerName() const;
+
+		// Get the owner object, if any.
+		MAYBE(RefSource *) STORM_FN owner() const;
+
+		// Get the actual RefSource to use, in case the content of one RefSource was stolen by
+		// another. Returns null most of the time.
+		virtual MAYBE(RefSource *) stolenBy() const;
+
 	private:
 		friend class RefSource;
 
 		// Last address and size.
 		UNKNOWN(PTR_GC) const void *lastAddress;
-		nat lastSize;
+		Nat lastSize;
 
 		// Owning RefSource.
-		RefSource *owner;
+		MAYBE(RefSource *) cOwner;
 	};
 
 	/**
