@@ -28,13 +28,13 @@ namespace code {
 		virtual void STORM_FN deepCopy(CloneEnv *env);
 
 		// Get the op-code.
-		op::OpCode STORM_FN op() const;
+		op::OpCode STORM_FN op() const { return iOp; }
 
 		// Get the source operand.
-		Operand STORM_FN src() const;
+		inline Operand STORM_FN src() const { return iSrc; }
 
 		// Get the destination operand.
-		Operand STORM_FN dest() const;
+		inline Operand STORM_FN dest() const { return iDest; }
 
 		// Get the mode of the destination operand. This determines how the instruction uses the
 		// destination (e.g. read/write, read, or write).
@@ -229,6 +229,10 @@ namespace code {
 	Instr *STORM_FN prolog(EnginePtr e);
 	Instr *STORM_FN epilog(EnginePtr e);
 
+	// Shadow mov operation. Does not impact detection of dirty registers, so can be used to
+	// save/restore registers without marking them as 'dirty'.
+	Instr *STORM_FN shadowMov(EnginePtr e, Operand dest, Operand src);
+
 	// Note that a register has been preserved on the stack. Intended to be used internally by backends.
 	Instr *STORM_FN preserve(EnginePtr e, Operand dest, Reg reg);
 
