@@ -39,8 +39,10 @@ namespace code {
 
 	BITMASK_OPERATORS(FreeOpt);
 
+	// Note: Both of these are intended for convenience from C++. They differ slightly from the
+	// standard output from a FreeOpt. They are mainly used for outputting a Listing, so this is fine.
 	wostream &operator <<(wostream &to, FreeOpt o);
-	StrBuf &STORM_FN operator <<(StrBuf &to, FreeOpt o);
+	StrBuf &operator <<(StrBuf &to, FreeOpt o);
 
 
 	/**
