@@ -318,24 +318,6 @@ namespace code {
 			return *this;
 	}
 
-	static Str *safeToS(RootObject *obj) {
-		if (TObject *tObj = as<TObject>(obj)) {
-			Thread *thread = tObj->associatedThread();
-			const os::Thread &osThread = thread->thread();
-			if (osThread != os::Thread::current()) {
-				os::Future<Str *> result;
-				os::FnCall<Str *, 1> call = os::fnCall().add(tObj);
-				Str *(CODECALL TObject::*ptr)() const = &TObject::toS;
-				os::UThread::spawn(address(ptr), true, call, result, &osThread);
-				return result.result();
-			} else {
-				return obj->toS();
-			}
-		} else {
-			return obj->toS();
-		}
-	}
-
 	wostream &operator <<(wostream &to, const Operand &o) {
 		if (o.type() != opRegister
 			&& o.type() != opCondFlag
@@ -394,7 +376,7 @@ namespace code {
 			return to << L"@" << o.ref().title();
 		case opObjReference:
 			if (o.object()) {
-				return to << L"&'" << safeToS(o.object())->c_str() << L"'";
+				return to << L"&'" << threadSafeToS(o.object())->c_str() << L"'";
 			} else {
 				return to << L"&null";
 			}
@@ -477,7 +459,7 @@ namespace code {
 			return;
 		case opObjReference:
 			if (object()) {
-				*to << S("&'") << safeToS(object()) << S("'");
+				*to << S("&'") << threadSafeToS(object()) << S("'");
 			} else {
 				*to << S("&null");
 			}
