@@ -6,6 +6,7 @@
 #include "Block.h"
 #include "Label.h"
 #include "Reference.h"
+#include "OffsetReference.h"
 #include "Core/SrcPos.h"
 
 namespace code {
@@ -43,13 +44,13 @@ namespace code {
 		// Register
 		STORM_NAME(opRegister, register),
 
-		// Relative to a register, ie. [reg + offset]
+		// Relative to a register. ie. [reg + offset/offset-ref]
 		STORM_NAME(opRelative, relative),
 
 		// Relative to a label. TODO: Implement for X86!
 		STORM_NAME(opRelativeLbl, relativeLabel),
 
-		// Variable (+ offset).
+		// Variable (+ offset/offset-ref).
 		STORM_NAME(opVariable, variable),
 
 		// Label.
@@ -64,6 +65,9 @@ namespace code {
 		// Reference to an object.
 		STORM_NAME(opObjReference, objReference),
 
+		// Reference to an offset.
+		STORM_NAME(opOffReference, offsetReference),
+
 		// Condition flag.
 		STORM_NAME(opCondFlag, condFlag),
 
@@ -150,6 +154,9 @@ namespace code {
 		// `variable`, and `relativeLabel`.
 		Offset STORM_FN offset() const;
 
+		// Does this operand have a reference that affects `offset`?
+		Bool STORM_FN hasOffsetRef() const;
+
 		// Get the condition flag. Valid if `type` returns `condFlag`.
 		CondFlag STORM_FN condFlag() const;
 
@@ -162,6 +169,9 @@ namespace code {
 		// Get the reference. Valid if `type` returns `reference`.
 		Ref STORM_FN ref() const;
 
+		// Get an offset reference.
+		OffsetRef STORM_FN offsetRef() const;
+
 		// Convenient internal access.
 		RefSource *refSource() const;
 
@@ -202,11 +212,18 @@ namespace code {
 		Operand(Offset o, Size size);
 		Operand(RootObject *obj);
 
-		// [Register,variable,label] + offset.
+		// Offset reference with specified size.
+		Operand(OffsetRef ref, Size size);
+
+		// [register,variable,label] + offset.
 		Operand(Reg r, Offset offset, Size size);
 		Operand(Var v, Offset offset, Size size);
 		Operand(Label l, Offset offset, Size size);
 
+		// [register,variable] + offset-ref
+		Operand(Reg r, OffsetRef ref, Size size);
+		Operand(Var r, OffsetRef ref, Size size);
+
 		// Our type. Note: may contain other flags as well!
 		OpType opType;
 
@@ -228,14 +245,18 @@ namespace code {
 
 		// Friends.
 		friend Operand intConst(Offset v);
+		friend Operand intConst(OffsetRef v);
 		friend Operand natConst(Size v);
 		friend Operand ptrConst(Size v);
 		friend Operand ptrConst(Offset v);
+		friend Operand ptrConst(OffsetRef v);
 		friend Operand xConst(Size s, Word w);
 		friend Operand objPtr(Object *ptr);
 		friend Operand objPtr(TObject *ptr);
 		friend Operand xRel(Size size, Reg reg, Offset offset);
+		friend Operand xRel(Size size, Reg reg, OffsetRef ref);
 		friend Operand xRel(Size size, Var v, Offset offset);
+		friend Operand xRel(Size size, Var v, OffsetRef ref);
 		friend Operand xRel(Size size, Label l, Offset offset);
 		friend wostream &operator <<(wostream &to, const Operand &o);
 	};
@@ -248,6 +269,7 @@ namespace code {
 	Operand STORM_FN byteConst(Byte v);
 	Operand STORM_FN intConst(Int v);
 	Operand STORM_FN intConst(Offset v);
+	Operand STORM_FN intConst(OffsetRef v);
 	Operand STORM_FN natConst(Nat v);
 	Operand STORM_FN natConst(Size v);
 	Operand STORM_FN longConst(Long v);
@@ -256,6 +278,7 @@ namespace code {
 	Operand STORM_FN doubleConst(Double v);
 	Operand STORM_FN ptrConst(Size v);
 	Operand STORM_FN ptrConst(Offset v);
+	Operand STORM_FN ptrConst(OffsetRef v);
 	Operand STORM_FN ptrConst(Nat v);
 	Operand STORM_FN xConst(Size s, Word v);
 
@@ -264,53 +287,69 @@ namespace code {
 	Operand STORM_FN objPtr(TObject *ptr);
 
 	// Relative values (dereference pointers).
-	Operand STORM_FN byteRel(Reg reg, Offset offset);
-	Operand STORM_FN intRel(Reg reg, Offset offset);
-	Operand STORM_FN longRel(Reg reg, Offset offset);
-	Operand STORM_FN floatRel(Reg reg, Offset offset);
-	Operand STORM_FN doubleRel(Reg reg, Offset offset);
-	Operand STORM_FN ptrRel(Reg reg, Offset offset);
-	Operand STORM_FN xRel(Size size, Reg reg, Offset offset);
+	inline Operand STORM_FN xRel(Size size, Reg reg, Offset offset) { return Operand(reg, offset, size); }
+	inline Operand STORM_FN byteRel(Reg reg, Offset offset) { return xRel(Size::sByte, reg, offset); }
+	inline Operand STORM_FN intRel(Reg reg, Offset offset) { return xRel(Size::sInt, reg, offset); }
+	inline Operand STORM_FN longRel(Reg reg, Offset offset) { return xRel(Size::sLong, reg, offset); }
+	inline Operand STORM_FN floatRel(Reg reg, Offset offset) { return xRel(Size::sFloat, reg, offset); }
+	inline Operand STORM_FN doubleRel(Reg reg, Offset offset) { return xRel(Size::sDouble, reg, offset); }
+	inline Operand STORM_FN ptrRel(Reg reg, Offset offset) { return xRel(Size::sPtr, reg, offset); }
 
-	Operand STORM_FN byteRel(Reg reg);
-	Operand STORM_FN intRel(Reg reg);
-	Operand STORM_FN longRel(Reg reg);
-	Operand STORM_FN floatRel(Reg reg);
-	Operand STORM_FN doubleRel(Reg reg);
-	Operand STORM_FN ptrRel(Reg reg);
-	Operand STORM_FN xRel(Size size, Reg reg);
+	inline Operand STORM_FN xRel(Size size, Reg reg) { return xRel(size, reg, Offset()); }
+	inline Operand STORM_FN byteRel(Reg reg) { return xRel(Size::sByte, reg, Offset()); }
+	inline Operand STORM_FN intRel(Reg reg) { return xRel(Size::sInt, reg, Offset()); }
+	inline Operand STORM_FN longRel(Reg reg) { return xRel(Size::sLong, reg, Offset()); }
+	inline Operand STORM_FN floatRel(Reg reg) { return xRel(Size::sFloat, reg, Offset()); }
+	inline Operand STORM_FN doubleRel(Reg reg) { return xRel(Size::sDouble, reg, Offset()); }
+	inline Operand STORM_FN ptrRel(Reg reg) { return xRel(Size::sPtr, reg, Offset()); }
+
+	inline Operand STORM_FN xRel(Size size, Reg reg, OffsetRef ref) { return Operand(reg, ref, size); }
+	inline Operand STORM_FN byteRel(Reg reg, OffsetRef ref) { return xRel(Size::sByte, reg, ref); }
+	inline Operand STORM_FN intRel(Reg reg, OffsetRef ref) { return xRel(Size::sInt, reg, ref); }
+	inline Operand STORM_FN longRel(Reg reg, OffsetRef ref) { return xRel(Size::sLong, reg, ref); }
+	inline Operand STORM_FN floatRel(Reg reg, OffsetRef ref) { return xRel(Size::sFloat, reg, ref); }
+	inline Operand STORM_FN doubleRel(Reg reg, OffsetRef ref) { return xRel(Size::sDouble, reg, ref); }
+	inline Operand STORM_FN ptrRel(Reg reg, OffsetRef ref) { return xRel(Size::sPtr, reg, ref); }
 
 	// Access offsets inside variables.
-	Operand STORM_FN byteRel(Var v, Offset offset);
-	Operand STORM_FN intRel(Var v, Offset offset);
-	Operand STORM_FN longRel(Var v, Offset offset);
-	Operand STORM_FN floatRel(Var v, Offset offset);
-	Operand STORM_FN doubleRel(Var v, Offset offset);
-	Operand STORM_FN ptrRel(Var v, Offset offset);
-	Operand STORM_FN xRel(Size size, Var v, Offset offset);
+	inline Operand STORM_FN xRel(Size size, Var v, Offset offset) { return Operand(v, offset, size); }
+	inline Operand STORM_FN byteRel(Var v, Offset offset) { return xRel(Size::sByte, v, offset); }
+	inline Operand STORM_FN intRel(Var v, Offset offset) { return xRel(Size::sInt, v, offset); }
+	inline Operand STORM_FN longRel(Var v, Offset offset) { return xRel(Size::sLong, v, offset); }
+	inline Operand STORM_FN floatRel(Var v, Offset offset) { return xRel(Size::sFloat, v, offset); }
+	inline Operand STORM_FN doubleRel(Var v, Offset offset) { return xRel(Size::sDouble, v, offset); }
+	inline Operand STORM_FN ptrRel(Var v, Offset offset) { return xRel(Size::sPtr, v, offset); }
 
-	Operand STORM_FN byteRel(Var v);
-	Operand STORM_FN intRel(Var v);
-	Operand STORM_FN longRel(Var v);
-	Operand STORM_FN floatRel(Var v);
-	Operand STORM_FN doubleRel(Var v);
-	Operand STORM_FN ptrRel(Var v);
-	Operand STORM_FN xRel(Size size, Var v);
+	inline Operand STORM_FN xRel(Size size, Var v) { return xRel(size, v, Offset()); }
+	inline Operand STORM_FN byteRel(Var v) { return xRel(Size::sByte, v, Offset()); }
+	inline Operand STORM_FN intRel(Var v) { return xRel(Size::sInt, v, Offset()); }
+	inline Operand STORM_FN longRel(Var v) { return xRel(Size::sLong, v, Offset()); }
+	inline Operand STORM_FN floatRel(Var v) { return xRel(Size::sFloat, v, Offset()); }
+	inline Operand STORM_FN doubleRel(Var v) { return xRel(Size::sDouble, v, Offset()); }
+	inline Operand STORM_FN ptrRel(Var v) { return xRel(Size::sPtr, v, Offset()); }
+
+	inline Operand STORM_FN xRel(Size size, Var v, OffsetRef ref) { return Operand(v, ref, size); }
+	inline Operand STORM_FN byteRel(Var v, OffsetRef ref) { return xRel(Size::sByte, v, ref); }
+	inline Operand STORM_FN intRel(Var v, OffsetRef ref) { return xRel(Size::sInt, v, ref); }
+	inline Operand STORM_FN longRel(Var v, OffsetRef ref) { return xRel(Size::sLong, v, ref); }
+	inline Operand STORM_FN floatRel(Var v, OffsetRef ref) { return xRel(Size::sFloat, v, ref); }
+	inline Operand STORM_FN doubleRel(Var v, OffsetRef ref) { return xRel(Size::sDouble, v, ref); }
+	inline Operand STORM_FN ptrRel(Var v, OffsetRef ref) { return xRel(Size::sPtr, v, ref); }
 
 	// Access offsets inside labels.
-	Operand STORM_FN byteRel(Label l, Offset offset);
-	Operand STORM_FN intRel(Label l, Offset offset);
-	Operand STORM_FN longRel(Label l, Offset offset);
-	Operand STORM_FN floatRel(Label l, Offset offset);
-	Operand STORM_FN doubleRel(Label l, Offset offset);
-	Operand STORM_FN ptrRel(Label l, Offset offset);
-	Operand STORM_FN xRel(Size size, Label l, Offset offset);
+	inline Operand STORM_FN xRel(Size size, Label l, Offset offset) { return Operand(l, offset, size); }
+	inline Operand STORM_FN byteRel(Label l, Offset offset) { return xRel(Size::sByte, l, offset); }
+	inline Operand STORM_FN intRel(Label l, Offset offset) { return xRel(Size::sInt, l, offset); }
+	inline Operand STORM_FN longRel(Label l, Offset offset) { return xRel(Size::sLong, l, offset); }
+	inline Operand STORM_FN floatRel(Label l, Offset offset) { return xRel(Size::sFloat, l, offset); }
+	inline Operand STORM_FN doubleRel(Label l, Offset offset) { return xRel(Size::sDouble, l, offset); }
+	inline Operand STORM_FN ptrRel(Label l, Offset offset) { return xRel(Size::sPtr, l, offset); }
 
-	Operand STORM_FN byteRel(Label l);
-	Operand STORM_FN intRel(Label l);
-	Operand STORM_FN longRel(Label l);
-	Operand STORM_FN floatRel(Label l);
-	Operand STORM_FN doubleRel(Label l);
-	Operand STORM_FN ptrRel(Label l);
-	Operand STORM_FN xRel(Size size, Label l);
+	inline Operand STORM_FN xRel(Size size, Label l) { return xRel(size, l, Offset()); }
+	inline Operand STORM_FN byteRel(Label l) { return xRel(Size::sByte, l, Offset()); }
+	inline Operand STORM_FN intRel(Label l) { return xRel(Size::sInt, l, Offset()); }
+	inline Operand STORM_FN longRel(Label l) { return xRel(Size::sLong, l, Offset()); }
+	inline Operand STORM_FN floatRel(Label l) { return xRel(Size::sFloat, l, Offset()); }
+	inline Operand STORM_FN doubleRel(Label l) { return xRel(Size::sDouble, l, Offset()); }
+	inline Operand STORM_FN ptrRel(Label l) { return xRel(Size::sPtr, l, Offset()); }
 }
