@@ -23,45 +23,6 @@
 namespace code {
 	namespace eh {
 
-		/**
-		 * Description of the data at the end of each function.
-		 */
-		struct FnData {
-			// Number of entries in the block table.
-			size_t blockCount;
-		};
-
-		/**
-		 * Description of a single entry in the block table.
-		 */
-		struct FnBlock {
-			// At which offset do we start?
-			Nat offset;
-
-			// Which block?
-			Nat block;
-		};
-
-		// Get the FnData from a function.
-		static const FnData *getData(const void *fn) {
-			size_t size = runtime::codeSize(fn);
-			const byte *end = (const byte *)fn + size;
-			return (const FnData *)(end - sizeof(FnData));
-		}
-
-		// Get the block data from a function.
-		static const FnBlock *getBlocks(const FnData *data) {
-			const FnBlock *end = (const FnBlock *)data;
-			return end - data->blockCount;
-		}
-
-		// Compare object.
-		struct BlockCompare {
-			bool operator()(Nat offset, const FnBlock &block) const {
-				return offset < block.offset;
-			}
-		};
-
 		// Our representation of a stack frame.
 		class StackFrame : public code::StackFrame {
 		public:
@@ -75,25 +36,6 @@ namespace code {
 			byte *rbp;
 		};
 
-		// Find the currently active block and active ID.
-		static Nat stormFindBlockActive(const FnData *data, size_t fn, size_t pc) {
-			const FnBlock *blocks = getBlocks(data);
-
-			Nat offset = pc - fn;
-			Nat invalid = Block().key();
-
-			// The entries are sorted by their 'offset' member. We can perform a binary search!
-			// Note: if there are multiple elements that are equal, we need to pick the last one.
-			const FnBlock *found = std::upper_bound(blocks, blocks + data->blockCount, offset, BlockCompare());
-			if (found == blocks) {
-				// Before any block, nothing to do!
-				return invalid;
-			}
-			found--;
-
-			return found->block;
-		}
-
 		// Storage from phase 1 to phase 2. Binary compatible with blocks of __cxa_exception in GCC.
 		// Names from the GCC implementation for simplicity.
 
@@ -230,7 +172,6 @@ namespace code {
 			size_t pc = _Unwind_GetIP(context);
 			size_t fp = _Unwind_GetGR(context, DWARF_EH_FRAME_POINTER);
 
-			const FnData *fnData = getData((const void *)fn);
 			Binary *binary = codeBinary((const void *)fn);
 
 			if (actions & _UA_SEARCH_PHASE) {
@@ -246,7 +187,7 @@ namespace code {
 					return _URC_CONTINUE_UNWIND;
 
 				// Find if it is desirable to actually catch it.
-				Nat encoded = stormFindBlockActive(fnData, fn, pc);
+				Nat encoded = findFunctionState((const void *)fn, pc - fn);
 				Nat block, active;
 				decodeFnState(encoded, block, active);
 
@@ -303,7 +244,7 @@ namespace code {
 
 				// Clean up what we can!
 				Nat block, active;
-				Nat encoded = stormFindBlockActive(fnData, fn, pc);
+				Nat encoded = findFunctionState((const void *)fn, pc - fn);
 				decodeFnState(encoded, block, active);
 				if (block != Block().key()) {
 					StackFrame frame(block, active, (void *)fp);
