@@ -351,7 +351,7 @@ namespace code {
 					preservedLocs->push(ptrRel(ptrFrame, Offset::sPtr * -Int(i + 1)));
 				}
 
-				TODO(L"Double check order of preserved regs above.");
+				// TODO(L"Double check order of preserved regs above.");
 
 				// Find the current block and active piece:
 				Nat active = findFunctionStateFromEnd(ehStart, offset);
