@@ -107,6 +107,7 @@ namespace code {
 			case opRelative:
 			case opReference:
 			case opObjReference:
+			case opOffReference:
 				modRm(to, opCode(0xFF), rmNone, 6, src);
 				break;
 			default:
@@ -180,6 +181,10 @@ namespace code {
 			}
 		}
 
+		void shadowMovOut(Output *to, Instr *instr) {
+			movOut(to, instr);
+		}
+
 		void swapOut(Output *to, Instr *instr) {
 			if (instr->size() == Size::sByte) {
 				modRm(to, opCode(0x86), rmByte, instr->dest(), instr->src());
@@ -391,6 +396,9 @@ namespace code {
 				// Special meaning, load the RefSource instead.
 				// Issues a 'mov' operation to simply load the address of the refsource.
 				modRm(to, opCode(0x8B), rmWide, regId, objPtr(src.refSource()));
+			} else if (src.type() == opOffReference) {
+				// Same as above.
+				modRm(to, opCode(0x8B), rmWide, regId, objPtr(src.offsetRef().source()));
 			} else {
 				modRm(to, opCode(0x8D), rmWide, regId, src);
 			}
@@ -417,6 +425,10 @@ namespace code {
 					to->putInt(Nat(src.constant()));
 				}
 				break;
+			case opOffReference:
+				modRm(to, opCode(0x69), wide(src), registerId(dest), instr->dest());
+				to->putInt(src.offsetRef());
+				break;
 			case opLabel:
 			case opReference:
 			case opObjReference:
@@ -480,6 +492,8 @@ namespace code {
 				assert(false, L"Calling or jumping to constant values are not supported. Use labels or references!");
 				break;
 			case opLabel:
+				if (!call)
+					to->markJump(src.label());
 				to->putByte(call ? 0xE8 : 0xE9);
 				to->putRelative(src.label());
 				break;
@@ -731,11 +745,19 @@ namespace code {
 			case opObjReference:
 				to->putObject(src.object());
 				break;
+			case opOffReference:
+				if (src.size() == Size::sInt)
+					to->putInt(src.offsetRef());
+				else if (src.size() == Size::sPtr)
+					to->putPtr(src.offsetRef());
+				else
+					throw new (to) InternalError(S("Unable to output offsets as anything but pointers or ints."));
+				break;
 			case opConstant:
 				to->putSize(src.constant(), src.size());
 				break;
 			default:
-				assert(false, L"Unsupported type for 'dat'.");
+				throw new (to) InternalError(S("Unsupported type for 'dat'."));
 				break;
 			}
 		}
@@ -757,6 +779,7 @@ namespace code {
 			OUTPUT(push),
 			OUTPUT(pop),
 			OUTPUT(mov),
+			OUTPUT(shadowMov),
 			OUTPUT(swap),
 			OUTPUT(add),
 			OUTPUT(adc),
