@@ -222,6 +222,7 @@ namespace code {
 				return;
 			}
 
+			// We need to perform a memcpy-like operation (only for variables).
 			if (p.src.type() != opVariable)
 				throw new (dest) InvalidValue(S("Can not pass non-variables larger than 8 bytes to functions."));
 
@@ -415,7 +416,7 @@ namespace code {
 				}
 			} else if (src.type() == opVariable) {
 				Size s(param.size());
-				*env.dest << mov(asSize(target, s), xRel(s, src.var(), Offset(param.offset())));
+				*env.dest << mov(asSize(target, s), xRel(s, src.var(), src.offsetRef() + Offset(param.offset())));
 			} else {
 				throw new (env.dest) InvalidValue(S("Can not pass non-variables larger than 8 bytes to functions."));
 			}
