@@ -83,6 +83,7 @@ namespace code {
 			*dest << dest->meta();
 
 			// Offset between RBP and RSP.
+			// Note: always aligned, so LSB is clear.
 			*dest << dat(ptrConst(-layout->last()));
 
 			// Output metadata table.
@@ -131,7 +132,7 @@ namespace code {
 			Var v = op.var();
 			if (!src->accessible(v, block))
 				throw new (this) VariableUseError(v, block);
-			return xRel(size, ptrFrame, layout->at(v.key()) + op.offset());
+			return xRel(size, ptrFrame, op.offsetRef() + layout->at(v.key()));
 		}
 
 		static bool spillAlways(const Param &, const Offset &) {
@@ -342,7 +343,7 @@ namespace code {
 				}
 			}
 
-			if (usingEH) {
+			if (true || usingEH) { // We need info for active updates.
 				padCallWithNop(dest);
 
 				// Remember where the block started.
@@ -405,7 +406,7 @@ namespace code {
 			}
 
 			block = dest->parent(block);
-			if (usingEH && table) {
+			if ((true || usingEH) && table) { // Need this for active updates.
 				padCallWithNop(dest);
 
 				Label lbl = dest->label();
