@@ -323,9 +323,7 @@ namespace storm {
 			MemberVar *var = new (owner) MemberVar(name->pos, name->v, t, owner);
 
 			if (init) {
-				var->initializer = new (owner) VariableInitializer(name->pos, t, owner->scope, init);
-				var->initializer->make(fnStatic);
-				var->initializer->parentLookup = var;
+				var->initializer(new (owner) VariableInitializer(name->pos, t, owner->scope, init));
 			}
 
 			return var;
@@ -339,12 +337,14 @@ namespace storm {
 							Array<NameParam> *params,
 							syntax::Node *body) {
 
-			return new (owner) BSFunction(owner->scope.value(result, pos),
-										name,
-										resolve(params, owner, owner->scope),
-										owner->scope,
-										null, // thread
-										body);
+			BSFunction *fn = new (owner) BSFunction(owner->scope.value(result, pos),
+													name,
+													resolve(params, owner, owner->scope),
+													owner->scope,
+													null, // thread
+													body);
+			fn->pos = pos;
+			return fn;
 		}
 
 		Function *classAbstractFn(Class *owner,
@@ -357,6 +357,7 @@ namespace storm {
 			BSRawFn *f = new (owner) BSAbstractFn(owner->scope.value(result, pos),
 												name,
 												resolve(params, owner, owner->scope));
+			f->pos = pos;
 
 			syntax::transformNode<void, Class *, BSRawFn *>(options, owner, f);
 
@@ -378,6 +379,7 @@ namespace storm {
 												owner->scope,
 												null, // thread
 												body);
+			f->pos = pos;
 			f->make(fnAssign);
 			return f;
 		}
