@@ -201,7 +201,7 @@ namespace storm {
 			else if (t == StormInfo<Word>::type(e))
 				*s->l << mov(to, wordConst(Word(intValue)));
 			else
-				assert(false, L"Unknown type for an integer constant.");
+				throw new (this) InternalError(S("Unknown type for an integer constant."));
 
 			r->created(s);
 		}
@@ -217,7 +217,7 @@ namespace storm {
 			else if (t == StormInfo<Double>::type(e))
 				*s->l << mov(to, doubleConst(floatValue));
 			else
-				assert(false, L"Unknown type for a float constant.");
+				throw new (this) InternalError(S("Unknown type for a float constant."));
 
 			r->created(s);
 		}
