@@ -52,7 +52,7 @@ namespace storm {
 			virtual Breakable::To STORM_FN breakTo();
 			virtual Breakable::To STORM_FN continueTo();
 
-		protected:
+			// Output.
 			virtual void STORM_FN toS(StrBuf *to) const;
 
 		private:
