@@ -246,5 +246,24 @@ namespace storm {
 				return new (e.v) LocalVar(param.name, param.type(), pos, true);
 		}
 
+
+		/**
+		 * RefVar.
+		 */
+
+		RefVar::RefVar(SrcPos pos, Block *block, Str *name, Expr *init) : Expr(pos), init(init) {
+			var = new (this) LocalVar(name, init->result().type(), pos, false);
+			block->add(var);
+		}
+
+		void RefVar::code(CodeGen *state, CodeResult *to) {
+			CodeResult *res = new (this) CodeResult(var->result, var->var);
+			init->code(state, res);
+		}
+
+		void RefVar::toS(StrBuf *to) const {
+			*to << var->name << S(" = ") << init->toS();
+		}
+
 	}
 }
