@@ -8,7 +8,9 @@ namespace storm {
 
 		VariableInitializer::VariableInitializer(SrcPos pos, Value type, Scope scope, MAYBE(syntax::Node *) initExpr)
 			: BSRawFn(type, new (engine()) syntax::SStr(S("init")), new (engine()) Array<ValParam>(), null),
-			  pos(pos), scope(scope), initExpr(initExpr) {
+			  scope(scope), initExpr(initExpr) {
+
+			this->pos = pos;
 
 			makeInline();
 		}
