@@ -195,6 +195,10 @@ namespace storm {
 	// Allocate and fill a `FnCall` object, copying arguments depending on `copy`.
 	code::Var STORM_FN createFnCall(CodeGen *to, Array<Value> *formals, Array<code::Operand> *actuals, Bool copy);
 
+	// Allocate and fill a `FnCall` object. Using 'env' to create and store a CloneEnv. The variable
+	// is assumed to be empty from the start.
+	code::Var STORM_FN createFnCall(CodeGen *to, Array<Value> *formals, Array<code::Operand> *actuals, code::Var env);
+
 	// Allocate an object on the heap. Store it in variable `to`.
 	void STORM_FN allocObject(CodeGen *s, Function *ctor, Array<code::Operand> *params, code::Var to);
 	code::Var STORM_FN allocObject(CodeGen *s, Function *ctor, Array<code::Operand> *params);
