@@ -57,8 +57,8 @@ namespace storm {
 #endif
 	}
 
-	Bool CompilerProtocol::operator ==(const Protocol &o) const {
-		const CompilerProtocol *other = as<const CompilerProtocol>(&o);
+	Bool CompilerProtocol::isEqualTo(const Protocol *o) const {
+		const CompilerProtocol *other = as<const CompilerProtocol>(o);
 		if (!other)
 			return false;
 
