@@ -27,8 +27,9 @@ namespace storm {
 		// Output.
 		virtual void STORM_FN toS(StrBuf *to) const;
 
+	protected:
 		// The same protocol as some other?
-		virtual Bool STORM_FN operator ==(const Protocol &o) const;
+		virtual Bool STORM_FN isEqualTo(const Protocol *o) const;
 
 	private:
 		// Library name (if any).
