@@ -13,9 +13,6 @@
 #include "Utils/Lock.h"
 #include "Utils/StackInfo.h"
 
-// TODO: Do not depend on path!
-#include "Utils/Path.h"
-
 namespace storm {
 
 	class Thread;
@@ -57,13 +54,19 @@ namespace storm {
 			reuseMain,
 		};
 
+		// Callback for creating an Url for the root.
+		typedef Url *(*CreateRootUrl)(Engine &e, void *param);
+
 		// Create the engine.
 		// 'root' is the location of the root package directory on disk. The package 'core' is
 		// assumed to be found as a subdirectory of the given root path.
 		// 'stackBase' is the base of the current thread's stack. Eg. the address of argc and/or
 		// argv or some other variable allocated on the stack near 'main'.
-		// TODO: Do not depend on Util/Path!
-		Engine(const Path &root, ThreadMode mode, void *stackBase);
+		Engine(const wchar *root, ThreadMode mode, void *stackBase);
+
+		// Create the engine. This overload accepts a function that will be called whenever the
+		// Engine needs to know the Url of the root of the name tree.
+		Engine(CreateRootUrl createRoot, void *callbackParam, ThreadMode mode, void *stackBase);
 
 		// Destroy. This will wait until all threads have terminated properly.
 		~Engine();
@@ -321,6 +324,9 @@ namespace storm {
 		StormInfo stackInfo;
 		int stackId;
 
+		// Create the engine.
+		void create(CreateRootUrl createRoot, void *rootParam, ThreadMode mode, void *stackBase);
+
 		// Cleanup code.
 		void destroy();
 	};
