@@ -149,19 +149,33 @@ namespace storm {
 		// Sort with operator <.
 		if (param().type->handle().lessFn) {
 			addSort();
+			addBinarySearch();
 		} else {
 			watchFor |= watchLess;
 		}
 
+		// Remove duplicates.
+		if (param().type->handle().lessFn || param().type->handle().equalFn) {
+			addRemoveDuplicates();
+		} else {
+			watchFor |= watchEquality;
+		}
 
-		// Sort with provided predicate function.
+		// Sort and remove duplicates with provided predicate function.
 		{
 			Array<Value> *pParams = new (e) Array<Value>();
 			*pParams << Value(StormInfo<Bool>::type(e));
 			*pParams << param() << param();
 			Value predicate = Value(fnType(pParams));
+
 			add(nativeFunction(e, Value(), S("sort"), valList(e, 2, t, predicate), address(&ArrayBase::sortRawPred)));
 			add(nativeFunction(e, t, S("sorted"), valList(e, 2, t, predicate), address(&sortedRawPred))->makePure());
+
+			add(nativeFunction(e, Value(), S("removeDuplicates"), valList(e, 2, t, predicate), address(&ArrayBase::removeDuplicatesRawPred)));
+			add(nativeFunction(e, t, S("withoutDuplicates"), valList(e, 2, t, predicate), address(&ArrayBase::withoutDuplicatesRawPred))->makePure());
+
+			add(new (e) TemplateFn(new (e) Str(S("lowerBound")), fnPtr(e, &ArrayType::createLowerBound, this)));
+			add(new (e) TemplateFn(new (e) Str(S("upperBound")), fnPtr(e, &ArrayType::createUpperBound, this)));
 		}
 
 		if (!param().type->isA(StormInfo<TObject>::type(engine))) {
@@ -219,6 +233,19 @@ namespace storm {
 
 			watchFor &= ~watchLess;
 			addSort();
+			addBinarySearch();
+			if (watchFor & watchEquality) {
+				watchFor &= ~watchEquality;
+				addRemoveDuplicates();
+			}
+		} else if (*fn->name == S("==") &&
+				fn->result == Value(StormInfo<Bool>::type(engine)) &&
+				fn->params->count() == 2 &&
+				fn->params->at(0).type == to &&
+				fn->params->at(1).type == to) {
+
+			watchFor &= ~watchEquality;
+			addRemoveDuplicates();
 		} else if (SerializeInfo *info = serializeInfo(param().type)) {
 			if (watchFor & watchSerialization) {
 				watchFor &= ~watchSerialization;
@@ -239,7 +266,101 @@ namespace storm {
 		add(nativeFunction(e, Value(), S("sort"), params, address(&ArrayBase::sortRaw)));
 		add(nativeFunction(e, Value(this), S("sorted"), params, address(&sortedRaw))->makePure());
 
-		// TODO: Would be nice to have < for comparison as well!
+		// < for comparing.
+		Value b(StormInfo<Bool>::type(e));
+		add(nativeFunction(e, b, S("<"), new (e) Array<Value>(2, thisPtr(this)), address(&ArrayBase::lessRaw)));
+	}
+
+	void ArrayType::addRemoveDuplicates() {
+		Engine &e = engine;
+		Array<Value> *params = valList(e, 1, thisPtr(this));
+
+		// Compare with == or <, whichever is present.
+		add(nativeFunction(e, Value(), S("removeDuplicates"), params, address(&ArrayBase::removeDuplicatesRaw)));
+		add(nativeFunction(e, Value(this), S("withoutDuplicates"), params, address(&ArrayBase::withoutDuplicatesRaw))->makePure());
+
+		// == for comparing.
+		Value b(StormInfo<Bool>::type(e));
+		add(nativeFunction(e, b, S("=="), new (e) Array<Value>(2, thisPtr(this)), address(&ArrayBase::equalRaw)));
+	}
+
+	void ArrayType::addBinarySearch() {
+		Engine &e = engine;
+		Value n(StormInfo<Nat>::type(e));
+		Array<Value> *params = valList(e, 2, thisPtr(this), param().asRef());
+
+		add(nativeFunction(e, n, S("lowerBound"), params, address(&ArrayBase::lowerBoundRaw)));
+		add(nativeFunction(e, n, S("upperBound"), params, address(&ArrayBase::upperBoundRaw)));
+	}
+
+	MAYBE(Named *) ArrayType::createLowerBound(Str *name, SimplePart *part) {
+		if (part->params->count() != 3)
+			return null;
+
+		Value findType = part->params->at(1).asRef(false);
+		if (!findType.type)
+			return null;
+
+		// Note: To help automatic deduction of template parameters, we ignore parameter #2 and
+		// produce a suitable match based on parameter 1 (parameter 2 is often deduced to 'void' for
+		// anonymous lambdas). To make this work, we need to re-use existing entities whenever
+		// possible.
+		NameOverloads *candidates = allOverloads(new (this) Str(S("lowerBound")));
+		for (Nat i = 0; i < candidates->count(); i++) {
+			Named *candidate = candidates->at(i);
+			if (candidate->params->count() == 3)
+				if (candidate->params->at(1).type == findType.type)
+					return candidate;
+		}
+
+		// Otherwise, create a new entity:
+		Value n(StormInfo<Nat>::type(engine));
+		Array<Value> *predParams = new (engine) Array<Value>(3, Value(StormInfo<Bool>::type(engine)));
+		predParams->at(1) = param();
+		predParams->at(2) = findType;
+
+		Array<Value> *params = new (engine) Array<Value>(3, thisPtr(this));
+		params->at(1) = findType.asRef();
+		params->at(2) = Value(fnType(predParams));
+
+		Function *created = nativeFunction(engine, n, S("lowerBound"), params, address(&ArrayBase::lowerBoundRawPred));
+		created->flags |= namedMatchNoInheritance;
+		return created;
+	}
+
+	MAYBE(Named *) ArrayType::createUpperBound(Str *name, SimplePart *part) {
+		if (part->params->count() != 3)
+			return null;
+
+		Value findType = part->params->at(1).asRef(false);
+		if (!findType.type)
+			return null;
+
+		// Note: To help automatic deduction of template parameters, we ignore parameter #2 and
+		// produce a suitable match based on parameter 1 (parameter 2 is often deduced to 'void' for
+		// anonymous lambdas). To make this work, we need to re-use existing entities whenever
+		// possible.
+		NameOverloads *candidates = allOverloads(new (this) Str(S("upperBound")));
+		for (Nat i = 0; i < candidates->count(); i++) {
+			Named *candidate = candidates->at(i);
+			if (candidate->params->count() == 3)
+				if (candidate->params->at(1).type == findType.type)
+					return candidate;
+		}
+
+		// Otherwise, create a new entity:
+		Value n(StormInfo<Nat>::type(engine));
+		Array<Value> *predParams = new (engine) Array<Value>(3, Value(StormInfo<Bool>::type(engine)));
+		predParams->at(1) = findType;
+		predParams->at(2) = param();
+
+		Array<Value> *params = new (engine) Array<Value>(3, thisPtr(this));
+		params->at(1) = findType.asRef();
+		params->at(2) = Value(fnType(predParams));
+
+		Function *created = nativeFunction(engine, n, S("upperBound"), params, address(&ArrayBase::upperBoundRawPred));
+		created->flags |= namedMatchNoInheritance;
+		return created;
 	}
 
 	void ArrayType::addSerialization(SerializeInfo *info) {
@@ -461,6 +582,11 @@ namespace storm {
 		new (Place(to)) ArrayBase::Iter(*from);
 	}
 
+	static void *CODECALL assignIterator(ArrayBase::Iter *to, const ArrayBase::Iter *from) {
+		*to = *from;
+		return to;
+	}
+
 	static bool CODECALL iteratorEq(ArrayBase::Iter &a, ArrayBase::Iter &b) {
 		return a == b;
 	}
@@ -496,6 +622,7 @@ namespace storm {
 		Value vBool = Value(StormInfo<Bool>::type(e));
 		Value vNat = Value(StormInfo<Nat>::type(e));
 		add(nativeFunction(e, Value(), Type::CTOR, refref, address(&copyIterator))->makePure());
+		add(nativeFunction(e, r, S("="), refref, address(&assignIterator))->makePure());
 		add(nativeFunction(e, vBool, S("=="), refref, address(&iteratorEq))->makePure());
 		add(nativeFunction(e, vBool, S("!="), refref, address(&iteratorNeq))->makePure());
 		add(nativeFunction(e, r, S("++*"), ref, address(&ArrayBase::Iter::preIncRaw)));
