@@ -29,6 +29,9 @@ namespace storm {
 		// Parameter.
 		Value STORM_FN param() const;
 
+		// Contained type.
+		Type *STORM_FN containedType() const { return contents; }
+
 		// Notifications.
 		virtual void STORM_FN notifyAdded(NameSet *to, Named *added);
 
