@@ -1,6 +1,7 @@
 #include "stdafx.h"
 #include "Bool.h"
 #include "Function.h"
+#include "Number.h"
 #include "Core/Io/Serialization.h"
 
 namespace storm {
@@ -124,6 +125,9 @@ namespace storm {
 		Array<Value> *rs = new (this) Array<Value>(2, Value(this, true));
 		rs->at(1) = StormInfo<StrBuf>::type(engine);
 		add(nativeFunction(engine, Value(), S("toS"), rs, address(&boolToS)));
+		add(inlinedFunction(engine, Value(StormInfo<Byte>::type(engine)), S("byte"), v, fnPtr(engine, &ucast))->makePure());
+		add(inlinedFunction(engine, Value(StormInfo<Nat>::type(engine)), S("nat"), v, fnPtr(engine, &ucast))->makePure());
+		add(inlinedFunction(engine, Value(StormInfo<Word>::type(engine)), S("word"), v, fnPtr(engine, &ucast))->makePure());
 
 		Array<Value> *is = new (this) Array<Value>(1, Value(StormInfo<IStream>::type(engine)));
 		add(nativeFunction(engine, Value(this), S("read"), is, address(&boolRead)));
