@@ -82,7 +82,7 @@ namespace storm {
 
 	Type *CODECALL rawPtrType(RootObject *ptr) {
 		if (ptr)
-			return runtime::typeOf(ptr);
+			return runtime::gcTypeOf(ptr)->type;
 		else
 			return null;
 	}
