@@ -5,6 +5,7 @@
 #include "Core/StrBuf.h"
 #include "Core/Str.h"
 #include "Exception.h"
+#include "Syntax/SStr.h"
 
 namespace storm {
 
@@ -29,12 +30,7 @@ namespace storm {
 			add(name->at(i));
 	}
 
-	Name::Name(const Name &o) : parts(new (engine()) Array<NamePart *>(*o.parts)) {}
-
-	void Name::deepCopy(CloneEnv *env) {
-		Object::deepCopy(env);
-		cloned(parts, env);
-	}
+	Name::Name(const Name &o) : ObjectOn<Compiler>(o), parts(new (engine()) Array<NamePart *>(*o.parts)) {}
 
 	void Name::add(NamePart *v) {
 		parts->push(v);
@@ -44,6 +40,10 @@ namespace storm {
 		parts->push(new (this) SimplePart(name));
 	}
 
+	void Name::add(syntax::SStr *name) {
+		parts->push(new (this) SimplePart(name->v));
+	}
+
 	void Name::add(Str *name, Array<Value> *params) {
 		parts->push(new (this) SimplePart(name, params));
 	}
@@ -98,11 +98,6 @@ namespace storm {
 
 	SrcName::SrcName(SimpleName *o, SrcPos pos) : Name(o), pos(pos) {}
 
-	void SrcName::deepCopy(CloneEnv *env) {
-		Name::deepCopy(env);
-		cloned(pos, env);
-	}
-
 	/**
 	 * SimpleName.
 	 */
@@ -119,12 +114,8 @@ namespace storm {
 		add(name, p);
 	}
 
-	SimpleName::SimpleName(const SimpleName &o) : parts(new (engine()) Array<SimplePart *>(*o.parts)) {}
-
-	void SimpleName::deepCopy(CloneEnv *env) {
-		Object::deepCopy(env);
-		cloned(parts, env);
-	}
+	SimpleName::SimpleName(const SimpleName &o)
+		: ObjectOn<Compiler>(o), parts(new (engine()) Array<SimplePart *>(*o.parts)) {}
 
 	void SimpleName::add(SimplePart *v) {
 		parts->push(v);
@@ -174,29 +165,20 @@ namespace storm {
 		}
 	}
 
-	Bool SimpleName::operator ==(const SimpleName &o) const {
-		if (!sameType(this, &o))
+	Bool SimpleName::sameAs(const SimpleName *o) const {
+		if (!sameType(this, o))
 			return false;
 
-		if (count() != o.count())
+		if (count() != o->count())
 			return false;
 
 		for (Nat i = 0; i < count(); i++)
-			if (*at(i) != *o.at(i))
+			if (!at(i)->sameAs(o->at(i)))
 				return false;
 
 		return true;
 	}
 
-	Nat SimpleName::hash() const {
-		// inspired from djb2
-		Nat r = 5381;
-		for (nat i = 0; i < parts->count(); i++) {
-			r = ((r << 5) + r) + parts->at(i)->hash();
-		}
-		return r;
-	}
-
 
 	/**
 	 * Parsing.
