@@ -5,13 +5,17 @@
 namespace storm {
 	STORM_PKG(core.lang);
 
+	namespace syntax {
+		class SStr;
+	}
+
 	class Scope;
 	class SimpleName;
 
 	/**
 	 * Representation of a name, either a relative name or an absolute name.
 	 */
-	class Name : public Object {
+	class Name : public ObjectOn<Compiler> {
 		STORM_CLASS;
 	public:
 		// Path to the root package.
@@ -29,12 +33,10 @@ namespace storm {
 		// Copy.
 		Name(const Name &o);
 
-		// Deep copy.
-		virtual void STORM_FN deepCopy(CloneEnv *env);
-
 		// Append a new entry.
 		void STORM_FN add(NamePart *v);
 		void STORM_FN add(Str *name);
+		void STORM_FN add(syntax::SStr *name);
 		void STORM_FN add(Str *name, Array<Value> *params);
 		void STORM_FN add(Str *name, Array<Name> *params);
 
@@ -83,15 +85,13 @@ namespace storm {
 		STORM_CTOR SrcName(SimpleName *o, SrcPos pos);
 
 		SrcPos pos;
-
-		virtual void STORM_FN deepCopy(CloneEnv *env);
 	};
 
 	/**
 	 * A name which only contains resolved parts. Created by Scopes when starting to resolve a
 	 * regular name.
 	 */
-	class SimpleName : public Object {
+	class SimpleName : public ObjectOn<Compiler> {
 		STORM_CLASS;
 	public:
 		// Path to the root package.
@@ -103,10 +103,7 @@ namespace storm {
 		STORM_CTOR SimpleName(Str *name, Array<Value> *params);
 
 		// Copy.
-		SimpleName(const SimpleName &simple);
-
-		// Deep copy.
-		virtual void STORM_FN deepCopy(CloneEnv *env);
+		STORM_CTOR SimpleName(const SimpleName &simple);
 
 		// Append a new entry.
 		void STORM_FN add(SimplePart *v);
@@ -143,11 +140,8 @@ namespace storm {
 		// ToS.
 		virtual void STORM_FN toS(StrBuf *to) const;
 
-		// Equals.
-		virtual Bool STORM_FN operator ==(const SimpleName &o) const;
-
-		// Hash.
-		virtual Nat STORM_FN hash() const;
+		// See if the name is the same as another name.
+		Bool STORM_FN sameAs(const SimpleName *other) const;
 
 	private:
 		// Data.
