@@ -6,6 +6,8 @@
 #include "Visibility.h"
 #include "Scope.h"
 #include "Doc.h"
+#include "ReplaceContext.h"
+#include "ReplaceTasks.h"
 
 namespace storm {
 	STORM_PKG(core.lang);
@@ -47,9 +49,26 @@ namespace storm {
 		// Parent name lookup. This should be set by the parent. If it is `null`, the default
 		// `parent` implementation asserts. Therefore, root objects need to override `parent` in
 		// order to return `null`.
-		MAYBE(NameLookup *) parentLookup;
+		MAYBE(NameLookup *) STORM_FN parentLookup() const { return pLookup; }
+		void STORM_ASSIGN parentLookup(MAYBE(NameLookup *) parent);
+
+	protected:
+		// Called when `parentLookup` was changed. The parameter indicates whether or not we have
+		// been added to a `NameSet` before or not.
+		virtual void STORM_FN onNewParent(MAYBE(NameLookup *) parent, Bool addedToNameSet);
+
+		// Set the parent of `other` to `this`. This is done with the intention of making `other`
+		// into a child that is kept track of and traversed as appropriately.
+		void STORM_FN makeChild(NameLookup *other);
+
+	private:
+		// The actual parent lookup.
+		MAYBE(NameLookup *) pLookup;
 	};
 
+	// Find the first instance of a specific type of thing.
+	MAYBE(NameLookup *) STORM_FN findTopmost(NameLookup *at, Type *type) ON(Compiler);
+
 
 	/**
 	 * An extension of NameLookup with a position.
@@ -63,6 +82,7 @@ namespace storm {
 		// Create.
 		STORM_CTOR LookupPos();
 		STORM_CTOR LookupPos(SrcPos pos);
+		STORM_CTOR LookupPos(MAYBE(NameLookup *) parent, SrcPos pos);
 
 		// Our location in source code.
 		SrcPos pos;
@@ -144,16 +164,69 @@ namespace storm {
 		// automatically unless instructed otherwise.
 		virtual void STORM_FN discardSource();
 
+		// Called when looking for equivalent types during a replacement. Should call 'addEq' for
+		// all equivalent types in the supplied context.
+		virtual void STORM_FN findEquivalentTypes(Named *old, ReplaceContext *ctx);
+
+		// See if this named entity is able to replace 'old'. If possible, null is returned. If not,
+		// a string containing an appropriate error message is returned. If this function does not
+		// return an error, it shall be safe to call 'replace' without errors. May also throw an
+		// exception to inhibit the system to try other approaches that would be tried if this
+		// function were to return an error message.
+		virtual MAYBE(Str *) STORM_FN canReplace(Named *old, ReplaceContext *ctx);
+
+		// Make this entity replace 'old'. If 'canReplace' did not return an error, this shall
+		// succeed without issues. Override 'doReplace' to customize the behavior of this function.
+		// Do not expect 'old' to be in a functional state after having been replaced.
+		void STORM_FN replace(Named *old, ReplaceTasks *tasks, ReplaceContext *ctx);
+
+		// Automatic replace: called by the system when entities marked with `namedAllowReplace` are
+		// replaced by the system. This does not happen during an operation requested by the system,
+		// which is why it is handled differently from regular calls to `replace`. Currently, this
+		// is more or less only used when the system silently inserts custom overrides of functions
+		// from TObject, when they are replaced with something user-defined. As such, we don't have
+		// to make sure that changes are immediate in this case (as is the case with the more
+		// advanced replacement mechanisms).
+		virtual void STORM_FN automaticReplace(Named *old);
+
 		// String representation.
 		virtual void STORM_FN toS(StrBuf *buf) const;
 
 		// Output the visibility to a string buffer.
 		void STORM_FN putVisibility(StrBuf *to) const;
 
+	protected:
+		// Called by 'replace' after verifying that 'canReplace' did not return an error.
+		virtual void STORM_FN doReplace(Named *old, ReplaceTasks *tasks, ReplaceContext *ctx);
+
+		// Keep track of when our parent lookup changes, so that we can discard our source as
+		// appropriate.
+		virtual void STORM_FN onNewParent(MAYBE(NameLookup *) parent, Bool addedToNameSet);
+
 	private:
 		// Find closest named parent.
 		MAYBE(Named *) closestNamed() const;
 		MAYBE(Named *) safeClosestNamed() const;
 	};
 
+	/**
+	 * A pair of named items. Used to remember what to update during a reload.
+	 */
+	class NamedPair {
+		STORM_VALUE;
+	public:
+		STORM_CTOR NamedPair(Named *from, Named *to) {
+			this->from = from;
+			this->to = to;
+		}
+
+		Named *from;
+		Named *to;
+
+		// Output.
+		inline void STORM_FN toS(StrBuf *to) const {
+			*to << from << S(" -> ") << to;
+		}
+	};
+
 }
