@@ -14,7 +14,7 @@ namespace storm {
 		STORM_CLASS;
 	public:
 		// Create.
-		STORM_CTOR NamedThread(Str *name);
+		STORM_CTOR NamedThread(SrcPos pos, Str *name);
 		STORM_CTOR NamedThread(syntax::SStr *name);
 
 		// Create, and assign to a specific thread.
@@ -29,6 +29,13 @@ namespace storm {
 		// Output.
 		virtual void STORM_FN toS(StrBuf *b) const;
 
+		// Replace.
+		virtual MAYBE(Str *) STORM_FN canReplace(Named *old, ReplaceContext *ctx);
+
+	protected:
+		// Replace.
+		virtual void STORM_FN doReplace(Named *old, ReplaceTasks *tasks, ReplaceContext *ctx);
+
 	private:
 		// The thread we're referring to.
 		Thread *myThread;
