@@ -8,7 +8,8 @@
 
 namespace storm {
 
-	PkgFiles::PkgFiles() {
+	PkgFiles::PkgFiles(SimpleName *name) {
+		this->name = name;
 		files = new (this) Array<Url *>();
 	}
 
@@ -27,7 +28,7 @@ namespace storm {
 
 			best = name->substr(dot + 1);
 			name = name->substr(name->begin(), dot);
-		} while (best->isNat());
+		} while (best->asNat().any());
 
 		return best;
 	}
@@ -65,12 +66,19 @@ namespace storm {
 		return r;
 	}
 
-	Map<SimpleName *, PkgFiles *> *readerName(Array<Url *> *files) {
-		Map<SimpleName *, PkgFiles *> *r = new (files) Map<SimpleName *, PkgFiles *>();
+	Map<Str *, PkgFiles *> *readerName(Array<Url *> *files) {
+		Map<Str *, PkgFiles *> *r = new (files) Map<Str *, PkgFiles *>();
 		for (nat i = 0; i < files->count(); i++) {
 			Url *f = files->at(i);
-			if (SimpleName *reader = readerName(f))
-				r->at(reader)->files->push(f);
+			if (SimpleName *reader = readerName(f)) {
+				Str *key = reader->toS();
+				PkgFiles *pf = r->get(key, null);
+				if (!pf) {
+					pf = new (r) PkgFiles(reader);
+					r->put(key, pf);
+				}
+				pf->files->push(f);
+			}
 		}
 		return r;
 	}
