@@ -46,7 +46,39 @@ namespace storm {
 			return e.refObjHandle();
 		}
 
+		// Find the current parent type for 'type' that matches 'vtable'.
+		// This is the slow path of 'typeOf'.
+		static Type *findCurrentParent(Type *type, const void *vtable) {
+			for (Type *curr = type->super(); curr; curr = curr->super()) {
+				if (curr->hasVTable(vtable))
+					return curr;
+			}
+			// Note: Looking up the top type will fail sometimes, for example when we have
+			// created a type from an .so file, so that we don't have the same instance of the
+			// vtable as in Storm's metadata (e.g. creating Str from a library). In those cases,
+			// just return the original one.
+			//
+			// Important note: this will never happen for Storm types, since we maintain exactly one
+			// vtable for those types! We could start trying to do C++-style dynamic casts using the
+			// typeinfo in the vtable. That would, however, likely be too slow.
+			return type;
+		}
+
 		Type *typeOf(const RootObject *o) {
+			Type *type = Gc::typeOf(o)->type;
+			// Note: 'type' here is the *allocated* type of the object. It does not correspond to
+			// the current type of the object. This is given by the vtable (they differ during
+			// object construction or destruction, before all constructors are finished). As such,
+			// inspect vtables to assess the situation. Important: we want the common case where
+			// 'type' is indeed the correct answer to be the fast path here. We don't expect
+			// downcasting during construction to be common, but it should be correct!
+			const void *vtable = vtable::from(o);
+			if (type->hasVTable(vtable))
+				return type;
+			return findCurrentParent(type, vtable);
+		}
+
+		Type *allocTypeOf(const RootObject *o) {
 			return Gc::typeOf(o)->type;
 		}
 
@@ -76,7 +108,7 @@ namespace storm {
 				os::Future<Type *> f;
 				os::FnCall<Type *> p = os::fnCall().add(name);
 				os::UThread::spawn(address(&fromIdentifierI), false, p, f, &t);
-				return f.result();
+				return f.result(&updateFutureExceptions, null);
 			} else {
 				return fromIdentifierI(name);
 			}
@@ -120,7 +152,7 @@ namespace storm {
 		}
 
 		static NOINLINE void allocFailType(const GcType *t, Type *type) {
-			PLN(L"Invalid type reference found! GcType: " << (void *)t << L", actual: " << type);
+			PLN(L"Invalid type reference found! GcType: " << (void *)t->type << L", actual: " << (void *)type);
 			debugAssertFailed();
 		}
 
@@ -173,7 +205,7 @@ namespace storm {
 		}
 
 		void setVTable(RootObject *object) {
-			typeOf(object)->vtable()->insert(object);
+			allocTypeOf(object)->vtable()->insert(object);
 		}
 
 		bool liveObject(RootObject *object) {
@@ -225,6 +257,10 @@ namespace storm {
 
 			Object *src = (Object *)obj;
 
+			// For robustness:
+			if (!env)
+				env = new (obj) CloneEnv();
+
 			if (Object *prev = env->cloned(src))
 				return prev;
 
@@ -241,9 +277,11 @@ namespace storm {
 			}
 
 			Object *result = (Object *)mem;
+			env->cloned(src, result);
+
+			// Note: needs to happen *after* cloned is called. Otherwise we don't handle cycles.
 			result->deepCopy(env);
 
-			env->cloned(src, result);
 			return result;
 		}
 
