@@ -821,6 +821,10 @@ namespace storm {
 			}
 		}
 
+		Str *File::contentStr() {
+			return content(parts);
+		}
+
 
 		/**
 		 * Work items posted.
