@@ -1,7 +1,10 @@
 #include "stdafx.h"
 #include "Server.h"
 #include "Engine.h"
+#include "Core/Io/OverlayProtocol.h"
 #include "Core/Timing.h"
+#include "Core/Variant.h"
+#include "Compiler/Lib/Fn.h"
 
 namespace storm {
 	namespace server {
@@ -27,7 +30,12 @@ namespace storm {
 			t = c->symbol(S("t"));
 			completeName = c->symbol(S("complete-name"));
 			documentation = c->symbol(S("documentation"));
+			replAvailable = c->symbol(S("repl-available"));
+			replEval = c->symbol(S("repl-eval"));
+			runSym = c->symbol(S("run"));
+			reload = c->symbol(S("reload"));
 			work = new (this) WorkQueue(this);
+			repls = new (this) Map<Str *, RunningRepl *>();
 			chunkChars = defaultChunkChars;
 		}
 
@@ -116,6 +124,14 @@ namespace storm {
 				onComplete(cell->rest);
 			} else if (*documentation == *kind) {
 				onDocumentation(cell->rest);
+			} else if (*replAvailable == *kind) {
+				onReplAvailable(cell->rest);
+			} else if (*replEval == *kind) {
+				onReplEval(cell->rest);
+			} else if (*runSym == *kind) {
+				onRun(cell->rest);
+			} else if (*reload == *kind) {
+				onReload(cell->rest);
 			} else {
 				print(TO_S(this, S("Unknown message: ") << msg));
 			}
@@ -543,6 +559,357 @@ namespace storm {
 			conn->send(list(engine(), 2, documentation, list(result)));
 		}
 
+		static bool hasRepl(Package *pkg) {
+			SimplePart *part = new (pkg) SimplePart(S("repl"));
+			Function *fn = as<Function>(pkg->find(part, Scope()));
+			if (!fn)
+				return false;
+
+			// Check the return value.
+			return Value(StormInfo<Repl>::type(pkg->engine())).mayStore(fn->result);
+		}
+
+		void Server::onReplAvailable(SExpr *expr) {
+			SExpr *result = null;
+			bool hasBs = false;
+
+			Package *lang = engine().package(S("lang"));
+			for (NameSet::Iter i = lang->begin(), e = lang->end(); i != e; ++i) {
+				if (Package *p = ::as<Package>(i.v())) {
+					if (hasRepl(p)) {
+						if (*p->name == S("bs"))
+							hasBs = true;
+						else
+							result = cons(engine(), new (this) String(p->name), result);
+					}
+				}
+			}
+
+			if (hasBs)
+				result = cons(engine(), new (this) String(S("bs")), result);
+
+			result = cons(engine(), replAvailable, result);
+			conn->send(result);
+		}
+
+		void Server::onReplEval(SExpr *expr) {
+			String *name = next(expr)->asStr();
+			String *eval = next(expr)->asStr();
+			String *context = null;
+			if (SExpr *ctx = next(expr))
+				context = ctx->asStr();
+
+			RunningRepl *repl = repls->get(name->v, null);
+			if (!repl) {
+				SimpleName *replName = new (this) SimpleName();
+				replName->add(new (this) Str(S("lang")));
+				replName->add(name->v);
+				replName->add(new (this) Str(S("repl")));
+				Function *fn = ::as<Function>(engine().scope().find(replName));
+				if (fn) {
+					if (!Value(StormInfo<Repl>::type(engine())).mayStore(fn->result))
+						fn = null;
+				}
+
+				if (!fn) {
+					print(TO_S(this, S("Failed to create REPL for language ") << name->v << S(".")));
+					return;
+				}
+
+				repl = new (this) RunningRepl(this, fn);
+				repls->put(name->v, repl);
+			}
+
+			Package *ctx = null;
+			if (context) {
+				Url *url = parsePath(context->v);
+				if (!url->dir())
+					url = url->parent();
+				ctx = engine().package(url);
+			}
+
+			repl->work->push(ReplTask(eval->v, ctx));
+
+			// If we filled the queue from zero, launch a thread to handle the task.
+			if (repl->work->count() == 1) {
+				os::FnCall<void, 2> call = os::fnCall().add(repl);
+				os::UThread::spawn(address(&RunningRepl::run), true, call);
+			}
+		}
+
+		Server::RunningRepl::RunningRepl(Server *server, Function *create)
+			: server(server), createFn(create), repl(null), work(new (server) Queue<ReplTask>()) {}
+
+		void Server::RunningRepl::run() {
+			while (work->any()) {
+				// Note: Don't pop until we are done.
+				ReplTask task = work->top();
+
+				// See if we need to create the repl:
+				if (!repl) {
+					typedef Repl *(*CreateFn)();
+					CreateFn create = (CreateFn)createFn->ref().address();
+					repl = (*create)();
+				}
+
+				SExpr *result = null;
+
+				try {
+					Repl::Result res = repl->eval(task.input, task.context);
+					if (res.isSuccess()) {
+						if (Str *r = res.result())
+							result = list(engine(), 1, new (this) String(r));
+					} else if (Str *e = res.isError()) {
+						result = list(engine(), 2, null, new (this) String(e));
+					} else if (res.isIncomplete()) {
+						result = list(engine(), 2, null, new (this) String(S("Incomplete input")));
+					} else if (res.isTerminate()) {
+						// Clear the REPL so that we create a new one the next time!
+						repl = null;
+						result = list(engine(), 1, new (this) String(S("REPL was terminated")));
+					}
+				} catch (const storm::Exception *error) {
+					result = list(engine(), 2, null, new (this) String(error->toS()));
+				}
+
+				result = cons(engine(), server->replEval, result);
+
+				{
+					Lock::Guard z(server->lock);
+					server->conn->send(result);
+				}
+
+				// Pop last now that we are done.
+				work->pop();
+			}
+		}
+
+		static void findFunctions(NameSet *inside, Url *file, Array<Function *> *fns) {
+			inside->forceLoad();
+			for (NameSet::Iter i = inside->begin(), e = inside->end(); i != e; ++i) {
+				Named *n = i.v();
+				if (!n->pos.file || *n->pos.file != *file)
+					continue;
+
+				if (NameSet *s = as<NameSet>(n)) {
+					findFunctions(s, file, fns);
+				} else if (Function *f = as<Function>(n)) {
+					*fns << f;
+				}
+			}
+		}
+
+		// Pick the function declared closest above the given position.
+		static Function *pickClosest(Array<Function *> *fns, Nat pos) {
+			Function *candidate = null;
+
+			for (Nat i = 0; i < fns->count(); i++) {
+				Function *f = fns->at(i);
+				if (f->pos.start > pos)
+					continue;
+
+				// Inside this function, that is perfect!
+				if (f->pos.end > pos)
+					return f;
+
+				// Closest one after (if the cursor is right after a function, or if only the first line is covered).
+				if (!candidate || f->pos.end > candidate->pos.end)
+					candidate = f;
+			}
+
+			return candidate;
+		}
+
+		void Server::onRun(SExpr *expr) {
+			Url *file = null;
+			SExpr *srcFile = next(expr);
+			if (Number *num = ::as<Number>(srcFile)) {
+				file = files->get(num->v)->url();
+			} else if (String *name = ::as<String>(srcFile)) {
+				file = parsePath(name->v);
+			} else {
+				return;
+			}
+
+			Package *pkg = engine().package(file->parent());
+			if (!pkg) {
+				Str *msg = TO_S(this, S("Failed to find a package containing ") << file << S("."));
+				conn->send(list(engine(), 3, runSym, null, new (this) String(msg)));
+				return;
+			}
+
+			Array<Function *> *fns = new (this) Array<Function *>();
+			findFunctions(pkg, file, fns);
+
+			Bool send = false;
+			if (expr) {
+				send = true;
+				Nat pos = next(expr)->asNum()->v;
+				Function *f = pickClosest(fns, pos);
+				fns->clear();
+				if (f) {
+					if (f->params->any()) {
+						Str *msg = TO_S(this, S("The function ") << f->identifier() << S(" requires parameters and ")
+										S("may therefore not be executed in this way."));
+						conn->send(list(engine(), 3, runSym, null, new (this) String(msg)));
+						return;
+					}
+					*fns << f;
+				}
+			}
+
+			if (fns->empty()) {
+				Str *msg = TO_S(this, S("Unable to find any functions to execute."));
+				conn->send(list(engine(), 3, runSym, null, new (this) String(msg)));
+				return;
+			}
+
+			for (Nat i = 0; i < fns->count(); i++) {
+				Function *f = fns->at(i);
+				if (f->params->any())
+					continue;
+
+				Server *me = this;
+				os::FnCall<void, 3> call = os::fnCall().add(me).add(f).add(send);
+				os::UThread::spawn(address(&Server::execThread), true, call);
+			}
+		}
+
+		void Server::execThread(Function *fn, Bool send) {
+			FnBase *ptr = pointer(fn);
+			Value type = fn->result;
+			SExpr *expr = null;
+			StrBuf *result = new (this) StrBuf();
+			if (!send)
+				*result << fn->identifier() << S(": ");
+
+			try {
+				if (type.isValue()) {
+					void *mem = alloca(type.size().current());
+					void **params[1] = { null };
+					ptr->rawCall().call(ptr, mem, params);
+
+					if (result) {
+						Variant r(mem, type.type);
+						*result << r;
+					}
+
+					type.type->handle().safeDestroy(mem);
+
+				} else if (type.isObject()) {
+					RootObject *obj;
+					void **params[1] = { null };
+					ptr->rawCall().call(ptr, &obj, params);
+
+					if (Object *o = ::as<Object>(obj))
+						*result << o;
+					else if (TObject *t = ::as<TObject>(obj))
+						*result << t;
+				} else {
+					// Must be 'void'.
+					os::FnCall<void, 1> params = os::fnCall();
+					ptr->callRaw(params, 0, null);
+
+					if (result)
+						*result << S("<void>");
+				}
+
+				if (send)
+					expr = list(engine(), 2, runSym, new (this) String(result->toS()));
+				else {
+					Lock::Guard z(lock);
+					print(result->toS());
+				}
+			} catch (Exception *e) {
+				if (send) {
+					expr = list(engine(), 3, runSym, null, new (this) String(e->toS()));
+				} else {
+					if (::as<CodeError>(e)) {
+						// Add a newline so that compilation errors can be jumped to.
+						*result << S("Error:\n");
+					}
+					*result << e;
+
+					Lock::Guard z(lock);
+					print(result->toS());
+				}
+			}
+
+			if (expr) {
+				Lock::Guard z(lock);
+				conn->send(expr);
+			}
+		}
+
+		void Server::onReload(SExpr *expr) {
+			typedef Map<Url *, PkgFiles *> PkgMap;
+			PkgMap *toReload = new (this) Map<Url *, PkgFiles *>();
+
+			Moment start;
+			Nat fileCount = 0;
+			Nat pkgCount = 0;
+
+			while (expr) {
+				SExpr *form = next(expr);
+				Url *file = null;
+				if (Number *num = ::as<Number>(form)) {
+					// Use the contents of the actual buffer.
+					File *f = files->get(num->v);
+					Url *fileUrl = f->url();
+					OverlayProtocol *overlay = new (this) OverlayProtocol(fileUrl->parent());
+					file = overlay->put(fileUrl->name(), f->contentStr());
+				} else if (String *str = ::as<String>(form)) {
+					file = parsePath(str->v);
+				} else {
+					continue;
+				}
+
+				Url *path = file;
+				if (!file->dir())
+					path = file->parent();
+
+				if (PkgFiles *f = toReload->get(path, null)) {
+					f->files->push(file);
+				} else {
+					f = new (this) PkgFiles(null); // Note: Small hack here...
+					f->files->push(file);
+					toReload->put(path, f);
+				}
+			}
+
+			for (PkgMap::Iter i = toReload->begin(), end = toReload->end(); i != end; ++i) {
+				Package *pkg = engine().package(i.k());
+				Array<Url *> *files = i.v()->files;
+				if (!pkg) {
+					StrBuf *msg = new (this) StrBuf();
+					*msg << S("Warning: No package found for the directory ") << i.k()
+						 << S(". Cannot reload the following files:");
+					for (Nat j = 0; j < files->count(); j++)
+						*msg << S("\n  ") << files->at(j);
+					print(msg->toS());
+					continue;
+				}
+
+				Bool dir = false;
+				for (Nat j = 0; j < files->count(); j++)
+					if (files->at(j)->dir())
+						dir = true;
+
+				if (dir) {
+					pkgCount++;
+					pkg->reload();
+				} else {
+					fileCount++;
+					pkg->reload(i.v()->files, false);
+				}
+			}
+
+			StrBuf *msg = new (this) StrBuf();
+			*msg << S("Reloaded ") << fileCount << S(" files and ") << pkgCount
+				 << S(" packages in ") << (Moment() - start) << S(".");
+			print(msg->toS());
+		}
+
 		/**
 		 * Update logic.
 		 */
