@@ -95,10 +95,10 @@ namespace storm {
 				src = null;
 			}
 
-			Bool Parser::parse(Rule *root, Str *str, Url *file, Str::Iter start) {
+			Bool Parser::parse(Rule *root, Str *str, Url *file, Str::Iter start, Str::Iter end) {
 				// Remember what we parsed.
 				src = str;
-				Nat len = str->peekLength() + 1 - start.offset();
+				Nat len = end.offset() + 1 - start.offset();
 				lastFinish = len + 1;
 				srcFile = file;
 				srcOffset = start.offset();
@@ -407,7 +407,7 @@ namespace storm {
 
 			template <class T>
 			static void setValue(Node *node, MemberVar *target, T *elem) {
-				int offset = target->offset().current();
+				int offset = target->rawOffset().current();
 				checkOffset(node, offset);
 				if (isArray(target->type)) {
 					// Arrays are initialized earlier.
@@ -485,7 +485,7 @@ namespace storm {
 				// Create any arrays needed.
 				for (nat i = 0; i < type->arrayMembers->count(); i++) {
 					MemberVar *v = type->arrayMembers->at(i);
-					int offset = v->offset().current();
+					int offset = v->rawOffset().current();
 					checkOffset(r, offset);
 					// This will actually create the correct subtype as long as we're creating something
 					// inherited from Object (which we are).
@@ -505,7 +505,7 @@ namespace storm {
 
 				for (nat i = 0; i < t->arrayMembers->count(); i++) {
 					MemberVar *v = t->arrayMembers->at(i);
-					int offset = v->offset().current();
+					int offset = v->rawOffset().current();
 
 					Array<Object *> *array = OFFSET_IN(node, offset, Array<Object *> *);
 					array->reverse();
