@@ -56,8 +56,8 @@ namespace storm {
 				return syntax->sameSyntax(o->syntax);
 			}
 
-			Bool Parser::parse(Rule *root, Str *str, Url *file, Str::Iter start) {
-				initParse(root, str, file, start);
+			Bool Parser::parse(Rule *root, Str *str, Url *file, Str::Iter start, Str::Iter end) {
+				initParse(root, str, file, start, end);
 				doParse(startPos);
 				finishParse(null);
 
@@ -89,16 +89,16 @@ namespace storm {
 			 * possibly skipping the offending characters using the error recovery rather than in a
 			 * regex.
 			 */
-			InfoErrors Parser::parseApprox(Rule *root, Str *str, Url *file, Str::Iter start, MAYBE(InfoInternal *) ctx) {
-				initParse(root, str, file, start);
+			InfoErrors Parser::parseApprox(Rule *root, Str *str, Url *file, Str::Iter start, Str::Iter end,
+										MAYBE(InfoInternal *) ctx) {
+				initParse(root, str, file, start, end);
 
 				// Start as usual.
 				Array<StackItem *> *lastSet = null;
 				Nat lastPos = 0;
 				doParse(startPos, lastSet, lastPos);
 
-				Nat length = source->peekLength();
-				while (lastPos <= length && (acceptingStack == null || acceptingStack->pos < length)) {
+				while (lastPos <= endPos && (acceptingStack == null || acceptingStack->pos < endPos)) {
 					// Advance productions to accommodate for missing characters.
 					stacks->set(0, lastSet);
 					currentPos = lastPos;
@@ -111,7 +111,7 @@ namespace storm {
 					// reduce operations we have already done in 'advanceAll()', which takes
 					// time. 'actorShift()' only looks at the possible shifts, which is enough as we
 					// have already done all possible reduces already.
-					while (!actorShift() && currentPos < length) {
+					while (!actorShift() && currentPos < endPos) {
 						// Advance to the next step in the input while keeping the current stack top.
 						currentPos++;
 						lastPos++;
@@ -119,7 +119,7 @@ namespace storm {
 
 					// Advance from the current state and keep on parsing, as we now know that we
 					// have something to do in the future.
-					if (currentPos < length) {
+					if (currentPos < endPos) {
 						currentPos++;
 						lastPos++;
 						stacks->pop();
@@ -137,9 +137,10 @@ namespace storm {
 				}
 			}
 
-			void Parser::initParse(Rule *root, Str *str, Url *file, Str::Iter start) {
+			void Parser::initParse(Rule *root, Str *str, Url *file, Str::Iter start, Str::Iter end) {
 				source = str;
 				startPos = start.offset();
+				endPos = end.offset();
 				sourceUrl = file;
 				parseRoot = syntax->lookup(root);
 
@@ -153,8 +154,7 @@ namespace storm {
 			}
 
 			void Parser::doParse(Nat from) {
-				Nat length = source->peekLength();
-				for (Nat i = from; i <= length; i++) {
+				for (Nat i = from; i <= endPos; i++) {
 					Array<StackItem *> *top = stacks->top();
 
 					// Process all states in 'top'.
@@ -172,8 +172,7 @@ namespace storm {
 				Nat badness = 0;
 
 				Nat newLine = from;
-				Nat length = source->peekLength();
-				for (Nat i = from; i <= length; i++) {
+				for (Nat i = from; i <= endPos; i++) {
 					Array<StackItem *> *top = stacks->top();
 
 					// Process all states in 'top'.
@@ -742,14 +741,14 @@ namespace storm {
 					return true;
 				if (acceptingStack->required.any())
 					return true;
-				return acceptingStack->pos < source->peekLength();
+				return acceptingStack->pos < endPos;
 			}
 
 			Str *Parser::errorMsg() const {
 				StrBuf *out = new (this) StrBuf();
 				if (acceptingStack && acceptingStack->required.any())
 					reqErrorMsg(out, acceptingStack);
-				else if (lastPos == source->peekLength())
+				else if (lastPos == endPos)
 					*out << S("Unexpected end of file.");
 				else if (lastSet)
 					errorMsg(out, lastPos, lastSet);
@@ -875,7 +874,7 @@ namespace storm {
 
 				for (Nat i = 0; i < t->arrayMembers->count(); i++) {
 					MemberVar *v = t->arrayMembers->at(i);
-					int offset = v->offset().current();
+					int offset = v->rawOffset().current();
 					Array<Object *> *array = OFFSET_IN(node, offset, Array<Object *> *);
 					array->reverse();
 				}
@@ -883,7 +882,7 @@ namespace storm {
 
 			template <class T>
 			static void setValue(Node *node, MemberVar *target, T *elem) {
-				int offset = target->offset().current();
+				int offset = target->rawOffset().current();
 				if (isArray(target->type)) {
 					// Arrays are initialized earlier.
 					OFFSET_IN(node, offset, Array<T *> *)->push(elem);
@@ -1011,7 +1010,7 @@ namespace storm {
 				// Create any arrays needed.
 				for (nat i = 0; i < type->arrayMembers->count(); i++) {
 					MemberVar *v = type->arrayMembers->at(i);
-					int offset = v->offset().current();
+					int offset = v->rawOffset().current();
 
 					// This will actually create the correct subtype as long as we're creating
 					// something inherited from Object or TObject (which we are).
