@@ -15,11 +15,12 @@ namespace storm {
 			return true;
 		}
 
-		Bool ParserBackend::parse(Rule *root, Str *str, Url *file, Str::Iter start) {
+		Bool ParserBackend::parse(Rule *root, Str *str, Url *file, Str::Iter start, Str::Iter end) {
 			return false;
 		}
 
-		InfoErrors ParserBackend::parseApprox(Rule *root, Str *str, Url *file, Str::Iter start, MAYBE(InfoInternal *) ctx) {
+		InfoErrors ParserBackend::parseApprox(Rule *root, Str *str, Url *file,
+											Str::Iter start, Str::Iter end, MAYBE(InfoInternal *) ctx) {
 			return infoFailure();
 		}
 
