@@ -112,13 +112,13 @@ namespace storm {
 		if (e) {
 			switch (info->si_code) {
 			case SEGV_MAPERR:
-				throw new (*e) MemoryAccessError(Word(info->si_addr), MemoryAccessError::notMapped);
+				throw new (*e) MemoryAccessError(Word(info->si_addr), MemoryAccessError::notMapped, context);
 			case SEGV_ACCERR:
-				throw new (*e) MemoryAccessError(Word(info->si_addr), MemoryAccessError::invalidAccess);
+				throw new (*e) MemoryAccessError(Word(info->si_addr), MemoryAccessError::invalidAccess, context);
 			case SI_KERNEL:
 				// Note: We can get SI_KERNEL (128), which means an invalid address. Sadly, the
 				// source address is not available in si_addr in that case...
-				throw new (*e) MemoryAccessError(Word(info->si_addr), MemoryAccessError::kernel);
+				throw new (*e) MemoryAccessError(Word(info->si_addr), MemoryAccessError::kernel, context);
 			default:
 				// We could add more...
 				break;
@@ -146,9 +146,9 @@ namespace storm {
 		if (e) {
 			switch (info->si_code) {
 			case BUS_ADRALN:
-				throw new (*e) MemoryAccessError(Word(info->si_addr), MemoryAccessError::invalidAlignment);
+				throw new (*e) MemoryAccessError(Word(info->si_addr), MemoryAccessError::invalidAlignment, context);
 			case BUS_ADRERR:
-				throw new (*e) MemoryAccessError(Word(info->si_addr), MemoryAccessError::notMapped);
+				throw new (*e) MemoryAccessError(Word(info->si_addr), MemoryAccessError::notMapped, context);
 			default:
 				// We could add more...
 				break;
