@@ -1207,8 +1207,19 @@ namespace storm {
 
 		// Update derived classes (Note: We don't need to update our super class).
 		if (o->chain) {
+			// Note: Calling 'setSuper' will modify the WeakSet inside o->chain that we use to find
+			// all children of the old class. This is not good, as it means that we might miss
+			// elements in it (e.g. when two or more elements form a chain in consecutive elements,
+			// then deleting the first one will make the map move the second one backwards).
+			// Therefore, we need to save the children in a separate array first.
+			Array<Type *> *toUpdate = new (this) Array<Type *>();
 			TypeChain::Iter iter = o->chain->children();
-			while (Type *child = iter.next()) {
+			while (Type *child = iter.next())
+				toUpdate->push(child);
+
+			// Now we can update them all without worrying.
+			for (Nat i = 0; i < toUpdate->count(); i++) {
+				Type *child = toUpdate->at(i);
 				if (child->super() != this)
 					child->setSuper(this, tasks);
 			}
