@@ -66,6 +66,15 @@ namespace storm {
 		cpp->insert(to, obj, code::Ref(source));
 	}
 
+	const void **VTable::pointer() {
+		return cpp->pointer();
+	}
+
+	void VTable::replace(VTable *old, ReplaceTasks *tasks) {
+		source->steal(old->source);
+		tasks->replace(old, this);
+	}
+
 	Array<Function *> *VTable::allSlots() {
 		Array<Function *> *cppFns = new (this) Array<Function *>(cpp->count(), null);
 		Array<Function *> *stormFns = null;
@@ -142,6 +151,13 @@ namespace storm {
 		if (!slot.valid()) {
 			// We need to allocate a slot for this function.
 			slot = allocSlot();
+		} else {
+			// See if something was in the slot before. If so, remove the corresponding updater.
+			// This happens if a user has replaced an entity marked with "namedAllowReplace".
+			Function *old = get(slot);
+			if (old) {
+				updaters->remove(old);
+			}
 		}
 
 		// Insert into the vtable.
@@ -153,6 +169,36 @@ namespace storm {
 			useLookup(fn, slot);
 	}
 
+	void VTable::remove(Function *fn) {
+		// Do we know this function?
+		VTableUpdater *u = updaters->get(fn, null);
+		if (!u)
+			return;
+
+		VTableSlot slot = u->slot();
+		clear(slot);
+
+		if (Type *s = owner->super())
+			rawVTable(s)->remove(slot);
+	}
+
+	void VTable::remove(VTableSlot slot) {
+		if (Function *fn = get(slot)) {
+			if (!hasOverride(slot)) {
+				// Disable vtable lookup for this function.
+				if (fn->ref().address() != fn->directRef().address()) {
+					fn->setLookup(null);
+				}
+
+				// No need to continue traversal.
+				return;
+			}
+		}
+
+		if (Type *s = owner->super())
+			rawVTable(s)->remove(slot);
+	}
+
 	void VTable::removeChild(Type *child) {
 		// For all slots: find a match to that function in a pre-existing child class.
 
