@@ -66,6 +66,10 @@ namespace storm {
 		// and will be set to use a lookup if needed.
 		void STORM_FN insert(Function *fn);
 
+		// Remove a function from this VTable. Will ensure that any parent functions that no longer
+		// require vtable lookup.
+		void STORM_FN remove(Function *fn);
+
 		// Notify that a child class has been removed. We assume it is not possible to reach 'type'
 		// by traversing the inheritance hierarchy.
 		void STORM_FN removeChild(Type *type);
@@ -79,6 +83,12 @@ namespace storm {
 		// Generate code for setting the vtable.
 		void STORM_FN insert(code::Listing *to, code::Var obj);
 
+		// Get the vtable pointer. Marks it as used.
+		const void **pointer();
+
+		// Make this vtable replace the other one (i.e. update refs).
+		void replace(VTable *old, ReplaceTasks *tasks);
+
 		// Get the "topmost" functions for each entry in the vtable. Unused slots are left out
 		// (since they would be null).
 		Array<Function *> *allSlots();
@@ -86,6 +96,15 @@ namespace storm {
 		// Dump the vtable contents to stdout.
 		void dbg_dump() const;
 
+		// Check if this object stores the same vtable as 'vtable' (extracted using 'vtable::from').
+		// Called from 'runtime::typeOf', so needs to be fairly fast.
+		inline Bool sameAs(const void *vtable) const {
+			if (cpp)
+				return cpp->sameAs(vtable);
+			else
+				return false;
+		}
+
 	private:
 		// Owning type.
 		Type *owner;
@@ -107,6 +126,9 @@ namespace storm {
 		// RefSource referring to the VTable.
 		code::RefSource *source;
 
+		// Called on super-classes whenever a parent slot has been vacated.
+		void remove(VTableSlot slot);
+
 		// Called when one of our parent classes have grown their vtable and we need to follow.
 		void parentGrown(Nat pos, Nat count);
 
