@@ -56,7 +56,9 @@ namespace storm {
 	Bool Value::mayStore(MAYBE(Type *) o) const {
 		if (type == o)
 			return true;
-		if (type == null) // note: implies o->type == null
+		if (type == null) // note: implies o != null
+			return false;
+		if (o == null) // note: implies type != null
 			return false;
 		// Only check inheritance for objects. We don't have to make sure that both are objects,
 		// since 'isA' will fail if one is an object and another is a value.
@@ -80,10 +82,15 @@ namespace storm {
 	}
 
 	Bool Value::matches(Value v, NamedFlags flags) const {
-		if (flags & namedMatchNoInheritance)
+		if ((flags & namedStrictRef) && (ref != v.ref))
+			return false;
+
+		if (flags & namedMatchNoInheritance) {
+			// TODO: We should probably be a bit more restrictive with ref vs no ref?
 			return type == v.type;
-		else
+		} else {
 			return mayReferTo(v);
+		}
 	}
 
 	Value common(Value a, Value b) {
