@@ -31,7 +31,7 @@ namespace storm {
 			*buf << data->v[0];
 
 		for (nat i = 1; i < count(); i++)
-			*buf << L", " << data->v[0];
+			*buf << L", " << data->v[i];
 
 		*buf << L"]";
 	}
