@@ -2,6 +2,7 @@
 #include "Named.h"
 #include "NamedThread.h"
 #include "RunOn.h"
+#include "Code/OffsetReference.h"
 #include "Core/Fn.h"
 #include "Core/Variant.h"
 
@@ -41,23 +42,43 @@ namespace storm {
 		STORM_CTOR MemberVar(Str *name, Value type, Type *memberOf);
 		STORM_CTOR MemberVar(SrcPos pos, Str *name, Value type, Type *memberOf);
 
-		// Get the offset of this member.
-		Offset STORM_FN offset() const;
+		// Get a reference to this variable. Use the reference whenever storing the offset for more
+		// than a couple of function calls, as it might change during reloads.
+		code::OffsetRef STORM_FN offset() const;
 
 		// Get our parent type.
 		Type *STORM_FN owner() const;
 
+		// Get the raw offset. Prefer to use the reference whenever possible.
+		Offset STORM_FN rawOffset() const;
+
 		// Set our offset.
 		void setOffset(Offset off);
 
 		// Initializer, if any.
-		MAYBE(Function *) initializer;
+		MAYBE(Function *) STORM_FN initializer() { return initFn; }
+		void STORM_ASSIGN initializer(MAYBE(Function *) fn);
+
+		// Keep track of the source of the initializer.
+		void STORM_FN discardSource();
+
+		// Check if it is possible to update.
+		virtual MAYBE(Str *) STORM_FN canReplace(Named *old, ReplaceContext *ctx);
+
+	protected:
+		// Replace with another variable.
+		virtual void STORM_FN doReplace(Named *old, ReplaceTasks *tasks, ReplaceContext *context);
 
-		// TODO: We might want to offer references for using the offset, so we can easily update it.
 	private:
 		// The actual offset. Updated by 'layout'.
 		Offset off;
 
+		// Reference for the offset.
+		code::OffsetSource *ref;
+
+		// Initializer.
+		MAYBE(Function *) initFn;
+
 		// Has the layout been produced?
 		Bool hasLayout;
 	};
