@@ -44,6 +44,29 @@ namespace storm {
 		return leading() == 0 ? 1 : 2;
 	}
 
+	Bool Char::utf16IsLeading(Nat point) {
+		return utf16::leading(nat16(point));
+	}
+
+	Bool Char::utf16IsTrailing(Nat point) {
+		return utf16::trailing(nat16(point));
+	}
+
+	Char Char::utf16Assemble(Nat leading, Nat trailing) {
+		nat16 l = nat16(leading);
+		nat16 t = nat16(trailing);
+
+		if (leading == 0) {
+			if (utf16::leading(t) || utf16::trailing(t))
+				return Char('\0');
+			return Char(t);
+		} else {
+			if (!utf16::leading(l) || !utf16::trailing(t))
+				return Char('\0');
+			return Char(utf16::assemble(l, t));
+		}
+	}
+
 	void Char::deepCopy(CloneEnv *env) {}
 
 	void Char::toS(StrBuf *to) const {
