@@ -20,4 +20,12 @@ namespace storm {
 		data->put(o, to);
 	}
 
+	Object *CloneEnv::get(Object *o) {
+		return data->get(o);
+	}
+
+	MAYBE(Object *) CloneEnv::at(Object *o) {
+		return data->get(o, null);
+	}
+
 }
