@@ -15,13 +15,19 @@ namespace storm {
 
 	// Convert from 'char' to 'wchar'.
 	size_t convert(const char *from, wchar *to, size_t maxCount);
-	size_t convert(const char *from, size_t inputCount, wchar *to, size_t maxCount);
+	size_t convert(const char *from, size_t fromCount, wchar *to, size_t maxCount);
+	size_t convert(const char *from, size_t fromCount, wchar *to, size_t maxCount, bool nullTerminate);
 	GcArray<wchar> *toWChar(Engine &e, const char *from);
-	GcArray<wchar> *toWChar(Engine &e, const char *from, size_t maxCount);
+	GcArray<wchar> *toWChar(Engine &e, const char *from, size_t fromCount);
+	GcArray<wchar> *toWCharNoNull(Engine &e, const char *from, size_t fromCount);
 
 	// Convert from 'wchar' to 'char'.
 	size_t convert(const wchar *from, char *to, size_t maxCount);
+	size_t convert(const wchar *from, size_t fromCount, char *to, size_t maxCount);
+	size_t convert(const wchar *from, size_t fromCount, char *to, size_t maxCount, bool nullTerminate);
 	GcArray<char> *toChar(Engine &e, const wchar *from);
+	GcArray<char> *toChar(Engine &e, const wchar *from, size_t fromCount);
+	GcArray<char> *toCharNoNull(Engine &e, const wchar *from, size_t fromCount);
 
 #ifdef POSIX
 
