@@ -2,6 +2,7 @@
 #include "Object.h"
 #include "TObject.h"
 #include "CloneEnv.h"
+#include "StrBuf.h"
 #include "OS/FnCall.h"
 #include "Utils/Templates.h"
 
@@ -66,19 +67,23 @@ namespace storm {
 		virtual const void *ptr() const = 0;
 
 		// Add some kind of label to a StrBuf for the string representation.
-		virtual void toS(StrBuf *to) const = 0;
+		virtual void toS(StrBuf *to, const RootObject *thisPtr) const = 0;
+
+		// Get the underlying constructor (usually the same as 'ptr', but not always).
+		virtual const void *ctor(const RootObject *thisPtr) const = 0;
 	};
 
 	/**
 	 * Target for raw function pointers.
 	 */
-	class RawFnTarget {
+	class RawFnTarget : public FnTarget {
 	public:
 		RawFnTarget(const void *ptr);
 
 		virtual void cloneTo(void *to, size_t size) const;
 		virtual const void *ptr() const;
-		virtual void toS(StrBuf *to) const;
+		virtual void toS(StrBuf *to, const RootObject *thisPtr) const;
+		virtual const void *ctor(const RootObject *thisPtr) const;
 	private:
 		const void *data;
 	};
@@ -93,8 +98,16 @@ namespace storm {
 		// Create from C++.
 		FnBase(const void *fn, const RootObject *thisPtr, Bool member, Thread *thread);
 
+		// Creation mode:
+		enum {
+			modeFreeFn    = 0x0, // Free function.
+			modeMember    = 0x1, // Member function.
+			modeCtor      = 0x2, // Constructor of values or classes
+			modeActorCtor = 0x3, // Constructor of actors (involves threading)
+		};
+
 		// Create with a generic target and all information given.
-		FnBase(const FnTarget &target, const RootObject *thisPtr, Bool member, Thread *thread);
+		FnBase(const FnTarget &target, const RootObject *thisPtr, Nat mode, Thread *thread);
 
 		// Copy.
 		FnBase(const FnBase &o);
@@ -105,25 +118,27 @@ namespace storm {
 		// To string.
 		virtual void STORM_FN toS(StrBuf *to) const;
 
+
 		/**
 		 * Call this function:
 		 */
 
 		// Call our function (low-level, not typesafe).
 		// Takes care of delegating to another thread if needed and adds a this pointer if needed.
-		// The 'first' parameter is the first parameter as a TObject (if the first parameter is a TObject)
-		// and is used to decide if a thread call is to be made or not.
+		// The 'firstInfo' parameter contains the first parameter, tagged to know the contents.
+		// If it is zero, then the first parameter was not interesting. If the LSB is 0, then
+		// it was a TObject. If the LSB is 1, then it was a Thread.
 		// Note: Does *not* copy parameters if needed, but handles the return value properly.
 		// Note: if 'params' are statically allocated, make sure it has room for at least one more element!
 		template <class R, int C>
-		R callRaw(const os::FnCall<R, C> &params, const TObject *first, CloneEnv *env) const {
+		R callRaw(const os::FnCall<R, C> &params, size_t firstInfo, CloneEnv *env) const {
 			byte d[sizeof(R)];
-			callRawI(d, params, first, env);
+			callRawI(d, params, firstInfo, env);
 			R *result = (R *)d;
 			R copy = *result;
 			result->~R();
 
-			if (needsCopy(first)) {
+			if (needsCopy(firstInfo) & 0x2) {
 				if (!env)
 					env = new (this) CloneEnv();
 				cloned(copy, env);
@@ -133,15 +148,21 @@ namespace storm {
 
 		// Specialization for returning void.
 		template <int C>
-		void callRaw(const os::FnCall<void, C> &params, const TObject *first, CloneEnv *env) const {
-			callRawI(null, params, first, env);
+		void callRaw(const os::FnCall<void, C> &params, size_t firstInfo, CloneEnv *env) const {
+			callRawI(null, params, firstInfo, env);
 		}
 
 		// Call function with a pointer to the return value. Low-level function used by other generated code.
-		void callRawI(void *output, const os::FnCallRaw &params, const TObject *first, CloneEnv *env) const;
+		// Note: 'env' is a reference that will be initialized if a CloneEnv was created.
+		void callRawI(void *output, const os::FnCallRaw &params, size_t firstInfo, CloneEnv *&env) const;
+
+		// Do we need to copy the parameters for this function given the first TObject? Returns 0x1
+		// if parameters need to be copied, 0x2 if the result needs to be copied, or 0x3 if both.
+		Byte CODECALL needsCopy(size_t firstInfo) const;
+
+		// Get a pointer to the underlying constructor if this FnBase refers to a constructor.
+		const void *rawCtor() const;
 
-		// Do we need to copy the parameters for this function given the first TObject?
-		bool CODECALL needsCopy(const TObject *first) const;
 
 		/**
 		 * Call the function using a generic parameter list in the form of an array. The
@@ -150,9 +171,10 @@ namespace storm {
 
 		virtual RawFn STORM_FN rawCall();
 
-	public:
-		// Are we calling a member function?
-		Bool callMember;
+
+	private:
+		// Type of function pointer.
+		Nat ptrMode;
 
 		// This pointer.
 		UNKNOWN(PTR_GC) const RootObject *thisPtr;
@@ -169,7 +191,7 @@ namespace storm {
 		inline FnTarget *target() const { return (FnTarget *)&target0; }
 
 		// Compute which thread we want to run on.
-		Thread *runOn(const TObject *first) const;
+		Thread *runOn(size_t firstInfo) const;
 	};
 
 	// Declare the template to Storm.
@@ -177,15 +199,21 @@ namespace storm {
 
 
 	/**
-	 * Get 'x' as a TObject.
-	 * Note: regular overloads have higher priority than templates as long as no implicit conversion has to be made.
+	 * Categorize the first parameter.
+	 * Note: Regular overloads have higher prioerty than templates as long as no implicit conversion has to be made.
+	 * Downcasts are OK.
 	 */
 	template <class T>
-	inline const TObject *asTObject(const T &) { return null; }
-	inline const TObject *asTObject(const TObject &t) { return &t; }
-	inline const TObject *asTObject(const TObject *t) { return t; }
-	inline const TObject *asTObject(TObject &t) { return &t; }
-	inline const TObject *asTObject(TObject *t) { return t; }
+	inline size_t categorizeFirst(const T &) { return 0; }
+	inline size_t categorizeFirst(const TObject &t) { return (size_t)&t; }
+	inline size_t categorizeFirst(const TObject *t) { return (size_t)t; }
+	inline size_t categorizeFirst(TObject &t) { return (size_t)&t; }
+	inline size_t categorizeFirst(TObject *t) { return (size_t)t; }
+	inline size_t categorizeFirst(const Thread &t) { return 0x1 | (size_t)&t; }
+	inline size_t categorizeFirst(const Thread *t) { return 0x1 | (size_t)t; }
+	inline size_t categorizeFirst(Thread &t) { return 0x1 | (size_t)&t; }
+	inline size_t categorizeFirst(Thread *t) { return 0x1 | (size_t)t; }
+
 
 	// Helper macro to add a cloned object to a os::FnParams object. We need to keep the cloned
 	// value alive until the function is actually called, so we can not use a function.
@@ -212,11 +240,22 @@ namespace storm {
 			runtime::setVTable(this);
 		}
 
+		// Note: parameter 'z' is for SFINAE to ensure that P1 is Q*
+		template <class Q>
+		Fn(R (CODECALL Q::*ptr)(P2), Thread *thread = null, Q *z = P1()) : FnBase(address(ptr), null, true, thread) {
+			runtime::setVTable(this);
+		}
+
 		template <class Q>
 		Fn(R (CODECALL Q::*ptr)(P1, P2), const Q *obj) : FnBase(address(ptr), obj, true, null) {
 			runtime::setVTable(this);
 		}
 
+		template <class B>
+		Fn(R (CODECALL *ptr)(B *, P1, P2), B *obj, Thread *thread = null) : FnBase(address(ptr), obj, false, thread) {
+			runtime::setVTable(this);
+		}
+
 		// Copy:
 		Fn(const Fn<R, P1, P2> &other) : FnBase(other) {
 			runtime::setVTable(this);
@@ -224,17 +263,17 @@ namespace storm {
 
 		// Call the function.
 		R call(P1 p1, P2 p2) const {
-			const TObject *first = asTObject(p1);
+			size_t firstInfo = categorizeFirst(p1);
 
-			if (needsCopy(first)) {
+			if (needsCopy(firstInfo) & 0x1) {
 				CloneEnv *env = new (this) CloneEnv();
 				FN_CLONE(P1, params, p1, env);
 				FN_CLONE(P2, params, p2, env);
 				os::FnCall<R, 2> params = os::fnCall().add(c_p1).add(c_p2);
-				return callRaw(params, first, env);
+				return callRaw(params, firstInfo, env);
 			} else {
 				os::FnCall<R, 2> params = os::fnCall().add(p1).add(p2);
-				return callRaw(params, first, null);
+				return callRaw(params, firstInfo, null);
 			}
 		}
 	};
@@ -256,11 +295,26 @@ namespace storm {
 			runtime::setVTable(this);
 		}
 
+		// Note: parameter 'z' is for SFINAE to ensure that P1 is Q*
+		template <class Q>
+		Fn(R (CODECALL Q::*ptr)(), Thread *thread = null, Q *z = P1()) : FnBase(address(ptr), null, true, thread) {
+			runtime::setVTable(this);
+		}
+
 		template <class Q>
 		Fn(R (CODECALL Q::*ptr)(P1), const Q *obj) : FnBase(address(ptr), obj, true, null) {
 			runtime::setVTable(this);
 		}
 
+		template <class B>
+		Fn(R (CODECALL *ptr)(B *, P1), B *obj, Thread *thread = null) : FnBase(address(ptr), obj, false, thread) {
+			runtime::setVTable(this);
+		}
+
+		Fn(const FnTarget &target, RootObject *thisPtr, Nat mode) : FnBase(target, thisPtr, mode, null) {
+			runtime::setVTable(this);
+		}
+
 		// Copy:
 		Fn(const Fn<R, P1> &other) : FnBase(other) {
 			runtime::setVTable(this);
@@ -268,16 +322,16 @@ namespace storm {
 
 		// Call the function.
 		R call(P1 p1) const {
-			const TObject *first = asTObject(p1);
+			size_t firstInfo = categorizeFirst(p1);
 
-			if (needsCopy(first)) {
+			if (needsCopy(firstInfo) & 0x1) {
 				CloneEnv *env = new (this) CloneEnv();
 				FN_CLONE(P1, params, p1, env);
 				os::FnCall<R, 1> params = os::fnCall().add(c_p1);
-				return callRaw(params, first, env);
+				return callRaw(params, firstInfo, env);
 			} else {
 				os::FnCall<R, 1> params = os::fnCall().add(p1);
-				return callRaw(params, first, null);
+				return callRaw(params, firstInfo, null);
 			}
 		}
 	};
@@ -304,6 +358,11 @@ namespace storm {
 			runtime::setVTable(this);
 		}
 
+		template <class B>
+		Fn(R (CODECALL *ptr)(B *), B *obj, Thread *thread = null) : FnBase(address(ptr), obj, false, thread) {
+			runtime::setVTable(this);
+		}
+
 		// Copy:
 		Fn(const Fn<R> &other) : FnBase(other) {
 			runtime::setVTable(this);
@@ -313,7 +372,7 @@ namespace storm {
 		R call() const {
 			// Note: 'callRaw' will create a CloneEnv if it requires one in this case.
 			os::FnCall<R, 1> params = os::fnCall();
-			return callRaw(params, null, null);
+			return callRaw(params, 0, null);
 		}
 	};
 
@@ -322,22 +381,36 @@ namespace storm {
 	 * Create easily.
 	 */
 
+	// Free functions.
 	template <class R>
-	Fn<R> *fnPtr(Engine &e, R (*fn)(), Thread *t = null) {
+	Fn<R> *fnPtr(Engine &e, R (CODECALL *fn)(), Thread *t = null) {
 		return new (e) Fn<R>(fn, t);
 	}
 
 	template <class R, class P1>
-	Fn<R, P1> *fnPtr(Engine &e, R (*fn)(P1), Thread *t = null) {
+	Fn<R, P1> *fnPtr(Engine &e, R (CODECALL *fn)(P1), Thread *t = null) {
 		return new (e) Fn<R, P1>(fn, t);
 	}
 
 	template <class R, class P1, class P2>
-	Fn<R, P1, P2> *fnPtr(Engine &e, R (*fn)(P1, P2), Thread *t = null) {
+	Fn<R, P1, P2> *fnPtr(Engine &e, R (CODECALL *fn)(P1, P2), Thread *t = null) {
 		return new (e) Fn<R, P1, P2>(fn, t);
 	}
 
 
+	// Member functions without bound 'this':
+	template <class R, class Q>
+	Fn<R, Q*> *fnPtr(Engine &e, R (CODECALL Q::*fn)(), Thread *t = null) {
+		return new (e) Fn<R, Q*>(fn);
+	}
+
+	template <class R, class Q, class P>
+	Fn<R, Q*, P> *fnPtr(Engine &e, R (CODECALL Q::*fn)(), Thread *t = null) {
+		return new (e) Fn<R, Q*, P>(fn);
+	}
+
+
+	// Member functions with bound 'this':
 	template <class R, class Q>
 	Fn<R> *fnPtr(Engine &e, R (CODECALL Q::*fn)(), const Q *obj) {
 		return new (e) Fn<R>(fn, obj);
@@ -353,6 +426,59 @@ namespace storm {
 		return new (e) Fn<R, P1, P2>(fn, obj);
 	}
 
+
+	// Free functions, but bind first parameter:
+	template <class R, class B>
+	Fn<R> *fnBoundPtr(Engine &e, R (CODECALL *fn)(B *), B *bound, Thread *t = null) {
+		return new (e) Fn<R>(fn, bound, t);
+	}
+
+	template <class R, class B, class P1>
+	Fn<R, P1> *fnBoundPtr(Engine &e, R (CODECALL *fn)(B *, P1), B *bound, Thread *t = null) {
+		return new (e) Fn<R, P1>(fn, bound, t);
+	}
+
+	template <class R, class B, class P1, class P2>
+	Fn<R, P1, P2> *fnBoundPtr(Engine &e, R (CODECALL *fn)(B *, P1, P2), B *bound, Thread *t = null) {
+		return new (e) Fn<R, P1, P2>(fn, bound, t);
+	}
+
+	namespace impl {
+		template <class T, class P1>
+		class RawCtorTarget : public FnTarget {
+		public:
+			RawCtorTarget() {}
+
+			virtual void cloneTo(void *to, size_t size) const {
+				new (to) RawCtorTarget<T, P1>();
+			}
+			virtual const void *ptr() const {
+				return address(&RawCtorTarget<T, P1>::alloc);
+			}
+			virtual const void *ctor(const RootObject *thisPtr) const {
+				return address(&RawCtorTarget<T, P1>::init);
+			}
+			virtual void toS(StrBuf *to, const RootObject *thisPtr) const {
+				*to << S("C++ ctor");
+			}
+
+		private:
+			static T *CODECALL alloc(Engine *e, P1 p1) {
+				return new (*e) T(p1);
+			}
+			static void CODECALL init(void *out, P1 p1) {
+				new (Place(out)) T(p1);
+			}
+		};
+	}
+
+	// Create pointers to constructors in C++. Only supports 1 parameter at the moment, for
+	// serialization. More complex pointers can be created in Storm.
+	template <class T, class P1>
+	Fn<T *, P1> *ctorPtr(Engine &e, Thread *t = null) {
+		return new (e) Fn<T *, P1>(impl::RawCtorTarget<T, P1>(), (RootObject *)&e, FnBase::modeCtor);
+	}
+
 #undef FN_CLONE
 
 }
