@@ -43,6 +43,34 @@ namespace storm {
 		return Color(r, g, b, a);
 	}
 
+	Float toLinear(Float component) {
+		if (component <= 0.04045f) {
+			return component / 12.92f;
+		} else {
+			return std::pow((component + 0.055f) / 1.055f, 2.4f);
+		}
+	}
+
+	Color Color::toLinear() const {
+		return Color(storm::toLinear(r), storm::toLinear(g), storm::toLinear(b), storm::toLinear(a));
+	}
+
+	Float fromLinear(Float component) {
+		if (component <= 0.0031308f) {
+			return component * 12.92f;
+		} else {
+			return 1.055f * std::pow(component, 1.0f / 2.4f) - 0.055f;
+		}
+	}
+
+	Color Color::fromLinear() const {
+		return Color(storm::fromLinear(r), storm::fromLinear(g), storm::fromLinear(b), storm::fromLinear(a));
+	}
+
+	Float Color::brightness() const {
+		return (r + g + b) / 3.0f;
+	}
+
 	wostream &operator <<(wostream &to, const Color &s) {
 		return to << L"(" << s.r << L", " << s.g << L", " << s.b << L", " << s.a << L")";
 	}
