@@ -7,6 +7,8 @@ namespace storm {
 
 	/**
 	 * Color + alpha channel. Alpha goes from 0 (transparent) to 1 (opaque)
+	 *
+	 * TODO: always use linear color space?
 	 */
 	class Color {
 		STORM_VALUE;
@@ -39,10 +41,23 @@ namespace storm {
 		// Get a variant with a different alpha value.
 		Color STORM_FN withAlpha(Float f) const;
 
+		// Convert from sRGB to linear color space.
+		Color STORM_FN toLinear() const;
+
+		// Convert from linear color space to sRGB.
+		Color STORM_FN fromLinear() const;
+
+		// Get the brightness (ignoring alpha).
+		Float STORM_FN brightness() const;
+
 		// Output.
 		void STORM_FN toS(StrBuf *to) const;
 	};
 
+	// From/to linear color space for individual components.
+	Float STORM_FN toLinear(Float v);
+	Float STORM_FN fromLinear(Float v);
+
 	wostream &operator <<(wostream &to, const Color &s);
 
 	// Some default colors.
