@@ -94,4 +94,19 @@ namespace storm {
 		return data->v + offset(x, y);
 	}
 
+	void Image::vflip() {
+		Nat halfHeight = h / 2;
+		for (Nat y = 0; y < halfHeight; y++) {
+			for (Nat x = 0; x < w; x++) {
+				byte *a = buffer(x, y);
+				byte *b = buffer(x, h - y - 1);
+
+				swap(a[0], b[0]);
+				swap(a[1], b[1]);
+				swap(a[2], b[2]);
+				swap(a[3], b[3]);
+			}
+		}
+	}
+
 }
