@@ -15,14 +15,17 @@ namespace storm {
 		STORM_CTOR MeterOStream(OStream *to);
 
 		// Write.
-		virtual void STORM_FN write(Buffer buf, Nat start);
+		virtual Nat STORM_FN write(Buffer buf, Nat start);
 
 		// Flush the stream.
-		virtual void STORM_FN flush();
+		virtual Bool STORM_FN flush();
 
 		// Close.
 		virtual void STORM_FN close();
 
+		// Get error.
+		virtual sys::ErrorCode STORM_FN error() const;
+
 		// Get the current position.
 		Word STORM_FN tell();
 
