@@ -8,6 +8,7 @@
 #include "MemStream.h"
 #include "Serialization.h"
 #include "SerializationUtils.h"
+#include "OS/IORequest.h"
 
 #ifdef WINDOWS
 #include <Shlwapi.h>
@@ -78,7 +79,13 @@ namespace storm {
 	}
 
 	Bool Protocol::operator ==(const Protocol &o) const {
-		return runtime::typeOf(this) == runtime::typeOf(&o);
+		// This allows making some protocols considered the same as other protocols, without them
+		// having to know about each other.
+		return isEqualTo(&o) || o.isEqualTo(this);
+	}
+
+	Bool Protocol::isEqualTo(const Protocol *other) const {
+		return runtime::typeOf(this) == runtime::typeOf(other);
 	}
 
 	SerializedType *Protocol::serializedType(EnginePtr e) {
@@ -191,6 +198,29 @@ namespace storm {
 		from->end();
 	}
 
+	// Compare children. Only compares the last component of the URL. The remaining components are
+	// assumed to be equal.
+	static Bool childCompare(Url *lhs, Url *rhs) {
+		// Avoid crashing at least:
+		if (lhs->count() != rhs->count() || lhs->empty() || rhs->empty())
+			return lhs->count() < rhs->count();
+
+		Nat lastIndex = lhs->count() - 1;
+		return *lhs->at(lastIndex) < *rhs->at(lastIndex);
+	}
+
+	// Helper to sort child files/directories. Used to return a consistent ordering for files and
+	// directories, so that dependency problems in the compiler behave deterministically between
+	// different filesystems at least.
+	static void sortChildren(Array<Url *> *children) {
+		if (children->empty())
+			return;
+
+		Url **first = &children->at(0);
+		Url **last = first + children->count();
+		std::sort(first, last, childCompare);
+	}
+
 #ifdef WINDOWS
 
 	Bool FileProtocol::partEq(Str *a, Str *b) {
@@ -252,6 +282,8 @@ namespace storm {
 
 		FindClose(h);
 
+		sortChildren(result);
+
 		return result;
 	}
 
@@ -342,6 +374,8 @@ namespace storm {
 
 		closedir(h);
 
+		sortChildren(result);
+
 		return result;
 	}
 
@@ -353,6 +387,30 @@ namespace storm {
 		return new (url) FileOStream(format(url));
 	}
 
+#if defined(LINUX_IO_URING)
+
+	StatType FileProtocol::stat(Url *url) {
+		const char *path = format(url)->utf8_str();
+		struct statx stat;
+
+		os::IORequest r(os::Handle(AT_FDCWD), os::Thread::current());
+		r.request.opcode = IORING_OP_STATX;
+		r.request.addr = reinterpret_cast<size_t>(path);
+		r.request.statx_flags = AT_STATX_SYNC_AS_STAT;
+		r.request.len = STATX_MODE;
+		r.request.off = reinterpret_cast<size_t>(&stat);
+
+		int ok = r.submit();
+		if (ok != 0)
+			return sNotFound;
+		else if (S_ISDIR(stat.stx_mode))
+			return sDirectory;
+		else
+			return sFile;
+	}
+
+#else
+
 	StatType FileProtocol::stat(Url *url) {
 		struct stat s;
 		if (::stat(format(url)->utf8_str(), &s) != 0)
@@ -364,6 +422,8 @@ namespace storm {
 			return sFile;
 	}
 
+#endif
+
 	Bool FileProtocol::createDir(Url *url) {
 		return mkdir(format(url)->utf8_str(), 0777) == 0;
 	}
@@ -386,59 +446,6 @@ namespace storm {
 #endif
 
 
-	/**
-	 * HTTP/HTTPS
-	 */
-
-	HttpProtocol::HttpProtocol(Bool secure) : secure(secure) {}
-
-	Bool HttpProtocol::partEq(Str *a, Str *b) {
-		return *a == *b;
-	}
-
-	Nat HttpProtocol::partHash(Str *a) {
-		return a->hash();
-	}
-
-	void HttpProtocol::toS(StrBuf *to) const {
-		if (secure)
-			*to << S("https://");
-		else
-			*to << S("http://");
-	}
-
-	Bool HttpProtocol::operator ==(const Protocol &o) const {
-		if (!Protocol::operator ==(o))
-			return false;
-
-		const HttpProtocol &other = (const HttpProtocol &)o;
-		return secure == other.secure;
-	}
-
-	SerializedType *HttpProtocol::serializedType(EnginePtr e) {
-		SerializedStdType *type = serializedStdType<HttpProtocol, Protocol>(e.v);
-		type->add(S("secure"), StormInfo<Bool>::type(e.v));
-		return type;
-	}
-
-	HttpProtocol *HttpProtocol::read(ObjIStream *from) {
-		return (HttpProtocol *)from->readClass(StormInfo<HttpProtocol>::type(from->engine()));
-	}
-
-	void HttpProtocol::write(ObjOStream *to) {
-		if (to->startClass(StormInfo<HttpProtocol>::type(engine()), this)) {
-			Protocol::write(to);
-			Serialize<Bool>::write(secure, to);
-			to->end();
-		}
-	}
-
-	HttpProtocol::HttpProtocol(ObjIStream *from) : Protocol(from) {
-		secure = Serialize<Bool>::read(from);
-		from->end();
-	}
-
-
 	MemoryProtocol::MemoryProtocol() {
 		files = new (this) Map<Str *, Buffer>();
 	}
@@ -517,6 +524,7 @@ namespace storm {
 	MemoryProtocol::MemoryProtocol(ObjIStream *from) {
 		// Note: Serialization does not save files in the memory protocol, just like paths are not
 		// transferred between different systems.
+		files = new (this) Map<Str *, Buffer>();
 		from->end();
 	}
 
