@@ -223,19 +223,14 @@ namespace storm {
 	};
 
 	// C++ helper:
-	template <class Type>
-	void CODECALL callReadCtor(Type *to, ObjIStream *from) {
-		new (Place(to)) Type(from);
-	}
-
 	template <class Type>
 	SerializedStdType *serializedStdType(Engine &e) {
-		return new (e) SerializedStdType(StormInfo<Type>::type(e), fnPtr(e, callReadCtor<Type>));
+		return new (e) SerializedStdType(StormInfo<Type>::type(e), ctorPtr<Type, ObjIStream *>(e));
 	}
 
 	template <class Type, class Parent>
 	SerializedStdType *serializedStdType(Engine &e) {
-		return new (e) SerializedStdType(StormInfo<Type>::type(e), fnPtr(e, callReadCtor<Type>), StormInfo<Parent>::type(e));
+		return new (e) SerializedStdType(StormInfo<Type>::type(e), ctorPtr<Type, ObjIStream *>(e), StormInfo<Parent>::type(e));
 	}
 
 
@@ -627,8 +622,10 @@ namespace storm {
 		// Read an object into a variant.
 		Variant readObject(Nat type);
 
-		// Read a value (internal version, does not call 'start').
-		void readValueI(Desc *type, void *out);
+		// Read a value (internal version, does not call 'start'). 'objectPtr' states if 'out' is
+		// really a pointer to Object *, rather than a pointer to a value. This happens when we try
+		// to turn values into objects during deserialization for example.
+		void readValueI(Desc *type, void *out, Bool objectPtr);
 
 		// Read a class (internal version, does not call 'start').
 		Object *readClassI(Desc *type, Type *t);
