@@ -44,6 +44,8 @@ namespace storm {
 			c = readChar();
 		if (c == Char(Nat(0)))
 			eof = true;
+		else
+			eof = false;
 		return c;
 	}
 
@@ -123,7 +125,7 @@ namespace storm {
 	Str *TextInput::readAllRaw() {
 		StrBuf *to = new (this) StrBuf();
 
-		while (more()) {
+		while (true) {
 			Char c = readRaw();
 			if (c == Char(Nat(0)))
 				break;
@@ -135,11 +137,6 @@ namespace storm {
 
 	void TextInput::close() {}
 
-	// Char TextInput::readChar() {
-	// 	return Char(nat(0));
-	// }
-
-
 	TextInput *STORM_FN readText(IStream *stream) {
 		nat16 bom = 0;
 
@@ -166,6 +163,10 @@ namespace storm {
 		return new (from) StrInput(from);
 	}
 
+	Str *STORM_FN readAllText(IStream *stream) {
+		return readText(stream)->readAll();
+	}
+
 	Str *STORM_FN readAllText(Url *file) {
 		IStream *src = file->read();
 		Str *result = readText(src)->readAll();
@@ -221,8 +222,6 @@ namespace storm {
 		}
 	}
 
-	// void TextOutput::writeChar(Char ch) {}
-
 	void TextOutput::flush() {}
 
 	void TextOutput::close() {}
