@@ -236,12 +236,36 @@ namespace storm {
 	// already present.
 	Url *STORM_FN userConfigUrl(Str *appName);
 
-	// Some exceptions.
-	class EXCEPTION_EXPORT InvalidName : public Exception {
+
+	/**
+	 * Exceptions from Url:s.
+	 */
+	class EXCEPTION_EXPORT UrlError : public Exception {
 		STORM_EXCEPTION;
 	public:
-		STORM_CTOR InvalidName() : name(null) {}
-		STORM_CTOR InvalidName(Str *name) : name(name) {}
+		STORM_CTOR UrlError(Str *msg) : msg(msg) {}
+
+		virtual void STORM_FN message(StrBuf *to) const {
+			*to << S("Url error: ") << msg;
+		}
+
+	protected:
+		STORM_CTOR UrlError() {
+			msg = new (this) Str(S(""));
+		}
+
+		Str *msg;
+	};
+
+
+	/**
+	 * Invalid name, thrown by Url.
+	 */
+	class EXCEPTION_EXPORT InvalidName : public UrlError {
+		STORM_EXCEPTION;
+	public:
+		STORM_CTOR InvalidName() : UrlError(), name(null) {}
+		STORM_CTOR InvalidName(Str *name) : UrlError(), name(name) {}
 
 		virtual void STORM_FN message(StrBuf *to) const {
 			if (!name)
@@ -249,20 +273,31 @@ namespace storm {
 			else
 				*to << S("The url part ") << name << S(" is not acceptable.");
 		}
+
 	private:
 		MAYBE(Str *) name;
 	};
 
-	class EXCEPTION_EXPORT UrlError : public Exception {
+
+	/**
+	 * Failed to open a stream.
+	 */
+	class EXCEPTION_EXPORT OpenError : public UrlError {
 		STORM_EXCEPTION;
 	public:
-		STORM_CTOR UrlError(Str *name) : name(name) {}
+		STORM_CTOR OpenError(Str *error, Str *file, Bool input) : UrlError(error), file(file), input(input) {}
 
 		virtual void STORM_FN message(StrBuf *to) const {
-			*to << S("Url error: ") << name;
+			*to << S("Failed to open: '")
+				<< file
+				<< S("' for ")
+				<< (input ? S("reading") : S("writing"))
+				<< S(": ") << msg;
 		}
 
 	private:
-		Str *name;
+		Str *file;
+		Bool input;
 	};
+
 }
