@@ -1,6 +1,7 @@
 #include "stdafx.h"
 #include "Net.h"
 #include "OS/IORequest.h"
+#include "Core/Io/HandleStream.h"
 
 namespace storm {
 
@@ -192,6 +193,95 @@ namespace storm {
 	}
 
 
+#elif defined(LINUX_IO_URING)
+
+	static os::Handle createSocket(int af, int mode, int proto) {
+		return socket(af, mode | SOCK_NONBLOCK | SOCK_CLOEXEC, proto);
+	}
+
+	bool bindSocket(os::Handle socket, const sockaddr *addr, socklen_t addrSize) {
+		return bind(socket.v(), addr, addrSize) == 0;
+	}
+
+	bool listenSocket(os::Handle socket, int backlog) {
+		return listen(socket.v(), backlog) == 0;
+	}
+
+	static bool setNonblocking(int fd) {
+		int flags = fcntl(fd, F_GETFL, 0);
+
+		if (flags == -1)
+			return false;
+
+		if (fcntl(fd, F_SETFL, flags | O_NONBLOCK | O_CLOEXEC) == -1)
+			return false;
+
+		return true;
+	}
+
+	os::Handle acceptSocket(os::Handle socket, const os::Thread &attached, sockaddr *addr, socklen_t addrSize) {
+		while (true) {
+			os::IORequest r(socket, attached);
+			r.request.opcode = IORING_OP_ACCEPT;
+			r.request.addr = reinterpret_cast<size_t>(addr);
+			r.request.addr2 = reinterpret_cast<size_t>(&addrSize);
+
+			int result = r.submit();
+			if (result >= 0) {
+				setNonblocking(result);
+				return os::Handle(result);
+			} else {
+				// Error from accept.
+				break;
+			}
+		}
+		return os::Handle();
+	}
+
+	bool connectSocket(os::Handle socket, const os::Thread &attached, sockaddr *addr, socklen_t addrSize) {
+		os::IORequest r(socket, attached);
+		r.request.opcode = IORING_OP_CONNECT;
+		r.request.addr = reinterpret_cast<size_t>(addr);
+		r.request.off = static_cast<size_t>(addrSize);
+
+		int ok = r.submit();
+		return ok == 0;
+	}
+
+	void closeSocket(os::Handle handle, const os::Thread &attached) {
+		os::Thread t = attached;
+		storm::close(handle, t);
+	}
+
+	bool getSocketName(os::Handle socket, sockaddr *out, socklen_t size) {
+		return getsockname(socket.v(), out, &size) == 0;
+	}
+
+	bool getSocketOpt(os::Handle socket, int level, int name, void *value, socklen_t size) {
+		return getsockopt(socket.v(), level, name, value, &size) == 0;
+	}
+
+	bool setSocketOpt(os::Handle socket, int level, int name, void *value, socklen_t size) {
+		return setsockopt(socket.v(), level, name, value, size) == 0;
+	}
+
+	Duration getSocketTime(os::Handle socket, int level, int name) {
+		struct timeval tv;
+		memset(&tv, 0, sizeof(tv));
+		if (!getSocketOpt(socket, level, name, &tv, sizeof(tv)))
+			return Duration();
+
+		return time::us(tv.tv_usec) + time::s(tv.tv_sec);
+	}
+
+	bool setSocketTime(os::Handle socket, int level, int name, const Duration &d) {
+		struct timeval tv;
+		tv.tv_usec = d.inUs() % 1000000;
+		tv.tv_sec = d.inS();
+
+		return setSocketOpt(socket, level, name, &tv, sizeof(tv));
+	}
+
 #elif defined(POSIX)
 
 	static os::Handle createSocket(int af, int mode, int proto) {
@@ -231,7 +321,7 @@ namespace storm {
 			if (fd >= 0) {
 				if (!setNonblocking(fd)) {
 					perror("fcntl");
-					close(fd);
+					::close(fd);
 					return os::Handle();
 				}
 
@@ -300,7 +390,7 @@ namespace storm {
 	void closeSocket(os::Handle handle, const os::Thread &attached) {
 		if (attached != os::Thread::invalid)
 			attached.detach(handle);
-		close(handle.v());
+		::close(handle.v());
 	}
 
 	bool getSocketName(os::Handle socket, sockaddr *out, socklen_t size) {
