@@ -110,6 +110,11 @@ namespace storm {
 #error "TODO: Implement keepalive for your OS."
 #endif
 
+	void NetStream::close() {
+		i->close();
+		o->close();
+	}
+
 	void NetStream::closeEnd(Nat which) {
 		Nat old, w;
 		do {
@@ -177,7 +182,11 @@ namespace storm {
 	 * IStream.
 	 */
 
-	NetIStream::NetIStream(NetStream *owner, os::Thread t) : HandleTimeoutIStream(owner->handle, t), owner(owner) {}
+	NetIStream::NetIStream(NetStream *owner, const os::Thread &t)
+		: HandleTimeoutIStream(owner->handle, t), owner(owner) {
+		skipClose();
+		skipPos();
+	}
 
 	NetIStream::~NetIStream() {
 		// The socket will close the handle.
@@ -185,8 +194,8 @@ namespace storm {
 	}
 
 	void NetIStream::close() {
-		owner->closeEnd(NetStream::closeRead);
 		handle = os::Handle();
+		owner->closeEnd(NetStream::closeRead);
 	}
 
 
@@ -194,7 +203,11 @@ namespace storm {
 	 * OStream.
 	 */
 
-	NetOStream::NetOStream(NetStream *owner, os::Thread t) : HandleOStream(owner->handle, t), owner(owner) {}
+	NetOStream::NetOStream(NetStream *owner, const os::Thread &t)
+		: HandleTimeoutOStream(owner->handle, t), owner(owner) {
+		skipClose();
+		skipPos();
+	}
 
 	NetOStream::~NetOStream() {
 		// The socket will close the handle.
@@ -202,8 +215,8 @@ namespace storm {
 	}
 
 	void NetOStream::close() {
-		owner->closeEnd(NetStream::closeWrite);
 		handle = os::Handle();
+		owner->closeEnd(NetStream::closeWrite);
 	}
 
 
