@@ -40,8 +40,15 @@ namespace storm {
 			Nat offset;
 
 			// ID of the backend producing this frame.
+			// If the ID is 'nextThread', this frame indicates that the remainder of the stack trace
+			// indicates the frames of the thread that waits for the initial thread to complete.
+			// This is to make it easier to trace the execution through thread switches. This data
+			// is added after the fact by the Future implementation.
 			Nat id;
 
+			// ID to use for "next thread":
+			static Nat STORM_FN nextThread() { return Nat(-1); }
+
 			// Compute the pointer.
 			inline void *ptr() const {
 				return (byte *)base + offset;
@@ -85,4 +92,7 @@ namespace storm {
 	// Collect a stack trace, passing a pointer to previously collected stack state.
 	StackTrace collectStackTrace(EnginePtr e, void *state);
 
+	// Append a stack trace to 'StackTrace' from the current thread. Inserts a "next thread" frame in between.
+	void appendStackTrace(StackTrace &to, Nat ignore);
+
 }
