@@ -85,7 +85,9 @@ namespace storm {
 #endif
 
 	Str::Str(const wchar *from, const wchar *to) {
-		nat count = nat(to - from);
+		nat count = 0;
+		if (from < to)
+			count = nat(to - from);
 		allocData(count + 1);
 		for (nat i = 0; i < count; i++) {
 			data->v[i] = from[i];
@@ -282,122 +284,108 @@ namespace storm {
 		return r;
 	}
 
-	Bool Str::isInt() const {
-		Nat start = 0;
-		if (data->v[0] == '-')
-			start++;
-		for (Nat i = start; i < data->count - 1; i++)
-			if (data->v[i] < '0' || data->v[i] > '9')
-				return false;
-		return true;
-	}
-
-	Bool Str::isNat() const {
-		for (nat i = 0; i < data->count - 1; i++)
-			if (data->v[i] < '0' || data->v[i] > '9')
-				return false;
-		return true;
-	}
-
-	Bool Str::isFloat() const {
-		Nat at = 0;
-		if (data->v[0] == '-')
-			at++;
-
-		Bool hasDecimal = false;
-		while (at < data->count - 1) {
-			if (data->v[at] == '.' && !hasDecimal) {
-				hasDecimal = true;
-			} else if (data->v[at] >= '0' && data->v[at] <= '9') {
-				// Number, OK
-			} else if (data->v[at] == 'e' || data->v[at] == 'E') {
-				break;
-			} else {
-				return false;
-			}
-
-			at++;
-		}
-
-		if (at == data->count - 1)
-			return true;
-
-		// If not at end already, it must be the exponential part:
-		if (data->v[at] != 'e' && data->v[at] != 'E')
-			return false;
-
-		if (data->v[at] == '-')
-			at++;
-
-		while (at < data->count - 1) {
-			if (data->v[at] < '0' || data->v[at] > '9')
-				return false;
-			at++;
-		}
-
-		return true;
-	}
-
-	Int Str::toInt() const {
+	Maybe<Int> Str::asInt() const {
 		wchar *end;
 		Int r = wcstol(data->v, &end, 10);
 		if (end != data->v + data->count - 1)
-			throw new (this) StrError(S("Not a number"));
-		return r;
+			return Maybe<Int>();
+		return Maybe<Int>(r);
 	}
 
-	Nat Str::toNat() const {
+	Int Str::toInt() const {
+		Maybe<Int> r = asInt();
+		if (r.empty())
+			throw new (this) StrError(S("Not a number"));
+		return r.value();
+	}
+
+	Maybe<Nat> Str::asNat() const {
 		wchar *end;
 		Nat r = wcstoul(data->v, &end, 10);
 		if (end != data->v + data->count - 1)
-			throw new (this) StrError(S("Not a number"));
-		return r;
+			return Maybe<Nat>();
+		return Maybe<Nat>(r);
 	}
 
-	Long Str::toLong() const {
+	Nat Str::toNat() const {
+		Maybe<Nat> r = asNat();
+		if (r.empty())
+			throw new (this) StrError(S("Not a number"));
+		return r.value();
+	}
+
+	Maybe<Long> Str::asLong() const {
 		wchar *end;
 		Long r = wcstoll(data->v, &end, 10);
 		if (end != data->v + data->count - 1)
-			throw new (this) StrError(S("Not a number"));
-		return r;
+			return Maybe<Long>();
+		return Maybe<Long>(r);
 	}
 
-	Word Str::toWord() const {
+	Long Str::toLong() const {
+		Maybe<Long> r = asLong();
+		if (r.empty())
+			throw new (this) StrError(S("Not a number"));
+		return r.value();
+	}
+
+	Maybe<Word> Str::asWord() const {
 		wchar *end;
 		Word r = wcstoull(data->v, &end, 10);
 		if (end != data->v + data->count - 1)
-			throw new (this) StrError(S("Not a number"));
-		return r;
+			return Maybe<Word>();
+		return Maybe<Word>(r);
 	}
 
-	Float Str::toFloat() const {
+	Word Str::toWord() const {
+		Maybe<Word> r = asWord();
+		if (r.empty())
+			throw new (this) StrError(S("Not a number"));
+		return r.value();
+	}
+
+	Maybe<Float> Str::asFloat() const {
 		Float r;
 		StdIBuf<100> buf(data->v, nat(data->count - 1));
 		std::wistream input(&buf);
 		input.imbue(std::locale::classic());
 		if (!(input >> r))
-			throw new (this) StrError(S("Not a floating-point number"));
+			return Maybe<Float>();
 
 		wchar_t probe;
 		if (input >> probe)
-			throw new (this) StrError(S("Not a floating-point number"));
+			return Maybe<Float>();
 
-		return r;
+		return Maybe<Float>(r);
 	}
 
-	Double Str::toDouble() const {
+	Float Str::toFloat() const {
+		Maybe<Float> r = asFloat();
+		if (r.empty())
+			throw new (this) StrError(S("Not a floating-point number"));
+		return r.value();
+	}
+
+	Maybe<Double> Str::asDouble() const {
 		Double r;
 		StdIBuf<100> buf(data->v, nat(data->count - 1));
 		std::wistream input(&buf);
 		input.imbue(std::locale::classic());
 		if (!(input >> r))
-			throw new (this) StrError(S("Not a floating-point number"));
+			return Maybe<Double>();
 
 		wchar_t probe;
 		if (input >> probe)
-			throw new (this) StrError(S("Not a floating-point number"));
+			return Maybe<Double>();
 
-		return r;
+		return Maybe<Double>(r);
+	}
+
+	Double Str::toDouble() const {
+		Maybe<Double> r = asDouble();
+		if (r.empty())
+			throw new (this) StrError(S("Not a floating-point number"));
+		return r.value();
 	}
 
 	static inline int hexDigit(wchar ch) {
@@ -410,27 +398,46 @@ namespace storm {
 		return -1;
 	}
 
-	Bool Str::isHex() const {
-		for (Nat i = 0; i < data->count - 1; i++)
-			if (hexDigit(data->v[i]) < 0)
-				return false;
-		return true;
+	Maybe<Nat> Str::asHexNat() const {
+		Nat r = 0;
+		for (nat i = 0; i < data->count - 1; i++) {
+			wchar ch = data->v[i];
+			int digit = hexDigit(ch);
+			if (digit < 0)
+				return Maybe<Nat>();
+			if (r >> (32 - 4))
+				return Maybe<Nat>();
+			r = (r << 4) | Nat(digit);
+		}
+		return Maybe<Nat>(r);
 	}
 
 	Nat Str::hexToNat() const {
-		return Nat(hexToWord());
+		Maybe<Nat> r = asHexNat();
+		if (r.empty())
+			throw new (this) StrError(S("Not a hexadecimal number"));
+		return r.value();
 	}
 
-	Word Str::hexToWord() const {
+	Maybe<Word> Str::asHexWord() const {
 		Word r = 0;
 		for (nat i = 0; i < data->count - 1; i++) {
 			wchar ch = data->v[i];
 			int digit = hexDigit(ch);
 			if (digit < 0)
-				throw new (this) StrError(S("Not a hexadecimal number"));
-			r = (r << 4) | Nat(digit);
+				return Maybe<Word>();
+			if (r >> (64 - 4))
+				return Maybe<Word>();
+			r = (r << 4) | Word(digit);
 		}
-		return r;
+		return Maybe<Word>(r);
+	}
+
+	Word Str::hexToWord() const {
+		Maybe<Word> r = asHexWord();
+		if (r.empty())
+			throw new (this) StrError(S("Not a hexadecimal number"));
+		return r.value();
 	}
 
 	template <bool useEscape>
@@ -452,6 +459,14 @@ namespace storm {
 			*out++ = '\v';
 			src++;
 			return true;
+		case 'b':
+			*out++ = '\b';
+			src++;
+			return true;
+		case 'f':
+			*out++ = '\f';
+			src++;
+			return true;
 		case '0':
 			*out++ = '\0';
 			src++;
@@ -566,6 +581,12 @@ namespace storm {
 		} else if (ch == Char('\v')) {
 			*to << L"\\v";
 			return true;
+		} else if (ch == Char('\b')) {
+			*to << L"\\b";
+			return true;
+		} else if (ch == Char('\f')) {
+			*to << L"\\f";
+			return true;
 		} else if (ch == Char('\\')) {
 			*to << L"\\\\";
 			return true;
@@ -639,6 +660,10 @@ namespace storm {
 		return true;
 	}
 
+	Bool Str::contains(const Str *s) const {
+		return find(s) != end();
+	}
+
 	static Nat toCrLfHelp(GcArray<wchar> *src, GcArray<wchar> *dest) {
 		Nat pos = 0;
 		for (Nat i = 0; i + 1 < src->count; i++) {
@@ -742,7 +767,7 @@ namespace storm {
 	}
 
 	Str::Iter Str::end() const {
-		return Iter();
+		return Iter(this, Nat(data->count - 1));
 	}
 
 	Str::Iter Str::posIter(Nat pos) const {
@@ -792,11 +817,11 @@ namespace storm {
 		return pos;
 	}
 
-	Str::Iter Str::find(Str *str) const {
+	Str::Iter Str::find(const Str *str) const {
 		return find(str, begin());
 	}
 
-	Str::Iter Str::find(Str *str, Iter start) const {
+	Str::Iter Str::find(const Str *str, Iter start) const {
 		Iter sFirst = str->begin();
 		Iter sLast = str->end();
 		Char firstCh = sFirst.v();
@@ -857,11 +882,11 @@ namespace storm {
 		return end();
 	}
 
-	Str::Iter Str::findLast(Str *str) const {
+	Str::Iter Str::findLast(const Str *str) const {
 		return findLast(str, end());
 	}
 
-	Str::Iter Str::findLast(Str *str, Iter last) const {
+	Str::Iter Str::findLast(const Str *str, Iter last) const {
 		if (!data)
 			return end();
 		if (last.owner && last.owner != this)
@@ -1271,6 +1296,10 @@ namespace storm {
 				end = at + 1;
 		}
 
+		// If the string *only* consisted of whitespace, then begin > end which will cause trouble.
+		if (begin > end)
+			end = begin;
+
 		return new (str) Str(begin, end);
 	}
 
