@@ -2,6 +2,7 @@
 #include "Object.h"
 #include "Char.h"
 #include "GcArray.h"
+#include "Maybe.h"
 
 namespace storm {
 	STORM_PKG(core);
@@ -66,15 +67,6 @@ namespace storm {
 		// Hash.
 		Nat STORM_FN hash() const;
 
-		// Does the string contain a signed integer? Does not validate the size of the number.
-		Bool STORM_FN isInt() const;
-
-		// Does the string contain an unsigned number? Does not validate the size of the number.
-		Bool STORM_FN isNat() const;
-
-		// Does the string contain a floating point number?
-		Bool STORM_FN isFloat() const;
-
 		// Convert to a number. Throws `StrError` on error.
 		Int STORM_FN toInt() const;
 		Nat STORM_FN toNat() const;
@@ -83,13 +75,20 @@ namespace storm {
 		Float STORM_FN toFloat() const;
 		Double STORM_FN toDouble() const;
 
-		// Is it a hexadecimal number? Does not validate the size of the number.
-		Bool STORM_FN isHex() const;
-
 		// Interpret as a hexadecimal number. Throws `StrError` on error.
 		Nat STORM_FN hexToNat() const;
 		Word STORM_FN hexToWord() const;
 
+		// Versions of the above that return a Maybe<T>.
+		Maybe<Int> STORM_FN STORM_NAME(asInt, int)() const;
+		Maybe<Nat> STORM_FN STORM_NAME(asNat, nat)() const;
+		Maybe<Long> STORM_FN STORM_NAME(asLong, long)() const;
+		Maybe<Word> STORM_FN STORM_NAME(asWord, word)() const;
+		Maybe<Float> STORM_FN STORM_NAME(asFloat, float)() const;
+		Maybe<Double> STORM_FN STORM_NAME(asDouble, double)() const;
+		Maybe<Nat> STORM_FN STORM_NAME(asHexNat, hexNat)() const;
+		Maybe<Word> STORM_FN STORM_NAME(asHexWord, hexWord)() const;
+
 		// Escape/unescape characters. Any unknown escape sequences are kept as they are. The
 		// parameters `extra` and `extra` are additional characters that should be escaped/unescaped
 		// if present.
@@ -113,6 +112,10 @@ namespace storm {
 		Bool STORM_FN endsWith(const Str *s) const;
 		Bool endsWith(const wchar *s) const;
 
+		// Does the string contain the substring `s`? Note that the implementation is not
+		// necessarily efficient for long search strings.
+		Bool STORM_FN contains(const Str *s) const;
+
 		// Deep copy (nothing needs to be done really).
 		virtual void STORM_FN deepCopy(CloneEnv *env);
 
@@ -226,8 +229,8 @@ namespace storm {
 
 		// Find a substring in the string. Returns the first match. Note: this approach is not
 		// necessarily optimal for long search strings.
-		Iter STORM_FN find(Str *str) const;
-		Iter STORM_FN find(Str *str, Iter start) const;
+		Iter STORM_FN find(const Str *str) const;
+		Iter STORM_FN find(const Str *str, Iter start) const;
 
 		// Find the last occurrence of `ch` in the string. Note that 'last' is *not* examined.
 		Iter STORM_FN findLast(Char ch) const;
@@ -235,8 +238,8 @@ namespace storm {
 
 		// Find the last occurrence of `str` in the string. Note that the match has to end before
 		// `last` if specified.
-		Iter STORM_FN findLast(Str *str) const;
-		Iter STORM_FN findLast(Str *str, Iter last) const;
+		Iter STORM_FN findLast(const Str *str) const;
+		Iter STORM_FN findLast(const Str *str, Iter last) const;
 
 		// Read/write (raw).
 		void STORM_FN write(OStream *to) const;
