@@ -1,5 +1,6 @@
 #pragma once
 #include "Object.h"
+#include "EnginePtr.h"
 
 namespace storm {
 	STORM_PKG(core);
@@ -49,6 +50,15 @@ namespace storm {
 		bool sameAs(const os::Thread &other) const;
 		bool sameAs(size_t id) const;
 
+		// Check if the caller of the function is the thread represented by this object.
+		Bool STORM_FN isCurrent() const;
+
+		// Get the current thread as a Thread object.
+		static Thread *STORM_FN current(EnginePtr e);
+
+		// To string.
+		virtual void STORM_FN toS(StrBuf *to) const;
+
 #ifdef STORM_COMPILER
 		/**
 		 * Allow stand-alone allocation of the first Thread.
