@@ -84,7 +84,7 @@ namespace storm {
 				*to << S(", ");
 			first = false;
 
-			*to << o;
+			*to << threadSafeToS(o);
 		}
 
 		*to << S("}");
@@ -211,6 +211,32 @@ namespace storm {
 		return c;
 	}
 
+	void WeakSetBase::dbg_print() {
+		std::wcout << L"Weak set contents:" << endl;
+		for (nat i = 0; i < capacity(); i++) {
+			std::wcout << std::setw(2) << i << L": ";
+			if (info->v[i].status == Info::free) {
+				std::wcout << L"free";
+			} else if (info->v[i].status == Info::end) {
+				std::wcout << toHex(info->v[i].hash) << L" end";
+			} else {
+				std::wcout << toHex(info->v[i].hash) << L" -> " << info->v[i].status;
+			}
+
+			if (info->v[i].status != Info::free) {
+				std::wcout << "   ";
+
+				RootObject *o = data->read(i);
+				if (runtime::liveObject(o)) {
+					std::wcout << o;
+				} else {
+					std::wcout << "(ptr to dead object)";
+				}
+			}
+			std::wcout << endl;
+		}
+	}
+
 	void WeakSetBase::alloc(nat cap) {
 		assert(info == null);
 		assert(data == null);
