@@ -640,10 +640,18 @@ static void parseNamespace(Tokenizer &tok, ParseEnv &env, const CppName &name) {
 			tok.skip();
 			parseEnum(tok, env, name);
 		} else if (t.token == L"template") {
-			// Skip until we find a {, and skip the body as well.
-			while (!tok.skipIf(L"{"))
+			// Skip until we find a { or a ;
+			while (true) {
+				if (tok.skipIf(L"{")) {
+					// Skip the body as well.
+					parseBlock(tok);
+					break;
+				} else if (tok.skipIf(L";")) {
+					// Fwd decl, we are done.
+					break;
+				}
 				tok.skip();
-			parseBlock(tok);
+			}
 		} else if (t.token == L"extern") {
 			tok.skip();
 			tok.skipIf(L"\"C\""); // for 'extern "C" {}'
