@@ -152,10 +152,15 @@ void World::resolveTypes() {
 	Auto<Doc> copy = new Doc(L"Copy constructor.");
 	Auto<Doc> assign = new Doc(L"Assignment operator.");
 
+	// Resolve types first.
 	for (nat i = 0; i < types.size(); i++) {
 		Type *t = types[i].borrow();
 		t->resolveTypes(*this);
+	}
 
+	// Now we can check if types are actors etc.
+	for (nat i = 0; i < types.size(); i++) {
+		Type *t = types[i].borrow();
 		if (Class *c = as<Class>(t)) {
 			// Add the default copy-constructor to the type unless it is an actor.
 			if (!c->isActor() && !c->external) {
