@@ -448,6 +448,13 @@ namespace ssl {
 		} else if (status == SEC_I_CONTEXT_EXPIRED) {
 			// The remote end shut the connection down.
 			m.shutdown = true;
+
+			// Consume the message so that we don't try to process it again later, if the user tries
+			// to read from the connection even though it is closed. Cursory testing indicates that
+			// the DecryptMessage function does not actually give an indication of the size of the
+			// end of stream message. Rather, it gives an error if it is too large. As such, we just
+			// assume that it consumed all data.
+			m.remainingStart = data.filled();
 		} else {
 			throwError(e, S("Failed to decrypt a message: "), status);
 		}
@@ -506,11 +513,14 @@ namespace ssl {
 		if (data->incomingEnd)
 			return false;
 
-		// Otherwise, see if we have more data to decrypt, or if there is more in the source.
+		// Otherwise, see if we have more data coming from the source stream.
+		return data->src->more();
+
+		// Note: We could check if we have more data in the input buffer. However, even if we have
+		// more data in the input buffer but the stream reports that no more data is available, we
+		// won't be able to decrypt it anyway.
 		return (data->remainingStart < data->incoming.filled()) // more encrypted data?
 			|| data->src->more(); // more from the source stream?
-
-		return true;
 	}
 
 	void SChannelSession::read(Buffer &to, void *gcData) {
@@ -588,6 +598,11 @@ namespace ssl {
 		// In case we fail somewhere...
 		data->decryptedStart = data->decryptedEnd = 0;
 
+		// Are we shutdown? If so, stop trying to decrypt even if there is more data in the stream.
+		// That will only produce errors.
+		if (data->incomingEnd)
+			return;
+
 		// Anything left in the buffer?
 		if (data->remainingStart >= data->incoming.filled()) {
 			// Nope. Get more. But this means that the buffer is entirely empty, use that.
