@@ -8,6 +8,18 @@ namespace ssl {
 
 #define SET_ERROR(MSG) case MSG: msg = WIDEN(#MSG); break
 
+	// Note: We can use FormatMessage for this:
+	// FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER|
+	// 			FORMAT_MESSAGE_FROM_SYSTEM|
+	// 			FORMAT_MESSAGE_IGNORE_INSERTS,
+	// 			NULL,
+	// 			status, <-- here
+	// 			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
+	// 			(LPTSTR)&message, <-- will be set
+	// 			0,
+	// 			NULL);
+
+
 	void throwError(Engine &e, const wchar *error, SECURITY_STATUS status) {
 		const wchar *msg = null;
 		switch (status) {
@@ -31,6 +43,7 @@ namespace ssl {
 			SET_ERROR(SEC_E_WRONG_PRINCIPAL);
 			SET_ERROR(SEC_E_ILLEGAL_MESSAGE);
 			SET_ERROR(SEC_E_ALGORITHM_MISMATCH);
+			SET_ERROR(SEC_E_DECRYPT_FAILURE);
 			SET_ERROR(ERROR_BUSY);
 			SET_ERROR(ERROR_INVALID_HANDLE);
 			SET_ERROR(ERROR_INVALID_PARAMETER);
