@@ -93,7 +93,7 @@ namespace storm {
 										typename Scanner::Source &source);
 
 			// Scan all blocks that may contain references to anything in the provided GenSet,
-			// possibly raising write barriers since a garbage collection sycle is almost over.
+			// possibly raising write barriers since a garbage collection cycle is almost over.
 			template <class Scanner>
 			typename Scanner::Result scanFinal(ArenaTicket &ticket,
 											GenSet refsTo,
@@ -429,7 +429,7 @@ namespace storm {
 
 		template <class Scanner>
 		typename Scanner::Result Generation::scan(GenChunk &chunk, typename Scanner::Source &source) {
-			Block *end = chunk.memory.end();
+			Block *end = (Block *)chunk.memory.end();
 
 			typename Scanner::Result r = typename Scanner::Result();
 			for (Block *at = (Block *)chunk.memory.at; at != end; at = (Block *)at->mem(at->size)) {
