@@ -6,6 +6,7 @@
 
 #include "Utils.h"
 #include <signal.h>
+#include <errno.h>
 
 namespace storm {
 	namespace smm {
@@ -29,6 +30,8 @@ namespace storm {
 
 			(void)signal;
 			(void)info;
+			// We need to preserve errno.
+			int oldErrno = errno;
 
 			if (tlInfo) {
 				tlInfo->context = (ucontext_t *)context;
@@ -36,6 +39,8 @@ namespace storm {
 				tlInfo->onResume.down();
 				clearLocalICache();
 			}
+
+			errno = oldErrno;
 		}
 
 
