@@ -10,6 +10,7 @@
 #include "Gc/Fault.h"
 #include <sys/mman.h>
 #include <signal.h>
+#include <errno.h>
 
 /**
  * The generic implementation for all Posix systems.
@@ -34,10 +35,13 @@ namespace storm {
 
 			(void)signal;
 			(void)context;
+			// We need to preserve errno.
+			int oldErrno = errno;
 
 			// If we can do something about the error, we don't need to panic.
 			if (VM::notifyWrite(info->si_addr)) {
 				clearLocalICache();
+				errno = oldErrno;
 				return;
 			}
 
