@@ -50,6 +50,28 @@ namespace storm {
 			return Result();
 		}
 
+		// Same as 'fix12', but for stack addresses that may contain authenticated pointers to
+		// binary objects.
+#ifndef ARM_USE_PAC
+		static inline Result fixStack12(Scanner &s, void **ptr) {
+			return fix12(s, ptr);
+		}
+#else
+		static inline Result fixStack12(Scanner &s, void **ptr) {
+			void *decoded = *ptr;
+			__asm__(
+				"mov x30, %0\n"
+				"xpaclri\n"
+				"mov %0, x30\n"
+				: "+r" (decoded) : : "r30"
+				);
+			// We never have to write back these.
+			if (decoded != *ptr)
+				ptr = &decoded;
+			return fix12(s, ptr);
+		}
+#endif
+
 	public:
 		// Shorthand for stacks.
 		typedef os::InlineSet<os::Stack> StackSet;
@@ -88,6 +110,7 @@ namespace storm {
 		// currently. The extent of this thread is specified by 'current' (the lower end, on X86).
 		// Additionally, returns the number of bytes scanned, which may be interesting for some GC
 		// implementations.
+		// If 'current' is null, the currently running thread is not scanned.
 		static Result stacks(Source &source, const StackSet &stacks, void *current, size_t *scanned) {
 			size_t bytesScanned = 0;
 
@@ -148,7 +171,7 @@ namespace storm {
 					void **high = (void **)stack->high();
 					bytesScanned += (char *)high - (char *)low;
 					for (; low < high; low++) {
-						Result r = fix12(s, low);
+						Result r = fixStack12(s, low);
 						if (r != Result())
 							return r;
 					}
@@ -165,10 +188,10 @@ namespace storm {
 				// thread to see this.
 				PLN(L"------------ We found an UThread in the process of switching! ------------");
 #endif
-			} else {
+			} else if (current) {
 				bytesScanned += (char *)to - (char *)current;
 				for (void **at = (void **)current; at < to; at++) {
-					Result r = fix12(s, at);
+					Result r = fixStack12(s, at);
 					if (r != Result())
 						return r;
 				}
