@@ -64,7 +64,7 @@ namespace graphics {
 	 * Structs used when parsing a Windows BMP/DIB file.
 	 *
 	 * NOTE: Numbers are stored in little endian format, which works fine as long as we are running
-	 * on an X86 cpu.
+	 * on an X86 or compatible cpu.
 	 */
 
 	/**
@@ -199,7 +199,7 @@ namespace graphics {
 
 	// Read data to an array.
 	template <class T>
-	T *read(IStream *src, Nat count) {
+	static T *read(IStream *src, Nat count) {
 		Nat size = count*sizeof(T);
 		Buffer r = src->fill(buffer(src->engine(), size));
 		if (r.filled() != size)
