@@ -4,6 +4,7 @@
 #include "Jpeg.h"
 #include "Bmp.h"
 #include "Ppm.h"
+#include "Ico.h"
 #include "Exception.h"
 #include "Core/Join.h"
 #include "Shared/Engine.h"
@@ -15,6 +16,7 @@ namespace graphics {
 		*to << jpegFormat(e);
 		*to << bmpFormat(e);
 		*to << ppmFormat(e);
+		*to << icoFormat(e);
 	}
 
 	ImageFormat::ImageFormat(Str *name, Array<Str *> *extensions,
@@ -84,6 +86,10 @@ namespace graphics {
 		return options()->load(from);
 	}
 
+	ImageSet *ImageFormat::loadSet(IStream *from) {
+		return options()->loadSet(from);
+	}
+
 	Image *ImageFormat::load(Url *from) {
 		if (!from->exists()) {
 			Str *msg = TO_S(from, S("The file ") << from << S(" does not exist."));
@@ -100,6 +106,22 @@ namespace graphics {
 		}
 	}
 
+	ImageSet *ImageFormat::loadSet(Url *from) {
+		if (!from->exists()) {
+			Str *msg = TO_S(from, S("The file ") << from << S(" does not exist."));
+			throw new (from) ImageLoadError(msg);
+		}
+		IStream *s = from->read();
+		try {
+			ImageSet *i = loadSet(s);
+			s->close();
+			return i;
+		} catch (...) {
+			s->close();
+			throw;
+		}
+	}
+
 	void ImageFormat::save(Image *image, OStream *to) {
 		return options()->save(image, to);
 	}
@@ -181,6 +203,30 @@ namespace graphics {
 		}
 	}
 
+	ImageSet *loadImageSet(IStream *from) {
+		ImageFormat *format = findFormat(from);
+		if (!format)
+			throw new (from) ImageLoadError(S("The file type is not supported."));
+
+		return format->loadSet(from);
+	}
+
+	ImageSet *loadImageSet(Url *from) {
+		if (!from->exists()) {
+			Str *msg = TO_S(from, S("The file ") << from << S(" does not exist."));
+			throw new (from) ImageLoadError(msg);
+		}
+		IStream *stream = from->read();
+		try {
+			ImageSet *out = loadImageSet(stream);
+			stream->close();
+			return out;
+		} catch (...) {
+			stream->close();
+			throw;
+		}
+	}
+
 
 	MAYBE(ImageFormat *) findFormat(Str *fileExt) {
 		Array<ImageFormat *> *fmts = supportedImageFormats(fileExt->engine());
