@@ -1,5 +1,6 @@
 #pragma once
 #include "Core/Graphics/Image.h"
+#include "Core/Graphics/ImageSet.h"
 #include "Core/Io/Stream.h"
 #include "Core/Io/Url.h"
 
@@ -16,9 +17,11 @@ namespace graphics {
 
 		// Load an image. Set properties in here to reflect the data in the loaded file.
 		virtual Image *STORM_FN load(IStream *from) ABSTRACT;
+		virtual ImageSet *STORM_FN loadSet(IStream *from);
 
 		// Convenience to load from an Url directly.
 		Image *STORM_FN load(Url *from);
+		ImageSet *STORM_FN loadSet(Url *from);
 
 		// Save an image. Called from 'saveImage'.
 		virtual void STORM_FN save(Image *image, OStream *to) ABSTRACT;
