@@ -36,6 +36,7 @@ namespace gui {
 	App::App() : appWait(null), creating(null) {
 		windows = new (this) Map<Handle, Window *>();
 		liveWindows = new (this) Set<Window *>();
+		inhibitingScreenSaver = new (this) Set<Window *>();
 
 		init();
 
@@ -52,8 +53,9 @@ namespace gui {
 	}
 
 	void App::terminate() {
-		// Reap all live windows.
-		for (WindowSet::Iter i = liveWindows->begin(), e = liveWindows->end(); i != e; ++i)
+		// Reap all live windows. Make a copy of the set to avoid modifying the set as we traverse it.
+		WindowSet *liveCopy = new (this) WindowSet(*liveWindows);
+		for (WindowSet::Iter i = liveCopy->begin(), e = liveCopy->end(); i != e; ++i)
 			i.v()->handle(Window::invalid);
 
 		if (appWait) {
@@ -84,6 +86,7 @@ namespace gui {
 	void App::removeWindow(Window *w) {
 		windows->remove(w->handle());
 		liveWindows->remove(w);
+		inhibitScreenSaver(w, false);
 	}
 
 	Window *App::findWindow(Handle h) {
@@ -133,6 +136,39 @@ namespace gui {
 		}
 	}
 
+	void App::inhibitScreenSaver(Window *window, Bool start) {
+		if (start) {
+			Bool emptyBefore = inhibitingScreenSaver->empty();
+			if (inhibitingScreenSaver->put(window) && emptyBefore) {
+				if (appWait)
+					appWait->screenSaver.inhibit();
+			}
+		} else {
+			if (inhibitingScreenSaver->remove(window) && inhibitingScreenSaver->empty()) {
+				if (appWait)
+					appWait->screenSaver.resume();
+			}
+		}
+	}
+
+	MAYBE(Icon *) App::icon() {
+		return defaultIcon;
+	}
+
+	void App::icon(Icon *icon) {
+		defaultIcon = icon;
+
+		for (WindowSet::Iter i = liveWindows->begin(), end = liveWindows->end(); i != end; ++i) {
+			if (Frame *f = as<Frame>(i.v())) {
+				f->updateDefaultIcon();
+			}
+		}
+	}
+
+	void App::icon(ImageSet *images) {
+		icon(new (this) Icon(images));
+	}
+
 	App *app(EnginePtr e) {
 		App *&v = appData(e.v);
 		if (!v) {
@@ -351,8 +387,6 @@ namespace gui {
 		WNDCLASSEX wc;
 		zeroMem(wc);
 		wc.cbSize = sizeof(WNDCLASSEX);
-		TODO(L"Load icon from somewhere!");
-		// HICON icon = LoadIcon(HINSTANCE, MAKEINTRESOURCE(iconResource));
 		HICON icon = LoadIcon(NULL, IDI_APPLICATION);
 		wc.style = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
 		wc.lpfnWndProc = &App::windowProc;
@@ -405,9 +439,13 @@ namespace gui {
 		if (!IsGUIThread(TRUE)) {
 			assert(false, L"Could not convert to a GUI thread.");
 		}
+
+		screenSaver.platformInit();
 	}
 
-	void AppWait::platformDestroy() {}
+	void AppWait::platformDestroy() {
+		screenSaver.platformDestroy();
+	}
 
 	bool AppWait::wait(os::IOHandle &io) {
 		// Since we know the semantics of wait(IOHandle, nat), we can exploit that...
@@ -597,7 +635,6 @@ namespace gui {
 		blockActive = false;
 	}
 
-
 #endif
 
 #ifdef GUI_GTK
@@ -660,6 +697,8 @@ namespace gui {
 		fcntl(pipeRead, F_SETFD, FD_CLOEXEC);
 		fcntl(pipeWrite, F_SETFD, FD_CLOEXEC);
 #endif
+
+		screenSaver.platformInit();
 	}
 
 	void AppWait::platformDestroy() {
@@ -678,6 +717,9 @@ namespace gui {
 		// Remove us.
 		currentWait = null;
 
+		// Clean up the screen saver interface.
+		screenSaver.platformDestroy();
+
 		// Release ownership of the main context.
 		g_main_context_release(context);
 
@@ -1142,7 +1184,6 @@ namespace gui {
 		}
 	}
 
-
 #endif
 
 }
