@@ -87,6 +87,47 @@ namespace gui {
 		device = cairo_create(surface);
 	}
 
+	Image *CairoSurface::grabImage(Engine &e) {
+		Image *result = new (e) Image(size);
+		// Need to keep the buffer on the stack, since we pass it to Cairo.
+		byte *volatile buffer = null;
+		atomicWrite(buffer, result->buffer());
+
+		cairo_surface_t *output = cairo_image_surface_create_for_data(buffer, CAIRO_FORMAT_ARGB32,
+																	(int)size.w, (int)size.h, (int)result->stride());
+
+		cairo_t *c = cairo_create(output);
+		cairo_set_source_surface(c, surface, 0, 0);
+		cairo_paint(c);
+		cairo_destroy(c);
+
+		cairo_surface_destroy(output);
+
+		// We need to convert the data...
+		for (Nat y = 0; y < result->height(); y++) {
+			// Interestingly enough it seems like pixels are flipped horizontally also.
+			std::reverse((Nat *)result->buffer(0, y), (Nat *)result->buffer(result->width(), y));
+
+			for (Nat x = 0; x < result->width(); x++) {
+				// As specified by Cairo:
+				byte *b = result->buffer(x, y);
+				Nat pixel = *(Nat *)b;
+
+				// Now, write according to our specification. Note, we don't really care about alpha
+				// here since we render to a non-alpha buffer anyway.
+				b[0] = (pixel >> 16) & 0xFF;
+				b[1] = (pixel >>  8) & 0xFF;
+				b[2] = (pixel >>  0) & 0xFF;
+				b[3] = (pixel >> 24) & 0xFF;
+			}
+		}
+
+		// Flip vertically as well.
+		result->vflip();
+
+		return result;
+	}
+
 
 	CairoBlitSurface::CairoBlitSurface(Nat id, Size size, Float scale, cairo_surface_t *surface)
 		: CairoSurface(id, size, scale, surface) {}
