@@ -40,9 +40,10 @@ namespace gui {
 		Map<Nat, RadioGroup *> *radioGroups;
 
 		// Padding due to 8-byte alignment from Window.
-		Nat pad;
+		size_t pad;
 	};
 
+
 	/**
 	 * A container is a window that can contain child windows.
 	 *
@@ -90,6 +91,9 @@ namespace gui {
 
 		// Move a child widget.
 		virtual void moveChild(GtkWidget *child, Rect pos);
+
+		// Update size.
+		virtual void updateGtkMinSize();
 #endif
 	protected:
 		// Create.
@@ -118,4 +122,51 @@ namespace gui {
 		void allocate(Window *window, Nat id);
 	};
 
+
+	/**
+	 * A container with a single child.
+	 *
+	 * This is used for ScrollWindow and Group, to make it easier to detect such containers in the
+	 * layout language. These containers generally resize the child based on the size of the parent.
+	 */
+	class SingleChildContainer : public ContainerBase {
+		STORM_CLASS;
+	public:
+		// Create.
+		STORM_CTOR SingleChildContainer();
+		STORM_CTOR SingleChildContainer(Window *child);
+
+		// Get the child.
+		MAYBE(Window *) STORM_FN content() {
+			return child;
+		}
+
+		// Set the child.
+		virtual void STORM_ASSIGN content(Window *window);
+
+		// Called when our parent is created.
+		virtual void parentCreated(nat id);
+
+		// Destroy notification.
+		virtual void windowDestroyed();
+
+#ifdef GUI_GTK
+		// Add a child widget to the layout here. Works when the container is a GTK_BIN.
+		virtual void addChild(GtkWidget *child, Rect pos);
+
+		// Move a child widget.
+		virtual void moveChild(GtkWidget *child, Rect pos);
+
+		// Update size.
+		virtual void updateGtkMinSize();
+#endif
+
+	private:
+		// The single child.
+		MAYBE(Window *) child;
+
+		// Padding due to 8-byte alignment from Window.
+		size_t pad;
+	};
+
 }
