@@ -15,24 +15,24 @@ namespace gui {
 	static void goBack(Handle wnd, Nat &oldStyle, Rect &oldPos);
 
 
-	Frame::Frame(Str *title) : myMenu(null), full(false), showCursor(true) {
+	Frame::Frame(Str *title) : myMenu(null), flags(showCursor) {
 		dpi = defaultDpi;
 		onClose = new (this) Event();
 		myAccelerators = new (this) Accelerators();
 		attachParent(this);
 		text(title);
 		pos(Rect(-10000, -10000, -10002, -10002));
-		posSet = false;
+		clearFlag(positionAltered);
 	}
 
-	Frame::Frame(Str *title, Size size) : myMenu(null), full(false), showCursor(true) {
+	Frame::Frame(Str *title, Size size) : myMenu(null), flags(showCursor) {
 		dpi = defaultDpi;
 		onClose = new (this) Event();
 		myAccelerators = new (this) Accelerators();
 		attachParent(this);
 		text(title);
 		pos(Rect(Point(-10000, -10000), size));
-		posSet = false;
+		clearFlag(positionAltered);
 	}
 
 	void Frame::create() {
@@ -56,21 +56,21 @@ namespace gui {
 	}
 
 	void Frame::size(Size s) {
-		if (!full) {
-			Bool oldSet = posSet;
+		if (!hasFlag(isFullscreen)) {
+			Bool oldSet = hasFlag(positionAltered);
 			Rect p = pos();
 			p.size(s);
 			pos(p);
-			posSet = oldSet;
+			setFlag(positionAltered, oldSet);
 		}
 	}
 
 	Bool Frame::cursorVisible() {
-		return showCursor;
+		return hasFlag(showCursor);
 	}
 
 	void Frame::fullscreen(Bool v) {
-		if (created() && full != v) {
+		if (created() && hasFlag(isFullscreen) != v) {
 			if (v) {
 				goFull(handle(), windowedStyle, windowedRect);
 			} else {
@@ -78,11 +78,11 @@ namespace gui {
 			}
 		}
 
-		full = v;
+		setFlag(isFullscreen, v);
 	}
 
 	Bool Frame::fullscreen() {
-		return full;
+		return hasFlag(isFullscreen);
 	}
 
 	void Frame::menu(MenuBar *menu) {
@@ -133,6 +133,35 @@ namespace gui {
 		setFocus = to;
 	}
 
+	Bool Frame::inhibitScreenSaver() {
+		return hasFlag(noScreenSaver);
+	}
+
+	void Frame::inhibitScreenSaver(Bool inhibit) {
+		setFlag(noScreenSaver, inhibit);
+
+		if (created() && !hasFlag(isMinimized)) {
+			app(engine())->inhibitScreenSaver(this, inhibit);
+		}
+	}
+
+	MAYBE(Icon *) Frame::icon() {
+		return myIcon;
+	}
+
+	void Frame::icon(ImageSet *images) {
+		icon(new (this) Icon(images));
+	}
+
+	void Frame::updateDefaultIcon() {
+		if (!myIcon)
+			icon((Icon *)null);
+	}
+
+	Bool Frame::minimized() {
+		return hasFlag(isMinimized);
+	}
+
 	void Frame::onResize(Size size) {
 		updateMinSize();
 		Window::onResize(size);
@@ -173,10 +202,10 @@ namespace gui {
 	}
 
 	void Frame::pos(Rect r) {
-		if (full)
+		if (hasFlag(isFullscreen))
 			return;
 
-		posSet = true;
+		setFlag(positionAltered);
 
 		// We need to do this to take the menu into account.
 		myPos = r;
@@ -251,8 +280,9 @@ namespace gui {
 		if (parent)
 			hParent = parent->handle().hwnd();
 		CreateFlags cFlags = cManualVisibility;
-		if (!posSet)
+		if (!hasFlag(positionAltered)) {
 			cFlags |= cAutoPos;
+		}
 		if (!createEx(NULL, styles, exStyles, hParent, 0, cFlags))
 			return false;
 
@@ -261,6 +291,8 @@ namespace gui {
 		// Create child windows (a bit of a hack, sorry!)
 		parentCreated(0);
 
+		// Set icons.
+		icon(myIcon);
 
 		if (visible()) {
 			// At some points, it seems we need to hide the window first in order for it to be
@@ -271,7 +303,7 @@ namespace gui {
 			update();
 		}
 
-		if (full) {
+		if (hasFlag(isFullscreen)) {
 			goFull(handle(), windowedStyle, windowedRect);
 		}
 
@@ -284,6 +316,10 @@ namespace gui {
 			setFocus->focus();
 		}
 
+		// Update inhibit screensaver state.
+		if (hasFlag(noScreenSaver))
+			app(engine())->inhibitScreenSaver(this, true);
+
 		return true;
 	}
 
@@ -340,7 +376,7 @@ namespace gui {
 			return msgResult(0);
 		}
 		case WM_SETCURSOR:
-			if (LOWORD(msg.lParam) == HTCLIENT && !showCursor) {
+			if (LOWORD(msg.lParam) == HTCLIENT && !hasFlag(showCursor)) {
 				SetCursor(NULL);
 				return msgResult(TRUE);
 			}
@@ -351,6 +387,20 @@ namespace gui {
 		case WM_MENUCOMMAND:
 			menuClicked((HMENU)msg.lParam, (Nat)msg.wParam);
 			return msgResult(TRUE);
+		case WM_SIZE:
+			if (msg.wParam == SIZE_MINIMIZED) {
+				if (!hasFlag(isMinimized) && hasFlag(noScreenSaver)) {
+					app(engine())->inhibitScreenSaver(this, false);
+				}
+				setFlag(isMinimized);
+			} else if (msg.wParam == SIZE_RESTORED) {
+				if (hasFlag(isMinimized) && hasFlag(noScreenSaver)) {
+					app(engine())->inhibitScreenSaver(this, true);
+				}
+				clearFlag(isMinimized);
+			}
+			// Note: Intentional to not stop message propagation - we only wish to detect what is happening!
+			break;
 		}
 
 		return Container::onMessage(msg);
@@ -379,7 +429,7 @@ namespace gui {
 
 	void Frame::cursorVisible(Bool v) {
 		// Will be altered soon anyway, we do not need to do anything here.
-		showCursor = v;
+		setFlag(showCursor, v);
 	}
 
 	void Frame::updateMinSize() {
@@ -404,7 +454,7 @@ namespace gui {
 
 		// Alter our height if needed.
 		int delta = original.bottom - changed.bottom;
-		if (delta && !full) {
+		if (delta && !hasFlag(isFullscreen)) {
 			GetWindowRect(handle().hwnd(), &original);
 			original.bottom += delta;
 			int width = original.right - original.left;
@@ -446,6 +496,20 @@ namespace gui {
 		TrackPopupMenuEx(menu->handle.menu(), TPM_LEFTALIGN | TPM_TOPALIGN, pt.x, pt.y, handle().hwnd(), NULL);
 	}
 
+	void Frame::icon(MAYBE(Icon *) icon) {
+		myIcon = icon;
+
+		if (created()) {
+			Icon *toSet = myIcon;
+			if (!toSet)
+				toSet = app(engine())->icon();
+
+			HWND hWnd = handle().hwnd();
+
+			SendMessage(hWnd, WM_SETICON, ICON_SMALL, LPARAM(toSet ? toSet->win32Small() : NULL));
+			SendMessage(hWnd, WM_SETICON, ICON_BIG, LPARAM(toSet ? toSet->win32Large() : NULL));
+		}
+	}
 
 #endif
 #ifdef GUI_GTK
@@ -456,8 +520,8 @@ namespace gui {
 
 	void Frame::pos(Rect r) {
 		myPos = r;
-		posSet = true;
-		if (!full && handle() != Handle()) {
+		setFlag(positionAltered);
+		if (!hasFlag(isFullscreen) && handle() != Handle()) {
 			gtk_window_resize(GTK_WINDOW(handle().widget()), r.size().w, r.size().h);
 		}
 	}
@@ -504,6 +568,7 @@ namespace gui {
 		gtk_widget_show(container);
 
 		Signal<void, Frame>::Connect<&Frame::close>::to(frame, "destroy", engine());
+		Signal<gboolean, Frame, GdkEventWindowState *>::Connect<&Frame::windowState>::to(frame, "window-state-event", engine());
 
 		Size sz = pos().size();
 		if (sz.w > 0 && sz.h > 0)
@@ -512,6 +577,9 @@ namespace gui {
 		handle(frame);
 		initSignals(frame, container);
 
+		// Set icon if applicable.
+		icon(myIcon);
+
 		// Attach accelerators.
 		myAccelerators->attach(handle());
 
@@ -526,9 +594,9 @@ namespace gui {
 		}
 
 		// Update cursor visibility.
-		cursorVisible(showCursor);
+		cursorVisible(hasFlag(showCursor));
 
-		if (full) {
+		if (hasFlag(isFullscreen)) {
 			goFull(handle(), windowedStyle, windowedRect);
 		}
 
@@ -541,6 +609,10 @@ namespace gui {
 			setFocus->focus();
 		}
 
+		// Update inhibit screensaver state.
+		if (hasFlag(noScreenSaver))
+			app(engine())->inhibitScreenSaver(this, true);
+
 		return true;
 	}
 
@@ -584,11 +656,11 @@ namespace gui {
 	}
 
 	void Frame::cursorVisible(Bool v) {
-		showCursor = v;
+		setFlag(showCursor, v);
 		if (created()) {
 			GtkWidget *widget = handle().widget();
 			GdkCursor *cursor = null;
-			if (showCursor)
+			if (hasFlag(showCursor))
 				cursor = gdk_cursor_new_from_name(gtk_widget_get_display(widget), "default");
 			else
 				cursor = gdk_cursor_new_for_display(gtk_widget_get_display(widget), GDK_BLANK_CURSOR);
@@ -635,7 +707,7 @@ namespace gui {
 			heightDiff += natural;
 		}
 
-		if (!full)
+		if (!hasFlag(isFullscreen))
 			gtk_window_resize(GTK_WINDOW(handle().widget()), oldWidth, oldHeight + heightDiff);
 
 	}
@@ -650,6 +722,31 @@ namespace gui {
 		gtk_menu_popup_at_pointer(GTK_MENU(menu->handle.widget()), NULL);
 	}
 
+	gboolean Frame::windowState(GdkEventWindowState *ws) {
+		Bool minimized = (ws->new_window_state & GDK_WINDOW_STATE_ICONIFIED) != 0;
+		if (minimized != hasFlag(isMinimized)) {
+			setFlag(isMinimized, minimized);
+			if (hasFlag(noScreenSaver)) {
+				app(engine())->inhibitScreenSaver(this, !minimized);
+			}
+		}
+		return TRUE;
+	}
+
+	void Frame::icon(MAYBE(Icon *) icon) {
+		myIcon = icon;
+
+		if (created()) {
+			Icon *toSet = myIcon;
+			if (!toSet)
+				toSet = app(engine())->icon();
+
+			// Note: We can't really ask Gtk to "remove" the icon if we ever set one.
+			if (toSet)
+				gtk_window_set_icon_list(GTK_WINDOW(handle().widget()), toSet->gtkList());
+		}
+	}
+
 #endif
 
 }
