@@ -3,6 +3,7 @@
 #include "Core/Event.h"
 #include "Menu.h"
 #include "Accelerators.h"
+#include "Icon.h"
 
 namespace gui {
 
@@ -101,6 +102,23 @@ namespace gui {
 		void focus(Window *child);
 		using Window::focus;
 
+		// Get/set the current icon. If null, we use the default icon set in the application itself.
+		MAYBE(Icon *) STORM_FN icon();
+		void STORM_ASSIGN icon(MAYBE(Icon *) icon);
+		void STORM_ASSIGN icon(ImageSet *icon);
+
+		// Called from App to let us know that there is a new default icon.
+		void updateDefaultIcon();
+
+		// Inhibit the screen saver while this window is visible on the screen (= not minimized).
+		// Useful for applications that e.g. play video, where the screen should stay on even though
+		// not much user interaction is expected.
+		Bool STORM_FN inhibitScreenSaver();
+		void STORM_ASSIGN inhibitScreenSaver(Bool inhibit);
+
+		// Check if the window is minimized (note that the window is still considered visible if minimized).
+		Bool STORM_FN minimized();
+
 	protected:
 		// Notification on window resizes.
 		virtual void onResize(Size size);
@@ -133,6 +151,9 @@ namespace gui {
 		// Window to assign focus after creation. Only used on Windows currently.
 		MAYBE(Window *) setFocus;
 
+		// Current icon.
+		MAYBE(Icon *) myIcon;
+
 		// Cached minimum size. Updated whenever the window is resized.
 		Size lastMinSize;
 
@@ -143,18 +164,52 @@ namespace gui {
 		Nat windowedStyle;
 		Rect windowedRect;
 
-		// Fullscreen mode?
-		Bool full;
+		// Misc. flags (instead of individual booleans).
+		enum {
+			isFullscreen = 0x01,
+			isMinimized = 0x02,
+			showCursor = 0x04,
+			positionAltered = 0x08,
+			noScreenSaver = 0x10,
+		};
 
-		// Cursor visible?
-		Bool showCursor;
+		// Set a flag to a boolean.
+		void setFlag(Nat flag, Bool val) {
+			if (val)
+				setFlag(flag);
+			else
+				clearFlag(flag);
+		}
+
+		// Set a flag.
+		void setFlag(Nat flag) {
+			flags |= flag;
+		}
+
+		// Cler a flag.
+		void clearFlag(Nat flag) {
+			flags &= ~flag;
+		}
+
+		// Has a flag.
+		Bool hasFlag(Nat flag) {
+			return (flags & flag) != 0;
+		}
+
+		// All flags.
+		Nat flags;
+
+		// Padding for alignment.
+		Nat pad;
 
-		// Was our position altered?
-		Bool posSet;
 
 #ifdef GUI_WIN32
 		// Find the clicked menu item.
 		void menuClicked(HMENU menu, Nat id);
+#endif
+#ifdef GUI_GTK
+		// Keep track of minimize state.
+		gboolean windowState(GdkEventWindowState *event);
 #endif
 	};
 
