@@ -20,6 +20,10 @@ struct _Basic {
 	GtkContainer container;
 
 	GList *children;
+
+	// Preferred width and height reported to GTK. Updated using basic_set_min_size.
+	gint min_width;
+	gint min_height;
 };
 
 struct _BasicClass {
@@ -48,4 +52,7 @@ void basic_put(Basic *layout, GtkWidget *widget, gint x, gint y, gint w, gint h)
 // Move a child widget.
 void basic_move(Basic *layout, GtkWidget *widget, gint x, gint y, gint w, gint h);
 
+// Set the minimum size of the container. Used to update values from Storm.
+void basic_set_min_size(Basic *layout, gint width, gint height);
+
 #endif
