@@ -17,6 +17,7 @@ namespace gui {
 		Handle(HMENU menu) { value = (size_t)menu; }
 		Handle(HINSTANCE instance) { value = (size_t)instance; }
 		Handle(HBRUSH brush) { value = (size_t)brush; }
+		Handle(HICON icon) { value = (size_t)icon; }
 
 		// Get Win32 types.
 		inline HWND hwnd() const { return (HWND)value; }
@@ -24,6 +25,7 @@ namespace gui {
 		inline HMENU menu() const { return (HMENU)value; }
 		inline HINSTANCE instance() const { return (HINSTANCE)value; }
 		inline HBRUSH brush() const { return (HBRUSH)value; }
+		inline HICON icon() const { return (HICON)value; }
 #endif
 
 #ifdef GUI_GTK
