@@ -5,7 +5,7 @@
 
 namespace gui {
 
-	static const Nat entries = 2;
+	static const Nat entries = 3;
 
 	static inline GcArray<void *> *data(Engine &e) {
 		return (GcArray<void *> *)e.data();
@@ -19,11 +19,17 @@ namespace gui {
 		return (RenderMgr *&)data(e)->v[1];
 	}
 
+	os::Lock &dataLock(Engine &e) {
+		return *(os::Lock *)data(e)->v[2];
+	}
+
 }
 
 void *createLibData(storm::Engine &e) {
 	using namespace gui;
-	return runtime::allocArray<void *>(e, &pointerArrayType, gui::entries);
+	GcArray<void *> *data = runtime::allocArray<void *>(e, &pointerArrayType, gui::entries);
+	data->v[2] = new os::Lock();
+	return data;
 }
 
 void destroyLibData(void *data) {
@@ -41,4 +47,5 @@ void destroyLibData(void *data) {
 	if (app)
 		app->terminate();
 
+	delete (os::Lock *)d->v[2];
 }
