@@ -98,6 +98,9 @@ namespace gui {
 		// Ensure that the surface is active.
 		void makeCurrent() { context->makeCurrent(); }
 
+		// Grab an image.
+		virtual Image *grabImage(Engine &e);
+
 	private:
 		// Context.
 		SkiaContext *context;
