@@ -17,9 +17,6 @@
 #include <Shobjidl.h>
 
 
-// HRESULT to string.
-String toS(HRESULT r);
-
 // It defines a macro 'interface' that messes with the rest of the system... (defined to 'struct')
 #undef interface
 
