@@ -232,7 +232,7 @@ namespace gui {
 			return msgResult(1);
 		}
 		case WM_TIMER: {
-			if (msg.wParam == 1) {
+			if (msg.wParam == (WPARAM)handle().hwnd()) {
 				onTimer();
 				return msgResult(0);
 			}
