@@ -19,6 +19,8 @@ namespace gui {
 		cNormal = 0x0,
 		cManualVisibility = 0x1,
 		cAutoPos = 0x2,
+		// For use in Win32: Subclass the control.
+		cWin32Subclass = 0x100,
 	};
 
 	BITMASK_OPERATORS(CreateFlags);
@@ -84,13 +86,16 @@ namespace gui {
 #ifdef GUI_WIN32
 		// Called when a regular message has been received. If no result is returned, calls the
 		// default window proc, or any message procedure declared by the window we're handling. Only
-		// works for windows with the default window class provided by App. Otherwise, use 'preTranslateMessage'.
+		// works for windows with the default window class provided by App. Otherwise, use 'beforeMessage'.
 		virtual MsgResult onMessage(const Message &msg);
 
 		// Called before a message is dispatched to the window procedure. Return a result to inhibit the regular
-		// dispatch.
+		// dispatch. Note: This only happens for messages that are posted from outside the process.
 		virtual MsgResult beforeMessage(const Message &msg);
 
+		// Called after messages that were marked as 'notify' from 'onMessage'.
+		virtual void afterMessage(const Message &msg);
+
 		// Called when a WM_COMMAND has been sent to (by) us. Return 'true' if it is handled. Type
 		// is the notification code specified by the message (eg BN_CLICK).
 		virtual bool onCommand(nat type);
@@ -105,12 +110,22 @@ namespace gui {
 
 		// Update this window to new DPI settings (i.e., "currentDpi()" has changed).
 		virtual void updateDpi(Bool move);
+
+		// Get the background color of this window as a brush. This is used to account for
+		// containers that draw a different background color than the default dialog color.
+		virtual void windowBackground(HBRUSH &brush, COLORREF &color);
+
 #endif
 #ifdef GUI_GTK
 		// Called when we received an EXPOSE event, but before Gtk+ is informed about the
 		// event. Return 'true' to inhibit the Gtk+ behaviour completely.
 		bool preExpose(GtkWidget *widget);
 
+		// Call to update the minimum size as seen by GTK. This is only relevant to do when asking
+		// GTK about the preferred size of something that contains this window. Also, it is only
+		// necessary for Containers with custom (manual) layout. This function calls 'minSize' to
+		// ask Storm for the minimum size to use.
+		virtual void updateGtkMinSize();
 #endif
 
 		// Check if the window is visible.
