@@ -393,7 +393,7 @@ static Url *createRootUrl(Engine &e, void *d) {
 	return data->created;
 }
 
-int stormMain(int argc, const wchar_t *argv[]) {
+int stormMain(int argc, const wchar_t *argv[], void *stackBase) {
 	Params p(argc, argv);
 
 	switch (p.mode) {
@@ -420,7 +420,7 @@ int stormMain(int argc, const wchar_t *argv[]) {
 
 	// Start an Engine.
 	RootUrlData root = { p, null };
-	Engine e(&createRootUrl, &root, Engine::reuseMain, &argv);
+	Engine e(&createRootUrl, &root, Engine::reuseMain, stackBase);
 	Moment end;
 
 	try {
@@ -504,7 +504,7 @@ int _tmain(int argc, const wchar *argv[]) {
 		SetConsoleOutputCP(targetCP);
 
 	try {
-		result = stormMain(argc, argv);
+		result = stormMain(argc, argv, &argv);
 	} catch (const ::Exception &e) {
 		wcerr << L"Unhandled exception: " << e << endl;
 		result = 1;
@@ -541,7 +541,7 @@ int main(int argc, const char *argv[]) {
 		for (int i = 0; i < argc; i++)
 			c_args[i] = args[i].c_str();
 
-		return stormMain(argc, &c_args[0]);
+		return stormMain(argc, &c_args[0], &argv);
 	} catch (const ::Exception &e) {
 		wcerr << "Unhandled exception: " << e << endl;
 		return 1;
