@@ -46,7 +46,7 @@ namespace os {
 		me->callRaw(fn, member, first, result);
 	}
 
-#if defined(X86) && defined(WINDOWS)
+#if (defined(X86) || defined(X64)) && defined(WINDOWS)
 
 	// Helper, used on X86 from StackCall.asm.
 	extern "C" void fnCallCatch(FnCallRaw *me, const void *fn, bool member, void *first, void *result) {
