@@ -1,11 +1,13 @@
 #include "stdafx.h"
 #include "Future.h"
 #include "PtrThrowable.h"
+#include "UThread.h"
 #include "Utils/TIB.h"
 
 namespace os {
 
-	FutureBase::FutureBase() : ptrException(null), resultPosted(resultEmpty), resultRead(readNone) {}
+	FutureBase::FutureBase(FutureMode::Mode mode)
+		: ptrException(null), resultPosted(resultEmpty), resultRead(readNone), mode(mode), notified(false) {}
 
 	FutureBase::~FutureBase() {
 		// Warn about uncaught exceptions if we didn't throw it yet.
@@ -129,6 +131,39 @@ namespace os {
 		}
 	}
 
+	void FutureBase::notify() {
+		// Keep lists locally in case the thread(s) we wake deallocate this future once they wake up.
+		InlineList<UThreadData> local;
+
+		{
+			util::Lock::L z(lock);
+			notified = true;
+
+			while (UThreadData *wake = waitingThreads.pop())
+				local.push(wake);
+		}
+
+		while (UThreadData *wake = local.pop())
+			wake->owner()->wake(wake);
+	}
+
+	void FutureBase::wait() {
+		UThreadState *state = null;
+
+		{
+			util::Lock::L z(lock);
+
+			if (notified)
+				return;
+
+			state = UThreadState::current();
+			waitingThreads.push(state->runningThread());
+		}
+
+		// If 'exclusive' is true, we only allow high priority threads to run.
+		state->wait(exclusive());
+	}
+
 #ifdef CUSTOM_EXCEPTION_PTR
 
 	/**
