@@ -55,13 +55,27 @@ namespace sql {
 	 * Statement.
 	 */
 
-	Statement::Statement() : resultAliveCount(0), resultSequence(0) {}
+	Statement::Statement(DBConnection *connection) : resultAliveCount(0), resultSequence(0) {
+		connection->statements->putRaw(reinterpret_cast<TObject *>(this));
+	}
 
 	void Statement::invalidateResult() {
 		atomicIncrement(resultSequence);
 		atomicWrite(resultAliveCount, 0);
 	}
 
+	void Statement::deregister(DBConnection *c) {
+		c->statements->removeRaw(reinterpret_cast<TObject *>(this));
+	}
+
+	Statement::Statement(const Statement &o) {
+		throw new (this) SQLError(new (this) Str(S("Can not copy a prepared statement.")));
+	}
+
+	void Statement::deepCopy(CloneEnv *env) {
+		throw new (this) SQLError(new (this) Str(S("Can not move a prepared statement across thread boundaries.")));
+	}
+
 
 	/**
 	 * Statement result.
@@ -135,12 +149,42 @@ namespace sql {
 	 * Connection.
 	 */
 
-	Bool DBConnection::implicitAutoIncrement() const {
-		return false;
+	DBConnection::DBConnection() {
+		statements = new (this) WeakSetBase();
+		cachedStatements = new (this) RefMap<CachedQuery *, Statement *>();
 	}
 
-	Bool DBConnection::fullAlterTable() const {
-		return true;
+	DBConnection::DBConnection(const DBConnection &o) {
+		throw new (this) SQLError(new (this) Str(S("Can not copy a database connection.")));
+	}
+
+	void DBConnection::deepCopy(CloneEnv *env) {
+		throw new (this) SQLError(new (this) Str(S("Can not move a database connection across thread boundaries.")));
+	}
+
+	Statement *DBConnection::prepare(QueryStr *query) {
+		return prepare(query->generate(visitor()));
+	}
+
+	Statement *DBConnection::prepare(CachedQuery *cached) {
+		Statement *out = cachedStatements->get(cached, null);
+		if (!out) {
+			out = cached->query(this);
+			cachedStatements->put(cached, out);
+		}
+		return out;
+	}
+
+	void DBConnection::close() {
+		// Note: We could close all 'cachedStatements' also, but those should all be in 'statements'
+		// anyway, so we can just clear it here.
+		cachedStatements->clear();
+
+		WeakSetBase::Iter iter(statements);
+		// Note: We know that everything that comes to us from the set will be alive and not finalized.
+		while (RootObject *stmt = iter.nextRaw()) {
+			reinterpret_cast<Statement *>(stmt)->finalize();
+		}
 	}
 
 }
