@@ -2,12 +2,19 @@
 #include "Core/Maybe.h"
 #include "Core/Array.h"
 #include "Core/Variant.h"
+#include "Core/WeakSet.h"
 #include "Row.h"
 #include "Schema.h"
+#include "Migration.h"
 #include "QueryStr.h"
+#include "Transaction.h"
+#include "CachedQuery.h"
+#include "Features.h"
 
 namespace sql {
 
+	class DBConnection;
+
 	/**
 	 * A prepared statement associated with a database connection.
 	 *
@@ -16,7 +23,12 @@ namespace sql {
 	class Statement : public Object {
 		STORM_ABSTRACT_CLASS;
 	public:
-		STORM_CTOR Statement();
+		// Create.
+		STORM_CTOR Statement(DBConnection *connection);
+
+		// Disallow copy and move across thread boundaries.
+		Statement(const Statement &o);
+		virtual void STORM_FN deepCopy(CloneEnv *env);
 
 		// Bind parameters.
 		virtual void STORM_FN bind(Nat pos, Str *str) ABSTRACT;
@@ -101,6 +113,9 @@ namespace sql {
 		// Called by iterators to get the number of changes.
 		virtual Nat STORM_FN changes() ABSTRACT;
 
+		// De-register from the DB connection to avoid multiple finalization messages.
+		void STORM_FN deregister(DBConnection *c);
+
 	private:
 		// Number of Result objects that are currently alive. When it reaches zero, we can end the
 		// current query even if not all rows were extracted.
@@ -131,12 +146,26 @@ namespace sql {
 	class DBConnection : public Object {
 		STORM_ABSTRACT_CLASS;
 	public:
+		// Create.
+		STORM_CTOR DBConnection();
+
+		// Disallow copying.
+		DBConnection(const DBConnection &o);
+
+		// Disallow moving across thread boundaries.
+		virtual void STORM_FN deepCopy(CloneEnv *env);
+
 		// Create a prepared statement. Throws an error if the statement is invalid.
 		virtual Statement *STORM_FN prepare(Str *query) ABSTRACT;
-		virtual Statement *STORM_FN prepare(QueryStr *query) ABSTRACT;
+		virtual Statement *STORM_FN prepare(QueryStr *query);
 
-		// Closes the connection to the database.
-		virtual void STORM_FN close() ABSTRACT;
+		// Create a prepared statement from a CachedQuery. This causes the connection to prepare the
+		// query the first time it is seen, and use the cached version for all subsequent calls.
+		Statement *STORM_FN prepare(CachedQuery *cached);
+
+		// Closes the connection to the database. The base implementation finalizes all prepared
+		// statements associated with this connection.
+		virtual void STORM_FN close();
 
 		// Returns all names of tables in SQLite connection in an Array of Str.
 		virtual Array<Str*> *STORM_FN tables() ABSTRACT;
@@ -144,17 +173,37 @@ namespace sql {
 		// Returns a Schema for a particular table.
 		virtual MAYBE(Schema *) STORM_FN schema(Str *table) ABSTRACT;
 
+		// Migrate data according to the supplied Migration object.
+		virtual void STORM_FN migrate(Migration *migration) ABSTRACT;
 
-		/**
-		 * Peculiarities of the database, that the database library might need to consider:
-		 */
+		// Get features of this database, that the database library or any other user might need to
+		// consider.
+		virtual features::DBFeatures STORM_FN features() const ABSTRACT;
 
-		// Does this database implicitly assign IDs to the first primary key? I.e., can we omit
-		// autoincrement in this case?
-		virtual Bool STORM_FN implicitAutoIncrement() const;
+	protected:
+		// Get the visitor for converting query strings.
+		virtual QueryStr::Visitor *STORM_FN visitor() const ABSTRACT;
 
-		// Does this database support full ALTER TABLE syntax?
-		virtual Bool STORM_FN fullAlterTable() const;
+		// Start a transaction.
+		virtual void STORM_FN beginTransaction() ABSTRACT;
+
+		// End a transaction.
+		virtual void STORM_FN endTransaction(Transaction::End end) ABSTRACT;
+
+	private:
+		friend class Statement;
+
+		// All prepared statements associated with this connection.
+		WeakSetBase *statements;
+
+		// Cached queries (based on object identity)
+		// TODO: This is a good candidate for a weak hash table where the keys are weak.
+		RefMap<CachedQuery *, Statement *> *cachedStatements;
+
+		friend class Transaction;
+
+		// Current topmost transaction, if any.
+		Transaction::State *transaction;
 	};
 
 }
