@@ -4,7 +4,7 @@
 
 using namespace code;
 
-BEGIN_TEST(OperandTest, Code) {
+BEGIN_TEST(OperandTest, CodeBasic) {
 	Operand op = ptrConst(Offset(-2));
 	CHECK_EQ(op.constant(), Word(-2));
 } END_TEST
