@@ -19,7 +19,7 @@ static void errorPtr() {
 
 struct ExThread {
 
-	os::FutureSema<Semaphore> result;
+	os::FutureBase result;
 
 	ExThread() : result() {}
 
