@@ -1,6 +1,7 @@
 #include "stdafx.h"
 #include "Fn.h"
 #include "Core/Fn.h"
+#include "Compiler/Lib/Fn.h"
 #include "Compiler/Debug.h"
 
 using namespace storm::debug;
@@ -141,6 +142,68 @@ BEGIN_TEST(StormFnPtrTest, BS) {
 
 } END_TEST
 
+BEGIN_TEST(StormFnPtrCtor, BS) {
+	Engine &e = gEngine();
+
+	// Values.
+	DbgVal::clear();
+	{
+		Type *dbgType = StormInfo<DbgVal>::type(e);
+		Function *ctor = as<Function>(dbgType->find(S("__init"), dbgType, Scope()));
+		VERIFY(ctor);
+
+		Fn<DbgVal> *fn = as<Fn<DbgVal>>(pointer(ctor));
+		VERIFY(fn);
+
+		DbgVal result = fn->call();
+		CHECK_EQ(result.v, 10);
+	}
+	CHECK(DbgVal::clear());
+
+	// Classes.
+	{
+		Type *dbgType = StormInfo<Dbg>::type(e);
+		Function *ctor = as<Function>(dbgType->find(S("__init"), dbgType, Scope()));
+		VERIFY(ctor);
+
+		Fn<Dbg *> *fn = as<Fn<Dbg *>>(pointer(ctor));
+		VERIFY(fn);
+
+		Dbg *result = fn->call();
+		CHECK_EQ(result->get(), 10);
+	}
+
+	// Actors, tied to specific threads.
+	{
+		Type *dbgType = StormInfo<DbgActor>::type(e);
+		Function *ctor = as<Function>(dbgType->find(S("__init"), dbgType, Scope()));
+		VERIFY(ctor);
+
+		Fn<DbgActor *> *fn = as<Fn<DbgActor *>>(pointer(ctor));
+		VERIFY(fn);
+
+		DbgActor *result = fn->call();
+		CHECK_EQ(result->get(), 0);
+	}
+
+	// Create functions to ctors from Storm:
+	{
+		Fn<DbgVal> *ctor = runFn<Fn<DbgVal> *>(S("tests.bs.createCtorValuePtr"));
+		CHECK_EQ(ctor->call().v, 10);
+	}
+	{
+		Fn<Dbg *, Int> *ctor = runFn<Fn<Dbg *, Int> *>(S("tests.bs.createCtorClassPtr"));
+		CHECK_EQ(ctor->call(8)->get(), 8);
+	}
+	{
+		Fn<Int> *ctor = runFn<Fn<Int> *>(S("tests.bs.createIntPtr"));
+		ctor->call();
+	}
+	// Just make sure these don't crash:
+	runFn<void>(S("tests.bs.createOtherPtr"));
+	runFn<void>(S("tests.bs.createDynActorPtr"));
+} END_TEST
+
 BEGIN_TEST(StormFnPtrCopyTest, BS) {
 	// Previously, the copy-ctor of Fn<T> was broken. Surfaced in sort() for example.
 	CHECK_EQ(runFn<Int>(S("tests.bs.copyIntPredicate")), 6);
