@@ -162,6 +162,7 @@
 (setq storm-mode-include '(
 			   ("~/Projects/storm/External/final" . "final")
 			   ("~/Projects/storm/External/thesis" . "thesis")
+			   ("~/Projects/storm/External/present" . "present")
 			   ))
 (setq storm-mode-compile-compiler t)
 
@@ -170,10 +171,15 @@
   (interactive)
   (setq mymake-compile-frame nil)
   (setq storm-mode-compile-compiler nil)
+  (setq storm-mode-show-server-buffer 'run)
   (set-face-attribute 'default nil :height 200)
   (global-storm-mode t))
 
 (global-set-key (kbd "C-c h") 'storm-doc)
+(global-set-key (kbd "C-c e") 'storm-repl-eval)
+(global-set-key (kbd "C-c r") 'storm-run-at-point)
+(global-set-key (kbd "C-c C-r") 'storm-run-all)
+(global-set-key (kbd "C-c u") 'storm-reload-current)
 
 ;;; For debugging ;;;
 
