@@ -22,7 +22,7 @@ cat - > storm-other.tar.gz
 # Make sure the repo is cloned.
 if [[ ! -e storm ]]
 then
-    git clone git@storm-lang.org:storm.git
+    git clone --recursive git://storm-lang.org/storm.git
     cd storm
 else
     cd storm
@@ -46,4 +46,4 @@ rm ~/build/storm-other.tar.gz
 cd ..
 
 # Now we can build! Do it inside a virtual X-session, so that we can run our tests.
-xvfb-run -s "-screen 0 640x480x24" ./release.sh step "$archlist" "$version" "$date" "$hash" ~/build/storm/release/release_notes.md || { echo "Build failed. Aborting."; exit 1; }
+xvfb-run -a -s "-screen 0 640x480x24" ./release.sh step "$archlist" "$version" "$date" "$hash" ~/build/storm/release/release_notes.md || { echo "Build failed. Aborting."; exit 1; }
